@@ -6,8 +6,9 @@ use Symfony\Component\DependencyInjection\ContainerBuilder;
 use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
 
 /**
- * Adds services tagged "needs_destruction" to the "kernel_destruct_subscriber"
- * service.
+ * Adds services to the "kernel.destructable_services" container parameter.
+ *
+ * Only services tagged with "needs_destruction" are added.
  *
  * @see \Drupal\Core\DestructableInterface
  */
@@ -17,15 +18,8 @@ class RegisterServicesForDestructionPass implements CompilerPassInterface {
    * {@inheritdoc}
    */
   public function process(ContainerBuilder $container) {
-    if (!$container->hasDefinition('kernel_destruct_subscriber')) {
-      return;
-    }
-
-    $definition = $container->getDefinition('kernel_destruct_subscriber');
     $services = $container->findTaggedServiceIds('needs_destruction');
-    foreach ($services as $id => $attributes) {
-      $definition->addMethodCall('registerService', [$id]);
-    }
+    $container->setParameter('kernel.destructable_services', array_keys($services));
   }
 
 }
