@@ -6,10 +6,10 @@
  */
 
 use Drupal\Core\Database\Database;
+use Drupal\Core\Link;
 use Drupal\Core\Url;
 use Drupal\Core\Utility\UpdateException;
 
-
 /**
  * @defgroup update_api Update API
  * @{
@@ -40,8 +40,8 @@ use Drupal\Core\Utility\UpdateException;
  *
  * @section sec_how How to write update code
  * Update code for a module is put into an implementation of hook_update_N(),
- * which goes into file mymodule.install (if your module's machine name is
- * mymodule). See the documentation of hook_update_N() and
+ * which goes into file my_module.install (if your module's machine name is
+ * my_module). See the documentation of hook_update_N() and
  * https://www.drupal.org/node/2535316 for details and examples.
  *
  * @section sec_test Testing update code
@@ -74,14 +74,12 @@ use Drupal\Core\Utility\UpdateException;
  *
  * See system_hook_info() for all hook groups defined by Drupal core.
  *
- * @return
+ * @return array
  *   An associative array whose keys are hook names and whose values are an
  *   associative array containing:
  *   - group: A string defining the group to which the hook belongs. The module
  *     system will determine whether a file with the name $module.$group.inc
  *     exists, and automatically load it when required.
- *
- * @see hook_hook_info_alter()
  */
 function hook_hook_info() {
   $hooks['token_info'] = [
@@ -96,7 +94,7 @@ function hook_hook_info() {
 /**
  * Alter the registry of modules implementing a hook.
  *
- * This hook is invoked during \Drupal::moduleHandler()->getImplementations().
+ * This hook is invoked in \Drupal::moduleHandler()->getImplementationInfo().
  * A module may implement this hook in order to reorder the implementing
  * modules, which are otherwise ordered by the module's system weight.
  *
@@ -119,7 +117,7 @@ function hook_hook_info() {
 function hook_module_implements_alter(&$implementations, $hook) {
   if ($hook == 'form_alter') {
     // Move my_module_form_alter() to the end of the list.
-    // \Drupal::moduleHandler()->getImplementations()
+    // \Drupal::moduleHandler()->getImplementationInfo()
     // iterates through $implementations with a foreach loop which PHP iterates
     // in the order that the items were added, so to move an item to the end of
     // the array, we remove it and then add it.
@@ -132,10 +130,9 @@ function hook_module_implements_alter(&$implementations, $hook) {
 /**
  * Alter the information parsed from module and theme .info.yml files.
  *
- * This hook is invoked in _system_rebuild_module_data() and in
- * \Drupal\Core\Extension\ThemeHandlerInterface::rebuildThemeData(). A module
- * may implement this hook in order to add to or alter the data generated by
- * reading the .info.yml file with \Drupal\Core\Extension\InfoParser.
+ * This hook is invoked in \Drupal\Core\Extension\ExtensionList::doList(). A
+ * module may implement this hook in order to add to or alter the data generated
+ * by reading the .info.yml file with \Drupal\Core\Extension\InfoParser.
  *
  * Using implementations of this hook to make modules required by setting the
  * $info['required'] key is discouraged. Doing so will slow down the module
@@ -155,7 +152,7 @@ function hook_module_implements_alter(&$implementations, $hook) {
 function hook_system_info_alter(array &$info, \Drupal\Core\Extension\Extension $file, $type) {
   // Only fill this in if the .info.yml file does not define a 'datestamp'.
   if (empty($info['datestamp'])) {
-    $info['datestamp'] = $file->getMTime();
+    $info['datestamp'] = $file->getFileInfo()->getMTime();
   }
 }
 
@@ -164,9 +161,16 @@ function hook_system_info_alter(array &$info, \Drupal\Core\Extension\Extension $
  *
  * @param string $module
  *   The name of the module about to be installed.
+ * @param bool $is_syncing
+ *   TRUE if the module is being installed as part of a configuration import. In
+ *   these cases, your hook implementation needs to carefully consider what
+ *   changes, if any, it should make. For example, it should not make any
+ *   changes to configuration objects or configuration entities. Those changes
+ *   should be made earlier and exported so during import there's no need to
+ *   do them again.
  */
-function hook_module_preinstall($module) {
-  mymodule_cache_clear();
+function hook_module_preinstall($module, bool $is_syncing) {
+  my_module_cache_clear();
 }
 
 /**
@@ -182,13 +186,23 @@ function hook_module_preinstall($module) {
  *
  * @param $modules
  *   An array of the modules that were installed.
+ * @param bool $is_syncing
+ *   TRUE if the module is being installed as part of a configuration import. In
+ *   these cases, your hook implementation needs to carefully consider what
+ *   changes, if any, it should make. For example, it should not make any
+ *   changes to configuration objects or configuration entities. Those changes
+ *   should be made earlier and exported so during import there's no need to
+ *   do them again.
  *
  * @see \Drupal\Core\Extension\ModuleInstaller::install()
  * @see hook_install()
  */
-function hook_modules_installed($modules) {
+function hook_modules_installed($modules, $is_syncing) {
   if (in_array('lousy_module', $modules)) {
-    \Drupal::state()->set('mymodule.lousy_module_compatibility', TRUE);
+    \Drupal::state()->set('my_module.lousy_module_compatibility', TRUE);
+  }
+  if (!$is_syncing) {
+    \Drupal::service('my_module.service')->doSomething($modules);
   }
 }
 
@@ -219,19 +233,25 @@ function hook_modules_installed($modules) {
  * available when this hook is called. Use cases could be displaying a user
  * message, or calling a module function necessary for initial setup, etc.
  *
- * Please be sure that anything added or modified in this function that can
- * be removed during uninstall should be removed with hook_uninstall().
+ * Ensure that anything added or modified in this function that can be removed
+ * during uninstall should be removed with hook_uninstall().
  *
+ * @param bool $is_syncing
+ *   TRUE if the module is being installed as part of a configuration import. In
+ *   these cases, your hook implementation needs to carefully consider what
+ *   changes to configuration objects or configuration entities. Those changes
+ *   should be made earlier and exported so during import there's no need to
+ *   do them again.
+ *
+ * @see \Drupal\Core\Config\ConfigInstallerInterface::isSyncing
  * @see hook_schema()
  * @see \Drupal\Core\Extension\ModuleInstaller::install()
  * @see hook_uninstall()
  * @see hook_modules_installed()
  */
-function hook_install() {
-  // Create the styles directory and ensure it's writable.
-  $directory = file_default_scheme() . '://styles';
-  $mode = isset($GLOBALS['install_state']['mode']) ? $GLOBALS['install_state']['mode'] : NULL;
-  file_prepare_directory($directory, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS, $mode);
+function hook_install($is_syncing) {
+  // Set general module variables.
+  \Drupal::state()->set('my_module.foo', 'bar');
 }
 
 /**
@@ -239,9 +259,15 @@ function hook_install() {
  *
  * @param string $module
  *   The name of the module about to be uninstalled.
+ * @param bool $is_syncing
+ *   TRUE if the module is being uninstalled as part of a configuration import.
+ *   In these cases, your hook implementation needs to carefully consider what
+ *   changes to configuration objects or configuration entities. Those changes
+ *   should be made earlier and exported so during import there's no need to
+ *   do them again.
  */
-function hook_module_preuninstall($module) {
-  mymodule_cache_clear();
+function hook_module_preuninstall($module, bool $is_syncing) {
+  my_module_cache_clear();
 }
 
 /**
@@ -256,14 +282,23 @@ function hook_module_preuninstall($module) {
  *
  * @param $modules
  *   An array of the modules that were uninstalled.
+ * @param bool $is_syncing
+ *   TRUE if the module is being uninstalled as part of a configuration import.
+ *   In these cases, your hook implementation needs to carefully consider what
+ *   changes to configuration objects or configuration entities. Those changes
+ *   should be made earlier and exported so during import there's no need to
+ *   do them again.
  *
  * @see hook_uninstall()
  */
-function hook_modules_uninstalled($modules) {
+function hook_modules_uninstalled($modules, $is_syncing) {
   if (in_array('lousy_module', $modules)) {
-    \Drupal::state()->delete('mymodule.lousy_module_compatibility');
+    \Drupal::state()->delete('my_module.lousy_module_compatibility');
+  }
+  my_module_cache_rebuild();
+  if (!$is_syncing) {
+    \Drupal::service('my_module.service')->doSomething($modules);
   }
-  mymodule_cache_rebuild();
 }
 
 /**
@@ -281,13 +316,26 @@ function hook_modules_uninstalled($modules) {
  * tables are removed, allowing your module to query its own tables during
  * this routine.
  *
+ * Adding custom logic to hook_uninstall implementations to check for
+ * criteria before uninstalling, does not take advantage of the module
+ * uninstall page UI. Instead, use
+ * \Drupal\Core\Extension\ModuleUninstallValidatorInterface.
+ *
+ * @param bool $is_syncing
+ *   TRUE if the module is being uninstalled as part of a configuration import.
+ *   In these cases, your hook implementation needs to carefully consider what
+ *   changes to configuration objects or configuration entities. Those changes
+ *   should be made earlier and exported so during import there's no need to
+ *   do them again.
+ *
  * @see hook_install()
  * @see hook_schema()
  * @see hook_modules_uninstalled()
+ * @see \Drupal\Core\Extension\ModuleUninstallValidatorInterface
  */
-function hook_uninstall() {
-  // Remove the styles directory and generated images.
-  file_unmanaged_delete_recursive(file_default_scheme() . '://styles');
+function hook_uninstall($is_syncing) {
+  // Delete remaining general module variables.
+  \Drupal::state()->delete('my_module.foo');
 }
 
 /**
@@ -397,51 +445,51 @@ function hook_install_tasks(&$install_state) {
   // Here, we define a variable to allow tasks to indicate that a particular,
   // processor-intensive batch process needs to be triggered later on in the
   // installation.
-  $myprofile_needs_batch_processing = \Drupal::state()->get('myprofile.needs_batch_processing', FALSE);
+  $my_profile_needs_batch_processing = \Drupal::state()->get('my_profile.needs_batch_processing', FALSE);
   $tasks = [
     // This is an example of a task that defines a form which the user who is
     // installing the site will be asked to fill out. To implement this task,
-    // your profile would define a function named myprofile_data_import_form()
+    // your profile would define a function named my_profile_data_import_form()
     // as a normal form API callback function, with associated validation and
     // submit handlers. In the submit handler, in addition to saving whatever
     // other data you have collected from the user, you might also call
-    // \Drupal::state()->set('myprofile.needs_batch_processing', TRUE) if the
+    // \Drupal::state()->set('my_profile.needs_batch_processing', TRUE) if the
     // user has entered data which requires that batch processing will need to
     // occur later on.
-    'myprofile_data_import_form' => [
+    'my_profile_data_import_form' => [
       'display_name' => t('Data import options'),
       'type' => 'form',
     ],
     // Similarly, to implement this task, your profile would define a function
-    // named myprofile_settings_form() with associated validation and submit
+    // named my_profile_settings_form() with associated validation and submit
     // handlers. This form might be used to collect and save additional
     // information from the user that your profile needs. There are no extra
     // steps required for your profile to act as an "installation wizard"; you
     // can simply define as many tasks of type 'form' as you wish to execute,
     // and the forms will be presented to the user, one after another.
-    'myprofile_settings_form' => [
+    'my_profile_settings_form' => [
       'display_name' => t('Additional options'),
       'type' => 'form',
     ],
     // This is an example of a task that performs batch operations. To
     // implement this task, your profile would define a function named
-    // myprofile_batch_processing() which returns a batch API array definition
+    // my_profile_batch_processing() which returns a batch API array definition
     // that the installer will use to execute your batch operations. Due to the
-    // 'myprofile.needs_batch_processing' variable used here, this task will be
+    // 'my_profile.needs_batch_processing' variable used here, this task will be
     // hidden and skipped unless your profile set it to TRUE in one of the
     // previous tasks.
-    'myprofile_batch_processing' => [
+    'my_profile_batch_processing' => [
       'display_name' => t('Import additional data'),
-      'display' => $myprofile_needs_batch_processing,
+      'display' => $my_profile_needs_batch_processing,
       'type' => 'batch',
-      'run' => $myprofile_needs_batch_processing ? INSTALL_TASK_RUN_IF_NOT_COMPLETED : INSTALL_TASK_SKIP,
+      'run' => $my_profile_needs_batch_processing ? INSTALL_TASK_RUN_IF_NOT_COMPLETED : INSTALL_TASK_SKIP,
     ],
     // This is an example of a task that will not be displayed in the list that
     // the user sees. To implement this task, your profile would define a
-    // function named myprofile_final_site_setup(), in which additional,
+    // function named my_profile_final_site_setup(), in which additional,
     // automated site setup operations would be performed. Since this is the
     // last task defined by your profile, you should also use this function to
-    // call \Drupal::state()->delete('myprofile.needs_batch_processing') and
+    // call \Drupal::state()->delete('my_profile.needs_batch_processing') and
     // clean up the state that was used above. If you want the user to pass
     // to the final Drupal installation tasks uninterrupted, return no output
     // from this function. Otherwise, return themed output that the user will
@@ -449,7 +497,7 @@ function hook_install_tasks(&$install_state) {
     // tasks are complete, with a link to reload the current page and therefore
     // pass on to the final Drupal installation tasks when the user is ready to
     // do so).
-    'myprofile_final_site_setup' => [],
+    'my_profile_final_site_setup' => [],
   ];
   return $tasks;
 }
@@ -475,36 +523,66 @@ function hook_install_tasks(&$install_state) {
 function hook_install_tasks_alter(&$tasks, $install_state) {
   // Replace the entire site configuration form provided by Drupal core
   // with a custom callback function defined by this installation profile.
-  $tasks['install_configure_form']['function'] = 'myprofile_install_configure_form';
+  $tasks['install_configure_form']['function'] = 'my_profile_install_configure_form';
 }
 
 /**
  * Perform a single update between minor versions.
  *
- * hook_update_N() can only be used to update between minor versions of a
- * module. To upgrade between major versions of Drupal (for example, between
- * Drupal 7 and 8), use the @link migrate Migrate API @endlink instead.
+ * Modules should use hook hook_update_N() to update between minor or major
+ * versions of the module. Sites upgrading from Drupal 6 or 7 to any higher
+ * version should use the @link migrate Migrate API @endlink instead.
  *
  * @section sec_naming Naming and documenting your function
  * For each change in a module that requires one or more actions to be performed
  * when updating a site, add a new implementation of hook_update_N() to your
- * mymodule.install file (assuming mymodule is the machine name of your module).
- * Implementations of hook_update_N() are named (module name)_update_(number).
+ * my_module.install file (assuming my_module is the machine name of your
+ * module). Implementations of hook_update_N() are named (module
+ * name)_update_(number).
+ *
+ * The number (N) must be higher than hook_update_last_removed().
+ *
  * The numbers are normally composed of three parts:
  * - 1 or 2 digits for Drupal core compatibility (Drupal 8, 9, 10, etc.). This
- *   convention must be followed.
- * - 1 digit for your module's major release version; for example, for 8.x-1.*
- *   use 1, for 8.x-2.* use 2, for Core 8.0.x use 0, and for Core 8.1.x use 1.
- *   This convention is optional but suggested for clarity.
+ *   convention must be followed. If your module is compatible with multiple
+ *   major versions (e.g., it has a core_version_requirement of '^8.8 || ^9'),
+ *   use the lowest major core branch it is compatible with (8 in this example).
+ * - 1 or 2 digits for your module's major release version. Examples:
+ *   - For 8.x-1.* or 1.y.x (semantic versioning), use 1 or 01.
+ *   - For 8.x-2.* or 2.y.x, use 2 or 02.
+ *   - For 8.x-10.* or 10.y.x, use 10.
+ *   - For core 8.0.x, use 0 or 00.
+ *   - For core 8.1.x, use 1 or 01.
+ *   - For core 8.10.x, use 10.
+ *   This convention is optional but suggested for clarity. Note: While four-
+ *   digit update hooks are standard, if you follow the above convention, you
+ *   may run into the need to start using five digits if you get to a major (or,
+ *   for core, a minor) version 10. This will work fine; what matters is that
+ *   you must ALWAYS increase the number when a new update hook is added. For
+ *   example, if you add hook_update_81001(), you cannot later
+ *   add hook_update_9101(). Instead, you would need to use hook_update_90101().
  * - 2 digits for sequential counting, starting with 01. Note that the x000
  *   number can never be used: the lowest update number that will be recognized
- *   and run for major version x is x001.
+ *   and run is 8001.
+ *
+ * @todo Remove reference to CORE_MINIMUM_SCHEMA_VERSION (e.g., x001) once
+ *   https://www.drupal.org/project/drupal/issues/3106712 is resolved.
+ *
  * Examples:
  * - node_update_8001(): The first update for the Drupal 8.0.x version of the
  *   Drupal Core node module.
- * - mymodule_update_8101(): The first update for your custom or contributed
+ * - node_update_81001(): The first update for the Drupal 8.10.x version of the
+ *   Drupal Core node module.
+ * - my_module_update_8101(): The first update for your custom or contributed
  *   module's 8.x-1.x versions.
- * - mymodule_update_8201(): The first update for the 8.x-2.x versions.
+ * - my_module_update_8201(): The first update for the 8.x-2.x versions.
+ * - my_module_update_80201(): Alternate five-digit format for the first update
+ *   for the 8.x-2.x versions. Subsequent update hooks must always be five
+ *   digits and higher than 80201.
+ * - my_module_update_8201(): The first update for the custom or contributed
+ *   module's 2.0.x versions, which are compatible with Drupal 8 and 9.
+ * - my_module_update_91001(): The first update for the custom or contributed
+ *   module's 10.0.x versions, which are compatible with Drupal 9.
  *
  * Never renumber update functions. The numeric part of the hook implementation
  * function is stored in the database to keep track of which updates have run,
@@ -550,9 +628,6 @@ function hook_install_tasks_alter(&$tasks, $install_state) {
  *   \Drupal::entityDefinitionUpdateManager()::getFieldStorageDefinition(). When
  *   adding a new definition always replicate it in the update function body as
  *   you would do with a schema definition.
- * - Never call \Drupal::entityDefinitionUpdateManager()::applyUpdates() in an
- *   update function, as it will apply updates for any module not only yours,
- *   which will lead to unpredictable results.
  * - Be careful about API functions and especially CRUD operations that you use
  *   in your update function. If they invoke hooks or use services, they may
  *   not behave as expected, and it may actually not be appropriate to use the
@@ -590,6 +665,82 @@ function hook_install_tasks_alter(&$tasks, $install_state) {
  * See the @link batch Batch operations topic @endlink for more information on
  * how to use the Batch API.
  *
+ * @section sec_equivalent_updates Multiple upgrade paths
+ * There are situations where changes require a hook_update_N() to be applied to
+ * different major branches. This results in more than one upgrade path from the
+ * current major version to the next major version.
+ *
+ * For example, if an update is added to 11.1.0 and 10.4.0, then a site on
+ * 10.3.7 can update either to 10.4.0 and from there to 11.1.0, or directly from
+ * 10.3.7 to 11.1.1. In one case, the update will run on the 10.x code base, and
+ * in another on the 11.x code base, but the update system needs to ensure that
+ * it doesn't run twice on the same site.
+ *
+ * hook_update_N() numbers are sequential integers, and numbers lower than the
+ * modules current schema version will never be run. This means once a site has
+ * run an update, for example, 11100, it will not run a later update added as
+ * 10400. Backporting of updates therefore needs to allow 'space' for the 10.4.x
+ * codebase to include updates which don't exist in 11.x (for example to ensure
+ * a later 11.x update goes smoothly).
+ *
+ * To resolve this, when handling potential backports of updates between major
+ * branches, we use different update numbers for each branch, but record the
+ * relationship between those updates in the older branches. This is best
+ * explained by an example showing the different branches updates could be
+ * applied to:
+ * - The first update, system_update_10300 is applied to 10.3.0.
+ * - Then, 11.0.0 is released with the update removed,
+ *   system_update_last_removed() is added which returns 10300.
+ * - The next update, system_update_11100, is applied to 11.1.x only.
+ * - Then 10.4.0 and 11.1.0 are released. system_update_11100 is not backported
+ *   to 11.0.x or any 10.x branch.
+ * - Finally, a critical data loss update is necessary. The bug-fix supported
+ *   branches are 11.1.x, 11.0.x, and 10.4.x. This results in adding the updates
+ *   system_update_10400 (10.4.x), system_update_11000 (11.0.x) and
+ *   system_update_11101 (11.1.x) and making the 10.4.1, 11.0.1 and 11.1.1
+ *   releases.
+ *
+ * This is a list of the example releases and the updates they contain:
+ * - 10.3.0: system_update_10300
+ * - 10.4.1: system_update_10300 and system_update_10400 (equivalent to
+ *   system_update_11101)
+ * - 11.0.0: No updates
+ * - 11.0.1: system_update_11000 (equivalent to system_update_11101)
+ * - 11.1.0: system_update_11100
+ * - 11.1.1: system_update_11100 and system_update_11101
+ *
+ * In this situation, sites on 10.4.1 or 11.0.1 will be required to update to
+ * versions that contain system_update_11101. For example, a site on 10.4.1
+ * would not be able to update to 11.0.0, because that would result in it going
+ * 'backwards' in terms of database schema, but it would be able to update to
+ * 11.1.1. The same is true for a site on 11.0.1.
+ *
+ * The following examples show how to implement a hook_update_N() that must be
+ * skipped in a future update process.
+ *
+ * Future updates can be marked as equivalent by adding the following code to an
+ * update.
+ * @code
+ * function my_module_update_10400() {
+ *   \Drupal::service('update.update_hook_registry')->markFutureUpdateEquivalent(11101, '11.1.1');
+ *
+ *   // The rest of the update function.
+ * }
+ * @endcode
+ *
+ * At the moment we need to add defensive coding in the future update to ensure
+ * it is skipped.
+ * @code
+ * function my_module_update_11101() {
+ *   $equivalent_update = \Drupal::service('update.update_hook_registry')->getEquivalentUpdate();
+ *   if ($equivalent_update instanceof \Drupal\Core\Update\EquivalentUpdate) {
+ *     return $equivalent_update->toSkipMessage();
+ *   }
+ *
+ *   // The rest of the update function.
+ * }
+ * @endcode
+ *
  * @param array $sandbox
  *   Stores information for batch updates. See above for more information.
  *
@@ -611,8 +762,6 @@ function hook_install_tasks_alter(&$tasks, $install_state) {
  * @see hook_update_last_removed()
  * @see update_get_update_list()
  * @see \Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface
- * @see node_update_8001
- * @see system_update_8004
  * @see https://www.drupal.org/node/2535316
  */
 function hook_update_N(&$sandbox) {
@@ -628,7 +777,7 @@ function hook_update_N(&$sandbox) {
     'not null' => FALSE,
   ];
   $schema = Database::getConnection()->schema();
-  $schema->addField('mytable1', 'newcol', $spec);
+  $schema->addField('my_table', 'newcol', $spec);
 
   // Example of what to do if there is an error during your update.
   if ($some_error_condition_met) {
@@ -641,26 +790,26 @@ function hook_update_N(&$sandbox) {
     // This must be the first run. Initialize the sandbox.
     $sandbox['progress'] = 0;
     $sandbox['current_pk'] = 0;
-    $sandbox['max'] = Database::getConnection()->query('SELECT COUNT(myprimarykey) FROM {mytable1}')->fetchField() - 1;
+    $sandbox['max'] = Database::getConnection()->query('SELECT COUNT([my_primary_key]) FROM {my_table}')->fetchField();
   }
 
   // Update in chunks of 20.
-  $records = Database::getConnection()->select('mytable1', 'm')
-    ->fields('m', ['myprimarykey', 'otherfield'])
-    ->condition('myprimarykey', $sandbox['current_pk'], '>')
+  $records = Database::getConnection()->select('my_table', 'm')
+    ->fields('m', ['my_primary_key', 'other_field'])
+    ->condition('my_primary_key', $sandbox['current_pk'], '>')
     ->range(0, 20)
-    ->orderBy('myprimarykey', 'ASC')
+    ->orderBy('my_primary_key', 'ASC')
     ->execute();
   foreach ($records as $record) {
     // Here, you would make an update something related to this record. In this
     // example, some text is added to the other field.
-    Database::getConnection()->update('mytable1')
-      ->fields(['otherfield' => $record->otherfield . '-suffix'])
-      ->condition('myprimarykey', $record->myprimarykey)
+    Database::getConnection()->update('my_table')
+      ->fields(['other_field' => $record->other_field . '-suffix'])
+      ->condition('my_primary_key', $record->my_primary_key)
       ->execute();
 
     $sandbox['progress']++;
-    $sandbox['current_pk'] = $record->myprimarykey;
+    $sandbox['current_pk'] = $record->my_primary_key;
   }
 
   $sandbox['#finished'] = empty($sandbox['max']) ? 1 : ($sandbox['progress'] / $sandbox['max']);
@@ -673,18 +822,34 @@ function hook_update_N(&$sandbox) {
 /**
  * Executes an update which is intended to update data, like entities.
  *
- * These implementations have to be placed in a MODULE.post_update.php file.
+ * These implementations have to be placed in a MODULE.post_update.php file or
+ * a THEME.post_update.php file.
  *
  * These updates are executed after all hook_update_N() implementations. At this
  * stage Drupal is already fully repaired so you can use any API as you wish.
  *
- * NAME can be arbitrary machine names. In contrast to hook_update_N() the order
- * of functions in the file is the only thing which ensures the execution order
- * of those functions.
+ * NAME can be arbitrary machine names. In contrast to hook_update_N() the
+ * alphanumeric naming of functions in the file is the only thing which ensures
+ * the execution order of those functions. If update order is mandatory,
+ * you should add numerical prefix to NAME or make it completely numerical.
  *
  * Drupal also ensures to not execute the same hook_post_update_NAME() function
  * twice.
  *
+ * @section sec_bulk Batch updates
+ * If running your update all at once could possibly cause PHP to time out, use
+ * the $sandbox parameter to indicate that the Batch API should be used for your
+ * update. In this case, your update function acts as an implementation of
+ * callback_batch_operation(), and $sandbox acts as the batch context
+ * parameter. In your function, read the state information from the previous
+ * run from $sandbox (or initialize), run a chunk of updates, save the state in
+ * $sandbox, and set $sandbox['#finished'] to a value between 0 and 1 to
+ * indicate the percent completed, or 1 if it is finished (you need to do this
+ * explicitly in each pass).
+ *
+ * See the @link batch Batch operations topic @endlink for more information on
+ * how to use the Batch API.
+ *
  * @param array $sandbox
  *   Stores information for batch updates. See above for more information.
  *
@@ -702,6 +867,7 @@ function hook_update_N(&$sandbox) {
  * @ingroup update_api
  *
  * @see hook_update_N()
+ * @see hook_removed_post_updates()
  */
 function hook_post_update_NAME(&$sandbox) {
   // Example of updating some content.
@@ -711,30 +877,39 @@ function hook_post_update_NAME(&$sandbox) {
 
   $result = t('Node %nid saved', ['%nid' => $node->id()]);
 
-  // Example of disabling blocks with missing condition contexts. Note: The
-  // block itself is in a state which is valid at that point.
-  // @see block_update_8001()
-  // @see block_post_update_disable_blocks_with_missing_contexts()
-  $block_update_8001 = \Drupal::keyValue('update_backup')->get('block_update_8001', []);
-
-  $block_ids = array_keys($block_update_8001);
-  $block_storage = \Drupal::entityManager()->getStorage('block');
-  $blocks = $block_storage->loadMultiple($block_ids);
-  /** @var $blocks \Drupal\block\BlockInterface[] */
-  foreach ($blocks as $block) {
-    // This block has had conditions removed due to an inability to resolve
-    // contexts in block_update_8001() so disable it.
-
-    // Disable currently enabled blocks.
-    if ($block_update_8001[$block->id()]['status']) {
-      $block->setStatus(FALSE);
-      $block->save();
-    }
+  // Example of updating some config.
+  if (\Drupal::moduleHandler()->moduleExists('taxonomy')) {
+    // Update the dependencies of all Vocabulary configuration entities.
+    \Drupal::classResolver(\Drupal\Core\Config\Entity\ConfigEntityUpdater::class)->update($sandbox, 'taxonomy_vocabulary');
   }
 
   return $result;
 }
 
+/**
+ * Return an array of removed hook_post_update_NAME() function names.
+ *
+ * This should be used to indicate post-update functions that have existed in
+ * some previous version of the module, but are no longer available.
+ *
+ * This implementation has to be placed in a MODULE.post_update.php file.
+ *
+ * @return string[]
+ *   An array where the keys are removed post-update function names, and the
+ *   values are the first stable version in which the update was removed.
+ *
+ * @ingroup update_api
+ *
+ * @see hook_post_update_NAME()
+ */
+function hook_removed_post_updates() {
+  return [
+    'my_module_post_update_foo' => '8.x-2.0',
+    'my_module_post_update_bar' => '8.x-3.0',
+    'my_module_post_update_baz' => '8.x-3.0',
+  ];
+}
+
 /**
  * Return an array of information about module update dependencies.
  *
@@ -743,10 +918,10 @@ function hook_post_update_NAME(&$sandbox) {
  * system to determine the appropriate order in which updates should be run, as
  * well as to search for missing dependencies.
  *
- * Implementations of this hook should be placed in a mymodule.install file in
- * the same directory as mymodule.module.
+ * Implementations of this hook should be placed in a my_module.install file in
+ * the same directory as my_module.module.
  *
- * @return
+ * @return array
  *   A multidimensional array containing information about the module update
  *   dependencies. The first two levels of keys represent the module and update
  *   number (respectively) for which information is being returned, and the
@@ -763,21 +938,21 @@ function hook_post_update_NAME(&$sandbox) {
  * @see hook_update_N()
  */
 function hook_update_dependencies() {
-  // Indicate that the mymodule_update_8001() function provided by this module
+  // Indicate that the my_module_update_8001() function provided by this module
   // must run after the another_module_update_8003() function provided by the
   // 'another_module' module.
-  $dependencies['mymodule'][8001] = [
+  $dependencies['my_module'][8001] = [
     'another_module' => 8003,
   ];
-  // Indicate that the mymodule_update_8002() function provided by this module
+  // Indicate that the my_module_update_8002() function provided by this module
   // must run before the yet_another_module_update_8005() function provided by
   // the 'yet_another_module' module. (Note that declaring dependencies in this
   // direction should be done only in rare situations, since it can lead to the
   // following problem: If a site has already run the yet_another_module
   // module's database updates before it updates its codebase to pick up the
-  // newest mymodule code, then the dependency declared here will be ignored.)
+  // newest my_module code, then the dependency declared here will be ignored.)
   $dependencies['yet_another_module'][8005] = [
-    'mymodule' => 8002,
+    'my_module' => 8002,
   ];
   return $dependencies;
 }
@@ -785,24 +960,24 @@ function hook_update_dependencies() {
 /**
  * Return a number which is no longer available as hook_update_N().
  *
- * If you remove some update functions from your mymodule.install file, you
+ * If you remove some update functions from your my_module.install file, you
  * should notify Drupal of those missing functions. This way, Drupal can
  * ensure that no update is accidentally skipped.
  *
- * Implementations of this hook should be placed in a mymodule.install file in
- * the same directory as mymodule.module.
+ * Implementations of this hook should be placed in a my_module.install file in
+ * the same directory as my_module.module.
  *
- * @return
+ * @return int
  *   An integer, corresponding to hook_update_N() which has been removed from
- *   mymodule.install.
+ *   my_module.install.
  *
  * @ingroup update_api
  *
  * @see hook_update_N()
  */
 function hook_update_last_removed() {
-  // We've removed the 8.x-1.x version of mymodule, including database updates.
-  // The next update function is mymodule_update_8200().
+  // We've removed the 8.x-1.x version of my_module, including database updates.
+  // The next update function is my_module_update_8200().
   return 8103;
 }
 
@@ -813,7 +988,7 @@ function hook_update_last_removed() {
  * of the Drupal file system, for example to update modules that have newer
  * releases, or to install a new theme.
  *
- * @return
+ * @return array
  *   An associative array of information about the updater(s) being provided.
  *   This array is keyed by a unique identifier for each updater, and the
  *   values are subarrays that can contain the following keys:
@@ -911,7 +1086,7 @@ function hook_updater_info_alter(&$updaters) {
  *   - runtime: The runtime requirements are being checked and shown on the
  *     status report page.
  *
- * @return
+ * @return array
  *   An associative array where the keys are arbitrary but must be unique (it
  *   is suggested to use the module short name as a prefix) and the values are
  *   themselves associative arrays with the following elements:
@@ -934,17 +1109,17 @@ function hook_requirements($phase) {
     $requirements['drupal'] = [
       'title' => t('Drupal'),
       'value' => \Drupal::VERSION,
-      'severity' => REQUIREMENT_INFO
+      'severity' => REQUIREMENT_INFO,
     ];
   }
 
   // Test PHP version
   $requirements['php'] = [
     'title' => t('PHP'),
-    'value' => ($phase == 'runtime') ? \Drupal::l(phpversion(), new Url('system.php')) : phpversion(),
+    'value' => ($phase == 'runtime') ? Link::fromTextAndUrl(phpversion(), Url::fromRoute('system.php'))->toString() : phpversion(),
   ];
-  if (version_compare(phpversion(), DRUPAL_MINIMUM_PHP) < 0) {
-    $requirements['php']['description'] = t('Your PHP installation is too old. Drupal requires at least PHP %version.', ['%version' => DRUPAL_MINIMUM_PHP]);
+  if (version_compare(phpversion(), \Drupal::MINIMUM_PHP) < 0) {
+    $requirements['php']['description'] = t('Your PHP installation is too old. Drupal requires at least PHP %version.', ['%version' => \Drupal::MINIMUM_PHP]);
     $requirements['php']['severity'] = REQUIREMENT_ERROR;
   }
 
@@ -957,13 +1132,13 @@ function hook_requirements($phase) {
     }
     else {
       $requirements['cron'] = [
-        'description' => t('Cron has not run. It appears cron jobs have not been setup on your system. Check the help pages for <a href=":url">configuring cron jobs</a>.', [':url' => 'https://www.drupal.org/cron']),
+        'description' => t('Cron has not run. It appears cron jobs have not been setup on your system. Check the help pages for <a href=":url">configuring cron jobs</a>.', [':url' => 'https://www.drupal.org/docs/administering-a-drupal-site/cron-automated-tasks/cron-automated-tasks-overview']),
         'severity' => REQUIREMENT_ERROR,
         'value' => t('Never run'),
       ];
     }
 
-    $requirements['cron']['description'] .= ' ' . t('You can <a href=":cron">run cron manually</a>.', [':cron' => \Drupal::url('system.run_cron')]);
+    $requirements['cron']['description'] .= ' ' . t('You can <a href=":cron">run cron manually</a>.', [':cron' => Url::fromRoute('system.run_cron')->toString()]);
 
     $requirements['cron']['title'] = t('Cron maintenance tasks');
   }
@@ -971,6 +1146,29 @@ function hook_requirements($phase) {
   return $requirements;
 }
 
+/**
+ * Alters requirements data.
+ *
+ * Implementations are able to alter the title, value, description or the
+ * severity of certain requirements defined by hook_requirements()
+ * implementations or even remove such entries.
+ *
+ * @param array $requirements
+ *   The requirements data to be altered.
+ *
+ * @see hook_requirements()
+ */
+function hook_requirements_alter(array &$requirements): void {
+  // Change the title from 'PHP' to 'PHP version'.
+  $requirements['php']['title'] = t('PHP version');
+
+  // Decrease the 'update status' requirement severity from warning to info.
+  $requirements['update status']['severity'] = REQUIREMENT_INFO;
+
+  // Remove a requirements entry.
+  unset($requirements['foo']);
+}
+
 /**
  * @} End of "addtogroup hooks".
  */
