@@ -2,21 +2,21 @@
 
 namespace Drupal\image_module_test\Plugin\Field\FieldFormatter;
 
+use Drupal\Core\Field\Attribute\FieldFormatter;
 use Drupal\Core\Field\FieldItemListInterface;
 use Drupal\Core\Field\FormatterBase;
+use Drupal\Core\StringTranslation\TranslatableMarkup;
 
 /**
  * Empty renderer for a dummy field with an AJAX handler.
- *
- * @FieldFormatter(
- *   id = "image_module_test_dummy_ajax_formatter",
- *   module = "image_module_test",
- *   label = @Translation("Dummy AJAX"),
- *   field_types= {
- *     "image_module_test_dummy_ajax"
- *   }
- * )
  */
+#[FieldFormatter(
+  id: 'image_module_test_dummy_ajax_formatter',
+  label: new TranslatableMarkup('Dummy AJAX'),
+  field_types: [
+    'image_module_test_dummy_ajax',
+  ],
+)]
 class DummyAjaxFormatter extends FormatterBase {
 
   /**
@@ -24,7 +24,7 @@ class DummyAjaxFormatter extends FormatterBase {
    */
   public function settingsSummary() {
     $summary = [];
-    $summary[] = t('Renders nothing');
+    $summary[] = $this->t('Renders nothing');
     return $summary;
   }
 
