@@ -1,20 +1,22 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\image\Functional;
 
-use Drupal\Core\Url;
-use Drupal\Component\Render\FormattableMarkup;
 use Drupal\Core\Entity\Entity\EntityViewDisplay;
+use Drupal\Core\Url;
+use Drupal\file\Entity\File;
 use Drupal\image\Entity\ImageStyle;
 use Drupal\image\ImageStyleInterface;
 use Drupal\node\Entity\Node;
-use Drupal\file\Entity\File;
 use Drupal\Tests\TestFileCreationTrait;
 
 /**
  * Tests creation, deletion, and editing of image styles and effects.
  *
  * @group image
+ * @group #slow
  */
 class ImageAdminStylesTest extends ImageFieldTestBase {
 
@@ -57,18 +59,18 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
   }
 
   /**
-   * Test creating an image style with a numeric name and ensuring it can be
-   * applied to an image.
+   * Tests creating an image style with a numeric name.
    */
-  public function testNumericStyleName() {
+  public function testNumericStyleName(): void {
     $style_name = rand();
     $style_label = $this->randomString();
     $edit = [
       'name' => $style_name,
       'label' => $style_label,
     ];
-    $this->drupalPostForm('admin/config/media/image-styles/add', $edit, 'Create new style');
-    $this->assertRaw(t('Style %name was created.', ['%name' => $style_label]));
+    $this->drupalGet('admin/config/media/image-styles/add');
+    $this->submitForm($edit, 'Create new style');
+    $this->assertSession()->statusMessageContains("Style {$style_label} was created.", 'status');
     $options = image_style_options();
     $this->assertArrayHasKey($style_name, $options);
   }
@@ -76,11 +78,11 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
   /**
    * General test to add a style, add/remove/edit effects to it, then delete it.
    */
-  public function testStyle() {
+  public function testStyle(): void {
     $admin_path = 'admin/config/media/image-styles';
 
     // Setup a style to be created and effects to add to it.
-    $style_name = strtolower($this->randomMachineName(10));
+    $style_name = $this->randomMachineName(10);
     $style_label = $this->randomString();
     $style_path = $admin_path . '/manage/' . $style_name;
     $effect_edits = [
@@ -118,8 +120,9 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
       'name' => $style_name,
       'label' => $style_label,
     ];
-    $this->drupalPostForm($admin_path . '/add', $edit, 'Create new style');
-    $this->assertRaw(t('Style %name was created.', ['%name' => $style_label]));
+    $this->drupalGet($admin_path . '/add');
+    $this->submitForm($edit, 'Create new style');
+    $this->assertSession()->statusMessageContains("Style {$style_label} was created.", 'status');
 
     // Ensure that the expected entity operations are there.
     $this->drupalGet($admin_path);
@@ -136,8 +139,11 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
         $edit_data['data[' . $field . ']'] = $value;
       }
       // Add the effect.
-      $this->drupalPostForm($style_path, ['new' => $effect], 'Add');
+      $this->drupalGet($style_path);
+      $this->submitForm(['new' => $effect], 'Add');
       if (!empty($edit)) {
+        $effect_label = \Drupal::service('plugin.manager.image.effect')->createInstance($effect)->label();
+        $this->assertSession()->pageTextContains("Add {$effect_label} effect to style {$style_label}");
         $this->submitForm($edit_data, 'Add effect');
       }
     }
@@ -148,7 +154,7 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
     // Ensure that third party settings were added to the config entity.
     // These are added by a hook_image_style_presave() implemented in
     // image_module_test module.
-    $this->assertEqual('bar', $style->getThirdPartySetting('image_module_test', 'foo'), 'Third party settings were added to the image style.');
+    $this->assertEquals('bar', $style->getThirdPartySetting('image_module_test', 'foo'), 'Third party settings were added to the image style.');
 
     // Ensure that the image style URI matches our expected path.
     $style_uri_path = $style->toUrl()->toString();
@@ -162,18 +168,13 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
       $uuids[$effect->getPluginId()] = $uuid;
       $effect_configuration = $effect->getConfiguration();
       foreach ($effect_edits[$effect->getPluginId()] as $field => $value) {
-        $this->assertEqual($effect_configuration['data'][$field], $value, new FormattableMarkup('The %field field in the %effect effect has the correct value of %value.', ['%field' => $field, '%effect' => $effect->getPluginId(), '%value' => $value]));
+        $this->assertEquals($effect_configuration['data'][$field], $value, "The $field field in the {$effect->getPluginId()} effect has the correct value of $value.");
       }
     }
 
     // Assert that every effect was saved.
     foreach (array_keys($effect_edits) as $effect_name) {
-      $this->assertTrue(isset($uuids[$effect_name]), new FormattableMarkup(
-        'A %effect_name effect was saved with ID %uuid',
-        [
-          '%effect_name' => $effect_name,
-          '%uuid' => $uuids[$effect_name],
-        ]));
+      $this->assertTrue(isset($uuids[$effect_name]), "A $effect_name effect was saved with ID $uuids[$effect_name]");
     }
 
     // Image style overview form (ordering and renaming).
@@ -193,7 +194,7 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
 
     // Test the style overview form.
     // Change the name of the style and adjust the weights of effects.
-    $style_name = strtolower($this->randomMachineName(10));
+    $style_name = $this->randomMachineName(10);
     $style_label = $this->randomMachineName();
     $weight = count($effect_edits);
     $edit = [
@@ -207,9 +208,10 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
 
     // Create an image to make sure it gets flushed after saving.
     $image_path = $this->createSampleImage($style);
-    $this->assertEqual(1, $this->getImageCount($style), new FormattableMarkup('Image style %style image %file successfully generated.', ['%style' => $style->label(), '%file' => $image_path]));
+    $this->assertEquals(1, $this->getImageCount($style), "Image style {$style->label()} image $image_path successfully generated.");
 
-    $this->drupalPostForm($style_path, $edit, 'Save');
+    $this->drupalGet($style_path);
+    $this->submitForm($edit, 'Save');
 
     // Note that after changing the style name, the style path is changed.
     $style_path = 'admin/config/media/image-styles/manage/' . $style_name;
@@ -219,13 +221,13 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
     $this->assertSession()->titleEquals("Edit style $style_label | Drupal");
 
     // Check that the available image effects are properly sorted.
-    $option = $this->xpath('//select[@id=:id]//option', [':id' => 'edit-new--2']);
+    $option = $this->assertSession()->selectExists('edit-new--2')->findAll('css', 'option');
     $this->assertEquals('Ajax test', $option[1]->getText(), '"Ajax test" is the first selectable effect.');
 
     // Check that the image was flushed after updating the style.
     // This is especially important when renaming the style. Make sure that
     // the old image directory has been deleted.
-    $this->assertEqual(0, $this->getImageCount($style), new FormattableMarkup('Image style %style was flushed after renaming the style and updating the order of effects.', ['%style' => $style->label()]));
+    $this->assertEquals(0, $this->getImageCount($style), "Image style {$style->label()} was flushed after renaming the style and updating the order of effects.");
 
     // Load the style by the new name with the new weights.
     $style = ImageStyle::load($style_name);
@@ -246,26 +248,22 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
 
     // Create an image to make sure it gets flushed after deleting an effect.
     $image_path = $this->createSampleImage($style);
-    $this->assertEqual(1, $this->getImageCount($style), new FormattableMarkup('Image style %style image %file successfully generated.', ['%style' => $style->label(), '%file' => $image_path]));
+    $this->assertEquals(1, $this->getImageCount($style), "Image style {$style->label()} image $image_path successfully generated.");
 
     // Delete the 'image_crop' effect from the style.
-    $this->drupalPostForm($style_path . '/effects/' . $uuids['image_crop'] . '/delete', [], 'Delete');
+    $this->drupalGet($style_path . '/effects/' . $uuids['image_crop'] . '/delete');
+    $this->submitForm([], 'Delete');
     // Confirm that the form submission was successful.
     $this->assertSession()->statusCodeEquals(200);
     $image_crop_effect = $style->getEffect($uuids['image_crop']);
-    $this->assertRaw(t('The image effect %name has been deleted.', ['%name' => $image_crop_effect->label()]));
+    $this->assertSession()->statusMessageContains("The image effect {$image_crop_effect->label()} has been deleted.", 'status');
     // Confirm that there is no longer a link to the effect.
     $this->assertSession()->linkByHrefNotExists($style_path . '/effects/' . $uuids['image_crop'] . '/delete');
     // Refresh the image style information and verify that the effect was
     // actually deleted.
     $entity_type_manager = $this->container->get('entity_type.manager');
     $style = $entity_type_manager->getStorage('image_style')->loadUnchanged($style->id());
-    $this->assertFalse($style->getEffects()->has($uuids['image_crop']), new FormattableMarkup(
-      'Effect with ID %uuid no longer found on image style %style',
-      [
-        '%uuid' => $uuids['image_crop'],
-        '%style' => $style->label(),
-      ]));
+    $this->assertFalse($style->getEffects()->has($uuids['image_crop']), "Effect with ID {$uuids['image_crop']} no longer found on image style {$style->label()}");
 
     // Additional test on Rotate effect, for transparent background.
     $edit = [
@@ -273,7 +271,8 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
       'data[random]' => 0,
       'data[bgcolor]' => '',
     ];
-    $this->drupalPostForm($style_path, ['new' => 'image_rotate'], 'Add');
+    $this->drupalGet($style_path);
+    $this->submitForm(['new' => 'image_rotate'], 'Add');
     $this->submitForm($edit, 'Add effect');
     $entity_type_manager = $this->container->get('entity_type.manager');
     $style = $entity_type_manager->getStorage('image_style')->loadUnchanged($style_name);
@@ -282,13 +281,14 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
     // Style deletion form.
 
     // Delete the style.
-    $this->drupalPostForm($style_path . '/delete', [], 'Delete');
+    $this->drupalGet($style_path . '/delete');
+    $this->submitForm([], 'Delete');
 
     // Confirm the style directory has been removed.
     $directory = 'public://styles/' . $style_name;
-    $this->assertDirectoryNotExists($directory);
+    $this->assertDirectoryDoesNotExist($directory);
 
-    $this->assertNull(ImageStyle::load($style_name), new FormattableMarkup('Image style %style successfully deleted.', ['%style' => $style->label()]));
+    $this->assertNull(ImageStyle::load($style_name), "Image style {$style->label()} successfully deleted.");
 
     // Test empty text when there are no image styles.
 
@@ -299,26 +299,25 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
 
     // Confirm that the empty text is correct on the image styles page.
     $this->drupalGet($admin_path);
-    $this->assertRaw(t('There are currently no styles. <a href=":url">Add a new one</a>.', [
-      ':url' => Url::fromRoute('image.style_add')->toString(),
-    ]));
+    $this->assertSession()->pageTextContains("There are currently no styles. Add a new one.");
+    $this->assertSession()->linkByHrefExists(Url::fromRoute('image.style_add')->toString());
 
   }
 
   /**
-   * Test deleting a style and choosing a replacement style.
+   * Tests deleting a style and choosing a replacement style.
    */
-  public function testStyleReplacement() {
+  public function testStyleReplacement(): void {
     // Create a new style.
-    $style_name = strtolower($this->randomMachineName(10));
+    $style_name = $this->randomMachineName(10);
     $style_label = $this->randomString();
     $style = ImageStyle::create(['name' => $style_name, 'label' => $style_label]);
     $style->save();
     $style_path = 'admin/config/media/image-styles/manage/';
 
     // Create an image field that uses the new style.
-    $field_name = strtolower($this->randomMachineName(10));
-    $this->createImageField($field_name, 'article');
+    $field_name = $this->randomMachineName(10);
+    $this->createImageField($field_name, 'node', 'article');
     \Drupal::service('entity_display.repository')
       ->getViewDisplay('node', 'article')
       ->setComponent($field_name, [
@@ -337,56 +336,61 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
     $original_uri = File::load($fid)->getFileUri();
 
     // Test that image is displayed using newly created style.
+    /** @var \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator */
+    $file_url_generator = \Drupal::service('file_url_generator');
+
     $this->drupalGet('node/' . $nid);
-    $this->assertRaw(file_url_transform_relative($style->buildUrl($original_uri)));
+    $this->assertSession()->responseContains($file_url_generator->transformRelative($style->buildUrl($original_uri)));
 
     // Rename the style and make sure the image field is updated.
-    $new_style_name = strtolower($this->randomMachineName(10));
+    $new_style_name = $this->randomMachineName(10);
     $new_style_label = $this->randomString();
     $edit = [
       'name' => $new_style_name,
       'label' => $new_style_label,
     ];
-    $this->drupalPostForm($style_path . $style_name, $edit, 'Save');
-    $this->assertText('Changes to the style have been saved.');
+    $this->drupalGet($style_path . $style_name);
+    $this->submitForm($edit, 'Save');
+    $this->assertSession()->statusMessageContains('Changes to the style have been saved.', 'status');
     $this->drupalGet('node/' . $nid);
 
     // Reload the image style using the new name.
     $style = ImageStyle::load($new_style_name);
-    $this->assertRaw(file_url_transform_relative($style->buildUrl($original_uri)));
+    $this->assertSession()->responseContains($file_url_generator->transformRelative($style->buildUrl($original_uri)));
 
     // Delete the style and choose a replacement style.
     $edit = [
       'replacement' => 'thumbnail',
     ];
-    $this->drupalPostForm($style_path . $new_style_name . '/delete', $edit, 'Delete');
-    $message = t('The image style %name has been deleted.', ['%name' => $new_style_label]);
-    $this->assertRaw($message);
+    $this->drupalGet($style_path . $new_style_name . '/delete');
+    $this->submitForm($edit, 'Delete');
+    $this->assertSession()->statusMessageContains("The image style {$new_style_label} has been deleted.", 'status');
 
     $replacement_style = ImageStyle::load('thumbnail');
     $this->drupalGet('node/' . $nid);
-    $this->assertRaw(file_url_transform_relative($replacement_style->buildUrl($original_uri)));
+    $this->assertSession()->responseContains($file_url_generator->transformRelative($replacement_style->buildUrl($original_uri)));
   }
 
   /**
    * Verifies that editing an image effect does not cause it to be duplicated.
    */
-  public function testEditEffect() {
+  public function testEditEffect(): void {
     // Add a scale effect.
     $style_name = 'test_style_effect_edit';
     $this->drupalGet('admin/config/media/image-styles/add');
     $this->submitForm(['label' => 'Test style effect edit', 'name' => $style_name], 'Create new style');
     $this->submitForm(['new' => 'image_scale_and_crop'], 'Add');
     $this->submitForm(['data[width]' => '300', 'data[height]' => '200'], 'Add effect');
-    $this->assertText('Scale and crop 300×200');
+    $this->assertSession()->pageTextContains('Scale and crop 300×200');
 
     // There should normally be only one edit link on this page initially.
-    $this->clickLink(t('Edit'));
+    $this->clickLink('Edit');
+    $this->assertSession()->pageTextContains("Edit Scale and crop effect on style Test style effect edit");
     $this->submitForm(['data[width]' => '360', 'data[height]' => '240'], 'Update effect');
-    $this->assertText('Scale and crop 360×240');
+    $this->assertSession()->pageTextContains('Scale and crop 360×240');
 
     // Check that the previous effect is replaced.
-    $this->assertNoText('Scale and crop 300×200');
+    $this->assertSession()->pageTextNotContains('Scale and crop 300×200');
 
     // Add another scale effect.
     $this->drupalGet('admin/config/media/image-styles/add');
@@ -395,7 +399,8 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
     $this->submitForm(['data[width]' => '12', 'data[height]' => '19'], 'Add effect');
 
     // Edit the scale effect that was just added.
-    $this->clickLink(t('Edit'));
+    $this->clickLink('Edit');
+    $this->assertSession()->pageTextContains("Edit Scale effect on style Test style scale edit scale");
     $this->submitForm(['data[width]' => '24', 'data[height]' => '19'], 'Update effect');
 
     // Add another scale effect and make sure both exist. Click through from
@@ -412,8 +417,8 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
     }
     $this->submitForm(['new' => 'image_scale'], 'Add');
     $this->submitForm(['data[width]' => '12', 'data[height]' => '19'], 'Add effect');
-    $this->assertText('Scale 24×19');
-    $this->assertText('Scale 12×19');
+    $this->assertSession()->pageTextContains('Scale 24×19');
+    $this->assertSession()->pageTextContains('Scale 12×19');
 
     // Try to edit a nonexistent effect.
     $uuid = $this->container->get('uuid');
@@ -422,13 +427,13 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
   }
 
   /**
-   * Test flush user interface.
+   * Tests flush user interface.
    */
-  public function testFlushUserInterface() {
+  public function testFlushUserInterface(): void {
     $admin_path = 'admin/config/media/image-styles';
 
     // Create a new style.
-    $style_name = strtolower($this->randomMachineName(10));
+    $style_name = $this->randomMachineName(10);
     $style = ImageStyle::create(['name' => $style_name, 'label' => $this->randomString()]);
     $style->save();
 
@@ -437,7 +442,7 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
     $image_uri = $files[0]->uri;
     $derivative_uri = $style->buildUri($image_uri);
     $this->assertTrue($style->createDerivative($image_uri, $derivative_uri));
-    $this->assertEqual(1, $this->getImageCount($style));
+    $this->assertEquals(1, $this->getImageCount($style));
 
     // Go to image styles list page and check if the flush operation link
     // exists.
@@ -446,25 +451,26 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
     $this->assertSession()->linkByHrefExists($flush_path);
 
     // Flush the image style derivatives using the user interface.
-    $this->drupalPostForm($flush_path, [], 'Flush');
+    $this->drupalGet($flush_path);
+    $this->submitForm([], 'Flush');
 
     // The derivative image file should have been deleted.
-    $this->assertEqual(0, $this->getImageCount($style));
+    $this->assertEquals(0, $this->getImageCount($style));
   }
 
   /**
    * Tests image style configuration import that does a delete.
    */
-  public function testConfigImport() {
+  public function testConfigImport(): void {
     // Create a new style.
-    $style_name = strtolower($this->randomMachineName(10));
+    $style_name = $this->randomMachineName(10);
     $style_label = $this->randomString();
     $style = ImageStyle::create(['name' => $style_name, 'label' => $style_label]);
     $style->save();
 
     // Create an image field that uses the new style.
-    $field_name = strtolower($this->randomMachineName(10));
-    $this->createImageField($field_name, 'article');
+    $field_name = $this->randomMachineName(10);
+    $this->createImageField($field_name, 'node', 'article');
     \Drupal::service('entity_display.repository')
       ->getViewDisplay('node', 'article')
       ->setComponent($field_name, [
@@ -484,7 +490,7 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
 
     // Test that image is displayed using newly created style.
     $this->drupalGet('node/' . $nid);
-    $this->assertRaw(file_url_transform_relative($style->buildUrl($original_uri)));
+    $this->assertSession()->responseContains(\Drupal::service('file_url_generator')->transformRelative($style->buildUrl($original_uri)));
 
     // Copy config to sync, and delete the image style.
     $sync = $this->container->get('config.storage.sync');
@@ -499,19 +505,53 @@ class ImageAdminStylesTest extends ImageFieldTestBase {
     $this->configImporter()->import();
 
     $this->assertNull(ImageStyle::load($style_name), 'Style deleted after config import.');
-    $this->assertEqual(0, $this->getImageCount($style), 'Image style was flushed after being deleted by config import.');
+    $this->assertEquals(0, $this->getImageCount($style), 'Image style was flushed after being deleted by config import.');
   }
 
   /**
    * Tests access for the image style listing.
    */
-  public function testImageStyleAccess() {
+  public function testImageStyleAccess(): void {
     $style = ImageStyle::create(['name' => 'style_foo', 'label' => $this->randomString()]);
     $style->save();
 
     $this->drupalGet('admin/config/media/image-styles');
-    $this->clickLink(t('Edit'));
-    $this->assertRaw(t('Select a new effect'));
+    $this->clickLink('Edit');
+    $this->assertSession()->pageTextContains("Select a new effect");
+  }
+
+  /**
+   * Tests the display of preview images using a private scheme.
+   */
+  public function testPreviewImageShowInPrivateScheme(): void {
+    $this->config('system.file')->set('default_scheme', 'private')->save();
+
+    /** @var \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator */
+    $file_url_generator = \Drupal::service('file_url_generator');
+
+    // Get the original preview image file in core config.
+    $original_path = $this->config('image.settings')->get('preview_image');
+    $style = ImageStyle::create(['name' => 'test_foo', 'label' => 'test foo']);
+    $style->save();
+
+    // Build the derivative preview image file with the Image Style.
+    // @see template_preprocess_image_style_preview()
+    $preview_file = $style->buildUri($original_path);
+    $style->createDerivative($original_path, $preview_file);
+
+    // Check if the derivative image exists.
+    $this->assertFileExists($preview_file);
+
+    // Generate itok token for the preview image.
+    $itok = $style->getPathToken('private://' . $original_path);
+
+    $url = $file_url_generator->generateAbsoluteString($preview_file);
+    $url .= '?itok=' . $itok;
+
+    // Check if the preview image with style is shown.
+    $this->drupalGet($url);
+    $this->assertSession()->statusCodeEquals(200);
+    $this->assertSession()->responseHeaderContains('Content-Type', 'image/png');
   }
 
 }
