@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\jsonapi\Kernel\Context;
 
 use Drupal\Core\Http\Exception\CacheableBadRequestHttpException;
@@ -11,18 +13,22 @@ use Drupal\Tests\jsonapi\Kernel\JsonapiKernelTestBase;
 /**
  * @coversDefaultClass \Drupal\jsonapi\Context\FieldResolver
  * @group jsonapi
- * @group legacy
+ * @group #slow
  *
  * @internal
  */
 class FieldResolverTest extends JsonapiKernelTestBase {
 
-  public static $modules = [
+  /**
+   * {@inheritdoc}
+   */
+  protected static $modules = [
     'entity_test',
+    'field',
+    'file',
     'jsonapi_test_field_aliasing',
     'jsonapi_test_field_filter_access',
     'serialization',
-    'field',
     'text',
     'user',
   ];
@@ -44,7 +50,7 @@ class FieldResolverTest extends JsonapiKernelTestBase {
   /**
    * {@inheritdoc}
    */
-  protected function setUp() {
+  protected function setUp(): void {
     parent::setUp();
 
     $this->installEntitySchema('entity_test_with_bundle');
@@ -83,7 +89,7 @@ class FieldResolverTest extends JsonapiKernelTestBase {
    * @covers ::resolveInternalEntityQueryPath
    * @dataProvider resolveInternalIncludePathProvider
    */
-  public function testResolveInternalIncludePath($expect, $external_path, $entity_type_id = 'entity_test_with_bundle', $bundle = 'bundle1') {
+  public function testResolveInternalIncludePath($expect, $external_path, $entity_type_id = 'entity_test_with_bundle', $bundle = 'bundle1'): void {
     $path_parts = explode('.', $external_path);
     $resource_type = $this->resourceTypeRepository->get($entity_type_id, $bundle);
     $this->assertEquals($expect, $this->sut->resolveInternalIncludePath($resource_type, $path_parts));
@@ -92,7 +98,7 @@ class FieldResolverTest extends JsonapiKernelTestBase {
   /**
    * Provides test cases for resolveInternalEntityQueryPath.
    */
-  public function resolveInternalIncludePathProvider() {
+  public static function resolveInternalIncludePathProvider() {
     return [
       'entity reference' => [[['field_test_ref2']], 'field_test_ref2'],
       'entity reference with multi target bundles' => [[['field_test_ref1']], 'field_test_ref1'],
@@ -125,9 +131,12 @@ class FieldResolverTest extends JsonapiKernelTestBase {
    * @covers ::resolveInternalIncludePath
    * @dataProvider resolveInternalIncludePathErrorProvider
    */
-  public function testResolveInternalIncludePathError($entity_type, $bundle, $external_path, $expected_message = '') {
+  public function testResolveInternalIncludePathError($entity_type, $bundle, $external_path, $expected_message = ''): void {
     $path_parts = explode('.', $external_path);
-    $this->setExpectedException(CacheableBadRequestHttpException::class, $expected_message);
+    $this->expectException(CacheableBadRequestHttpException::class);
+    if (!empty($expected_message)) {
+      $this->expectExceptionMessage($expected_message);
+    }
     $resource_type = $this->resourceTypeRepository->get($entity_type, $bundle);
     $this->sut->resolveInternalIncludePath($resource_type, $path_parts);
   }
@@ -135,7 +144,7 @@ class FieldResolverTest extends JsonapiKernelTestBase {
   /**
    * Provides test cases for ::testResolveInternalIncludePathError.
    */
-  public function resolveInternalIncludePathErrorProvider() {
+  public static function resolveInternalIncludePathErrorProvider() {
     return [
       // Should fail because none of these bundles have these fields.
       ['entity_test_with_bundle', 'bundle1', 'host.fail!!.deep'],
@@ -165,14 +174,15 @@ class FieldResolverTest extends JsonapiKernelTestBase {
    * @covers ::resolveInternalEntityQueryPath
    * @dataProvider resolveInternalEntityQueryPathProvider
    */
-  public function testResolveInternalEntityQueryPath($expect, $external_path, $entity_type_id = 'entity_test_with_bundle', $bundle = 'bundle1') {
-    $this->assertEquals($expect, $this->sut->resolveInternalEntityQueryPath($entity_type_id, $bundle, $external_path));
+  public function testResolveInternalEntityQueryPath($expect, $external_path, $entity_type_id = 'entity_test_with_bundle', $bundle = 'bundle1'): void {
+    $resource_type = $this->resourceTypeRepository->get($entity_type_id, $bundle);
+    $this->assertEquals($expect, $this->sut->resolveInternalEntityQueryPath($resource_type, $external_path));
   }
 
   /**
    * Provides test cases for ::testResolveInternalEntityQueryPath.
    */
-  public function resolveInternalEntityQueryPathProvider() {
+  public static function resolveInternalEntityQueryPathProvider() {
     return [
       'config entity as base' => [
         'uuid', 'id', 'entity_test_bundle', 'entity_test_bundle',
@@ -245,15 +255,19 @@ class FieldResolverTest extends JsonapiKernelTestBase {
    * @covers ::resolveInternalEntityQueryPath
    * @dataProvider resolveInternalEntityQueryPathErrorProvider
    */
-  public function testResolveInternalEntityQueryPathError($entity_type, $bundle, $external_path, $expected_message = '') {
-    $this->setExpectedException(CacheableBadRequestHttpException::class, $expected_message);
-    $this->sut->resolveInternalEntityQueryPath($entity_type, $bundle, $external_path);
+  public function testResolveInternalEntityQueryPathError($entity_type, $bundle, $external_path, $expected_message = ''): void {
+    $this->expectException(CacheableBadRequestHttpException::class);
+    if (!empty($expected_message)) {
+      $this->expectExceptionMessage($expected_message);
+    }
+    $resource_type = $this->resourceTypeRepository->get($entity_type, $bundle);
+    $this->sut->resolveInternalEntityQueryPath($resource_type, $external_path);
   }
 
   /**
    * Provides test cases for ::testResolveInternalEntityQueryPathError.
    */
-  public function resolveInternalEntityQueryPathErrorProvider() {
+  public static function resolveInternalEntityQueryPathErrorProvider() {
     return [
       'nested fields' => [
         'entity_test_with_bundle', 'bundle1', 'none.of.these.exist',
@@ -301,17 +315,17 @@ class FieldResolverTest extends JsonapiKernelTestBase {
       'entity reference then no delta with property specifier `target_id`' => [
         'entity_test_with_bundle', 'bundle1',
         'field_test_ref1.target_id',
-        'Invalid nested filtering. The property `target_id`, given in the path `field_test_ref1.target_id`, does not exist. Filter by `field_test_ref1`, not `field_test_ref1.target_id` (the JSON:API module elides property names from single-property fields).',
+        'Invalid nested filtering. The field `target_id`, given in the path `field_test_ref1.target_id`, does not exist.',
       ],
       'entity reference then delta 0 with property specifier `target_id`' => [
         'entity_test_with_bundle', 'bundle1',
         'field_test_ref1.0.target_id',
-        'Invalid nested filtering. The property `target_id`, given in the path `field_test_ref1.0.target_id`, does not exist. Filter by `field_test_ref1.0`, not `field_test_ref1.0.target_id` (the JSON:API module elides property names from single-property fields).',
+        'Invalid nested filtering. The field `target_id`, given in the path `field_test_ref1.0.target_id`, does not exist.',
       ],
       'entity reference then delta 1 with property specifier `target_id`' => [
         'entity_test_with_bundle', 'bundle1',
         'field_test_ref1.1.target_id',
-        'Invalid nested filtering. The property `target_id`, given in the path `field_test_ref1.1.target_id`, does not exist. Filter by `field_test_ref1.1`, not `field_test_ref1.1.target_id` (the JSON:API module elides property names from single-property fields).',
+        'Invalid nested filtering. The field `target_id`, given in the path `field_test_ref1.1.target_id`, does not exist.',
       ],
 
       'entity reference then no reference property then a complex field' => [
