@@ -1,8 +1,11 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\layout_builder\FunctionalJavascript;
 
 use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
+use Drupal\layout_builder\Entity\LayoutBuilderEntityViewDisplay;
 use Drupal\Tests\contextual\FunctionalJavascript\ContextualLinkClickTrait;
 
 /**
@@ -27,7 +30,7 @@ class BlockFormMessagesTest extends WebDriverTestBase {
   /**
    * {@inheritdoc}
    */
-  protected $defaultTheme = 'classy';
+  protected $defaultTheme = 'starterkit_theme';
 
   /**
    * {@inheritdoc}
@@ -35,30 +38,25 @@ class BlockFormMessagesTest extends WebDriverTestBase {
   protected function setUp(): void {
     parent::setUp();
     $this->createContentType(['type' => 'bundle_with_section_field']);
+    LayoutBuilderEntityViewDisplay::load('node.bundle_with_section_field.default')
+      ->enableLayoutBuilder()
+      ->setOverridable()
+      ->save();
+    $this->createNode(['type' => 'bundle_with_section_field']);
   }
 
   /**
    * Tests that validation messages are shown on the block form.
    */
-  public function testValidationMessage() {
-    // @todo Work out why this fixes random fails in this test.
-    //   https://www.drupal.org/project/drupal/issues/3055982
-    $this->getSession()->resizeWindow(800, 1000);
+  public function testValidationMessage(): void {
     $assert_session = $this->assertSession();
     $page = $this->getSession()->getPage();
 
     $this->drupalLogin($this->drupalCreateUser([
       'access contextual links',
       'configure any layout',
-      'administer node display',
-      'administer node fields',
     ]));
-    $field_ui_prefix = 'admin/structure/types/manage/bundle_with_section_field';
-    // Enable layout builder.
-    $this->drupalGet($field_ui_prefix . '/display/default');
-    $this->submitForm(['layout[enabled]' => TRUE], 'Save');
-    $page->findLink('Manage layout')->click();
-    $assert_session->addressEquals($field_ui_prefix . '/display/default/layout');
+    $this->drupalGet('node/1/layout');
     $page->findLink('Add block')->click();
     $this->assertNotEmpty($assert_session->waitForElementVisible('css', '#drupal-off-canvas .block-categories'));
     $page->findLink('Powered by Drupal')->click();
@@ -75,11 +73,10 @@ class BlockFormMessagesTest extends WebDriverTestBase {
     $assert_session->assertWaitOnAjaxRequest();
     $this->drupalGet($this->getUrl());
     $page->findButton('Save layout')->click();
-    $this->assertNotEmpty($assert_session->waitForElement('css', 'div:contains("The layout has been saved")'));
+    $this->assertNotEmpty($assert_session->waitForElement('css', 'div:contains("The layout override has been saved")'));
 
     // Ensure that message are displayed when configuring an existing block.
-    $this->drupalGet($field_ui_prefix . '/display/default/layout');
-    $assert_session->assertWaitOnAjaxRequest();
+    $this->drupalGet('node/1/layout');
     $this->clickContextualLink($block_css_locator, 'Configure', TRUE);
     $this->assertNotEmpty($assert_session->waitForElementVisible('css', '#drupal-off-canvas [name="settings[label]"]'));
     $page->findField('Title')->setValue('');
