@@ -3,6 +3,7 @@
 namespace Drupal\migrate;
 
 use Drupal\Component\Utility\Bytes;
+use Drupal\Core\StringTranslation\ByteSizeMarkup;
 use Drupal\Core\Utility\Error;
 use Drupal\Core\StringTranslation\StringTranslationTrait;
 use Drupal\migrate\Event\MigrateEvents;
@@ -14,7 +15,7 @@ use Drupal\migrate\Event\MigrateRowDeleteEvent;
 use Drupal\migrate\Exception\RequirementsException;
 use Drupal\migrate\Plugin\MigrateIdMapInterface;
 use Drupal\migrate\Plugin\MigrationInterface;
-use Symfony\Component\EventDispatcher\EventDispatcherInterface;
+use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;
 
 /**
  * Defines a migrate executable class.
@@ -77,7 +78,7 @@ class MigrateExecutable implements MigrateExecutableInterface {
   /**
    * The event dispatcher.
    *
-   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
+   * @var \Symfony\Contracts\EventDispatcher\EventDispatcherInterface
    */
   protected $eventDispatcher;
 
@@ -96,16 +97,14 @@ class MigrateExecutable implements MigrateExecutableInterface {
    * @param \Drupal\migrate\Plugin\MigrationInterface $migration
    *   The migration to run.
    * @param \Drupal\migrate\MigrateMessageInterface $message
-   *   The migrate message service.
-   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
-   *   The event dispatcher.
-   *
-   * @throws \Drupal\migrate\MigrateException
+   *   (optional) The migrate message service.
+   * @param \Symfony\Contracts\EventDispatcher\EventDispatcherInterface $event_dispatcher
+   *   (optional) The event dispatcher.
    */
-  public function __construct(MigrationInterface $migration, MigrateMessageInterface $message, EventDispatcherInterface $event_dispatcher = NULL) {
+  public function __construct(MigrationInterface $migration, ?MigrateMessageInterface $message = NULL, ?EventDispatcherInterface $event_dispatcher = NULL) {
     $this->migration = $migration;
-    $this->message = $message;
-    $this->migration->getIdMap()->setMessage($message);
+    $this->message = $message ?: new MigrateMessage();
+    $this->getIdMap()->setMessage($this->message);
     $this->eventDispatcher = $event_dispatcher;
     // Record the memory limit in bytes
     $limit = trim(ini_get('memory_limit'));
@@ -113,7 +112,7 @@ class MigrateExecutable implements MigrateExecutableInterface {
       $this->memoryLimit = PHP_INT_MAX;
     }
     else {
-      $this->memoryLimit = Bytes::toInt($limit);
+      $this->memoryLimit = Bytes::toNumber($limit);
     }
   }
 
@@ -135,7 +134,7 @@ class MigrateExecutable implements MigrateExecutableInterface {
   /**
    * Gets the event dispatcher.
    *
-   * @return \Symfony\Component\EventDispatcher\EventDispatcherInterface
+   * @return \Symfony\Contracts\EventDispatcher\EventDispatcherInterface
    */
   protected function getEventDispatcher() {
     if (!$this->eventDispatcher) {
@@ -157,7 +156,7 @@ class MigrateExecutable implements MigrateExecutableInterface {
         ]), 'error');
       return MigrationInterface::RESULT_FAILED;
     }
-    $this->getEventDispatcher()->dispatch(MigrateEvents::PRE_IMPORT, new MigrateImportEvent($this->migration, $this->message));
+    $this->getEventDispatcher()->dispatch(new MigrateImportEvent($this->migration, $this->message), MigrateEvents::PRE_IMPORT);
 
     // Knock off migration if the requirements haven't been met.
     try {
@@ -166,11 +165,10 @@ class MigrateExecutable implements MigrateExecutableInterface {
     catch (RequirementsException $e) {
       $this->message->display(
         $this->t(
-          'Migration @id did not meet the requirements. @message @requirements',
+          'Migration @id did not meet the requirements. @message',
           [
             '@id' => $this->migration->id(),
             '@message' => $e->getMessage(),
-            '@requirements' => $e->getRequirementsString(),
           ]
         ),
         'error'
@@ -180,101 +178,133 @@ class MigrateExecutable implements MigrateExecutableInterface {
     }
 
     $this->migration->setStatus(MigrationInterface::STATUS_IMPORTING);
-    $return = MigrationInterface::RESULT_COMPLETED;
     $source = $this->getSource();
-    $id_map = $this->migration->getIdMap();
 
     try {
       $source->rewind();
     }
     catch (\Exception $e) {
       $this->message->display(
-        $this->t('Migration failed with source plugin exception: @e', ['@e' => $e->getMessage()]), 'error');
+        $this->t('Migration failed with source plugin exception: @e in @file line @line', [
+          '@e' => $e->getMessage(),
+          '@file' => $e->getFile(),
+          '@line' => $e->getLine(),
+        ]), 'error');
       $this->migration->setStatus(MigrationInterface::STATUS_IDLE);
       return MigrationInterface::RESULT_FAILED;
     }
 
-    $destination = $this->migration->getDestinationPlugin();
-    while ($source->valid()) {
-      $row = $source->current();
-      $this->sourceIdValues = $row->getSourceIdValues();
-
+    // Get the process pipeline.
+    $pipeline = FALSE;
+    if ($source->valid()) {
       try {
-        $this->processRow($row);
-        $save = TRUE;
+        $pipeline = $this->migration->getProcessPlugins();
       }
       catch (MigrateException $e) {
-        $this->migration->getIdMap()->saveIdMapping($row, [], $e->getStatus());
+        $row = $source->current();
+        $this->sourceIdValues = $row->getSourceIdValues();
+        $this->getIdMap()->saveIdMapping($row, [], $e->getStatus());
         $this->saveMessage($e->getMessage(), $e->getLevel());
-        $save = FALSE;
-      }
-      catch (MigrateSkipRowException $e) {
-        if ($e->getSaveToMap()) {
-          $id_map->saveIdMapping($row, [], MigrateIdMapInterface::STATUS_IGNORED);
-        }
-        if ($message = trim($e->getMessage())) {
-          $this->saveMessage($message, MigrationInterface::MESSAGE_INFORMATIONAL);
-        }
-        $save = FALSE;
-      }
-
-      if ($save) {
-        try {
-          $this->getEventDispatcher()->dispatch(MigrateEvents::PRE_ROW_SAVE, new MigratePreRowSaveEvent($this->migration, $this->message, $row));
-          $destination_id_values = $destination->import($row, $id_map->lookupDestinationId($this->sourceIdValues));
-          $this->getEventDispatcher()->dispatch(MigrateEvents::POST_ROW_SAVE, new MigratePostRowSaveEvent($this->migration, $this->message, $row, $destination_id_values));
-          if ($destination_id_values) {
-            // We do not save an idMap entry for config.
-            if ($destination_id_values !== TRUE) {
-              $id_map->saveIdMapping($row, $destination_id_values, $this->sourceRowStatus, $destination->rollbackAction());
-            }
-          }
-          else {
-            $id_map->saveIdMapping($row, [], MigrateIdMapInterface::STATUS_FAILED);
-            if (!$id_map->messageCount()) {
-              $message = $this->t('New object was not saved, no error provided');
-              $this->saveMessage($message);
-              $this->message->display($message);
-            }
-          }
-        }
-        catch (MigrateException $e) {
-          $this->migration->getIdMap()->saveIdMapping($row, [], $e->getStatus());
-          $this->saveMessage($e->getMessage(), $e->getLevel());
-        }
-        catch (\Exception $e) {
-          $this->migration->getIdMap()->saveIdMapping($row, [], MigrateIdMapInterface::STATUS_FAILED);
-          $this->handleException($e);
-        }
-      }
-
-      $this->sourceRowStatus = MigrateIdMapInterface::STATUS_IMPORTED;
-
-      // Check for memory exhaustion.
-      if (($return = $this->checkStatus()) != MigrationInterface::RESULT_COMPLETED) {
-        break;
-      }
-
-      // If anyone has requested we stop, return the requested result.
-      if ($this->migration->getStatus() == MigrationInterface::STATUS_STOPPING) {
-        $return = $this->migration->getInterruptionResult();
-        $this->migration->clearInterruptionResult();
-        break;
-      }
-
-      try {
-        $source->next();
-      }
-      catch (\Exception $e) {
-        $this->message->display(
-          $this->t('Migration failed with source plugin exception: @e',
-            ['@e' => $e->getMessage()]), 'error');
-        $this->migration->setStatus(MigrationInterface::STATUS_IDLE);
-        return MigrationInterface::RESULT_FAILED;
       }
     }
 
-    $this->getEventDispatcher()->dispatch(MigrateEvents::POST_IMPORT, new MigrateImportEvent($this->migration, $this->message));
+    $return = MigrationInterface::RESULT_COMPLETED;
+    if ($pipeline) {
+      $id_map = $this->getIdMap();
+      $destination = $this->migration->getDestinationPlugin();
+      while ($source->valid()) {
+        $row = $source->current();
+        $this->sourceIdValues = $row->getSourceIdValues();
+
+        try {
+          foreach ($pipeline as $destination_property_name => $plugins) {
+            $this->processPipeline($row, $destination_property_name, $plugins, NULL);
+          }
+          $save = TRUE;
+        }
+        catch (MigrateException $e) {
+          $this->getIdMap()->saveIdMapping($row, [], $e->getStatus());
+          $msg = sprintf("%s:%s:%s", $this->migration->getPluginId(), $destination_property_name, $e->getMessage());
+          $this->saveMessage($msg, $e->getLevel());
+          $save = FALSE;
+        }
+        catch (MigrateSkipRowException $e) {
+          if ($e->getSaveToMap()) {
+            $id_map->saveIdMapping($row, [], MigrateIdMapInterface::STATUS_IGNORED);
+          }
+          if ($message = trim($e->getMessage())) {
+            $msg = sprintf("%s:%s: %s", $this->migration->getPluginId(), $destination_property_name, $message);
+            $this->saveMessage($msg, MigrationInterface::MESSAGE_INFORMATIONAL);
+          }
+          $save = FALSE;
+        }
+
+        if ($save) {
+          try {
+            $this->getEventDispatcher()
+              ->dispatch(new MigratePreRowSaveEvent($this->migration, $this->message, $row), MigrateEvents::PRE_ROW_SAVE);
+            $destination_ids = $id_map->lookupDestinationIds($this->sourceIdValues);
+            $destination_id_values = $destination_ids ? reset($destination_ids) : [];
+            $destination_id_values = $destination->import($row, $destination_id_values);
+            $this->getEventDispatcher()
+              ->dispatch(new MigratePostRowSaveEvent($this->migration, $this->message, $row, $destination_id_values), MigrateEvents::POST_ROW_SAVE);
+            if ($destination_id_values) {
+              // We do not save an idMap entry for config.
+              if ($destination_id_values !== TRUE) {
+                $id_map->saveIdMapping($row, $destination_id_values, $this->sourceRowStatus, $destination->rollbackAction());
+              }
+            }
+            else {
+              $id_map->saveIdMapping($row, [], MigrateIdMapInterface::STATUS_FAILED);
+              if (!$id_map->messageCount()) {
+                $message = $this->t('New object was not saved, no error provided');
+                $this->saveMessage($message);
+                $this->message->display($message);
+              }
+            }
+          }
+          catch (MigrateException $e) {
+            $this->getIdMap()->saveIdMapping($row, [], $e->getStatus());
+            $this->saveMessage($e->getMessage(), $e->getLevel());
+          }
+          catch (\Exception $e) {
+            $this->getIdMap()
+              ->saveIdMapping($row, [], MigrateIdMapInterface::STATUS_FAILED);
+            $this->handleException($e);
+          }
+        }
+
+        $this->sourceRowStatus = MigrateIdMapInterface::STATUS_IMPORTED;
+
+        // Check for memory exhaustion.
+        if (($return = $this->checkStatus()) != MigrationInterface::RESULT_COMPLETED) {
+          break;
+        }
+
+        // If anyone has requested we stop, return the requested result.
+        if ($this->migration->getStatus() == MigrationInterface::STATUS_STOPPING) {
+          $return = $this->migration->getInterruptionResult();
+          $this->migration->clearInterruptionResult();
+          break;
+        }
+
+        try {
+          $source->next();
+        }
+        catch (\Exception $e) {
+          $this->message->display(
+            $this->t('Migration failed with source plugin exception: @e in @file line @line', [
+              '@e' => $e->getMessage(),
+              '@file' => $e->getFile(),
+              '@line' => $e->getLine(),
+            ]), 'error');
+          $this->migration->setStatus(MigrationInterface::STATUS_IDLE);
+          return MigrationInterface::RESULT_FAILED;
+        }
+      }
+    }
+
+    $this->getEventDispatcher()->dispatch(new MigrateImportEvent($this->migration, $this->message), MigrateEvents::POST_IMPORT);
     $this->migration->setStatus(MigrationInterface::STATUS_IDLE);
     return $return;
   }
@@ -290,27 +320,28 @@ class MigrateExecutable implements MigrateExecutableInterface {
     }
 
     // Announce that rollback is about to happen.
-    $this->getEventDispatcher()->dispatch(MigrateEvents::PRE_ROLLBACK, new MigrateRollbackEvent($this->migration));
+    $this->getEventDispatcher()->dispatch(new MigrateRollbackEvent($this->migration), MigrateEvents::PRE_ROLLBACK);
 
     // Optimistically assume things are going to work out; if not, $return will be
     // updated to some other status.
     $return = MigrationInterface::RESULT_COMPLETED;
 
     $this->migration->setStatus(MigrationInterface::STATUS_ROLLING_BACK);
-    $id_map = $this->migration->getIdMap();
+    $id_map = $this->getIdMap();
     $destination = $this->migration->getDestinationPlugin();
 
     // Loop through each row in the map, and try to roll it back.
-    foreach ($id_map as $map_row) {
+    $id_map->rewind();
+    while ($id_map->valid()) {
       $destination_key = $id_map->currentDestination();
       if ($destination_key) {
         $map_row = $id_map->getRowByDestination($destination_key);
-        if ($map_row['rollback_action'] == MigrateIdMapInterface::ROLLBACK_DELETE) {
+        if (!isset($map_row['rollback_action']) || $map_row['rollback_action'] == MigrateIdMapInterface::ROLLBACK_DELETE) {
           $this->getEventDispatcher()
-            ->dispatch(MigrateEvents::PRE_ROW_DELETE, new MigrateRowDeleteEvent($this->migration, $destination_key));
+            ->dispatch(new MigrateRowDeleteEvent($this->migration, $destination_key), MigrateEvents::PRE_ROW_DELETE);
           $destination->rollback($destination_key);
           $this->getEventDispatcher()
-            ->dispatch(MigrateEvents::POST_ROW_DELETE, new MigrateRowDeleteEvent($this->migration, $destination_key));
+            ->dispatch(new MigrateRowDeleteEvent($this->migration, $destination_key), MigrateEvents::POST_ROW_DELETE);
         }
         // We're now done with this row, so remove it from the map.
         $id_map->deleteDestination($destination_key);
@@ -321,6 +352,7 @@ class MigrateExecutable implements MigrateExecutableInterface {
         $source_key = $id_map->currentSource();
         $id_map->delete($source_key);
       }
+      $id_map->next();
 
       // Check for memory exhaustion.
       if (($return = $this->checkStatus()) != MigrationInterface::RESULT_COMPLETED) {
@@ -336,62 +368,113 @@ class MigrateExecutable implements MigrateExecutableInterface {
     }
 
     // Notify modules that rollback attempt was complete.
-    $this->getEventDispatcher()->dispatch(MigrateEvents::POST_ROLLBACK, new MigrateRollbackEvent($this->migration));
+    $this->getEventDispatcher()->dispatch(new MigrateRollbackEvent($this->migration), MigrateEvents::POST_ROLLBACK);
     $this->migration->setStatus(MigrationInterface::STATUS_IDLE);
 
     return $return;
   }
 
+  /**
+   * Get the ID map from the current migration.
+   *
+   * @return \Drupal\migrate\Plugin\MigrateIdMapInterface
+   *   The ID map.
+   */
+  protected function getIdMap() {
+    return $this->migration->getIdMap();
+  }
+
   /**
    * {@inheritdoc}
    */
-  public function processRow(Row $row, array $process = NULL, $value = NULL) {
+  public function processRow(Row $row, ?array $process = NULL, $value = NULL) {
     foreach ($this->migration->getProcessPlugins($process) as $destination => $plugins) {
-      $multiple = FALSE;
-      /** @var $plugin \Drupal\migrate\Plugin\MigrateProcessInterface */
-      foreach ($plugins as $plugin) {
-        $definition = $plugin->getPluginDefinition();
-        // Many plugins expect a scalar value but the current value of the
-        // pipeline might be multiple scalars (this is set by the previous
-        // plugin) and in this case the current value needs to be iterated
-        // and each scalar separately transformed.
-        if ($multiple && !$definition['handle_multiples']) {
-          $new_value = [];
-          if (!is_array($value)) {
-            throw new MigrateException(sprintf('Pipeline failed at %s plugin for destination %s: %s received instead of an array,', $plugin->getPluginId(), $destination, $value));
-          }
-          $break = FALSE;
-          foreach ($value as $scalar_value) {
-            try {
-              $new_value[] = $plugin->transform($scalar_value, $this, $row, $destination);
-            }
-            catch (MigrateSkipProcessException $e) {
-              $new_value[] = NULL;
-              $break = TRUE;
-            }
-          }
-          $value = $new_value;
-          if ($break) {
-            break;
-          }
+      $this->processPipeline($row, $destination, $plugins, $value);
+    }
+  }
+
+  /**
+   * Runs a process pipeline.
+   *
+   * @param \Drupal\migrate\Row $row
+   *   The $row to be processed.
+   * @param string $destination
+   *   The destination property name.
+   * @param array $plugins
+   *   The process pipeline plugins.
+   * @param mixed $value
+   *   (optional) Initial value of the pipeline for the destination.
+   *
+   * @see \Drupal\migrate\MigrateExecutableInterface::processRow
+   *
+   * @throws \Drupal\migrate\MigrateException
+   */
+  protected function processPipeline(Row $row, string $destination, array $plugins, $value) {
+    $multiple = FALSE;
+    /** @var \Drupal\migrate\Plugin\MigrateProcessInterface $plugin */
+    foreach ($plugins as $plugin) {
+      $definition = $plugin->getPluginDefinition();
+      // Many plugins expect a scalar value but the current value of the
+      // pipeline might be multiple scalars (this is set by the previous plugin)
+      // and in this case the current value needs to be iterated and each scalar
+      // separately transformed.
+      if ($multiple && !$definition['handle_multiples']) {
+        $new_value = [];
+        if (!is_array($value)) {
+          throw new MigrateException(sprintf('Pipeline failed at %s plugin for destination %s: %s received instead of an array,', $plugin->getPluginId(), $destination, $value));
         }
-        else {
+        $break = FALSE;
+        foreach ($value as $scalar_value) {
+          $plugin->reset();
           try {
-            $value = $plugin->transform($value, $this, $row, $destination);
+            $new_value[] = $plugin->transform($scalar_value, $this, $row, $destination);
           }
           catch (MigrateSkipProcessException $e) {
-            $value = NULL;
-            break;
+            $new_value[] = NULL;
+            $break = TRUE;
           }
-          $multiple = $plugin->multiple();
+          catch (MigrateException $e) {
+            // Prepend the process plugin id to the message.
+            $message = sprintf("%s: %s", $plugin->getPluginId(), $e->getMessage());
+            throw new MigrateException($message);
+          }
+          if ($plugin->isPipelineStopped()) {
+            $break = TRUE;
+          }
+        }
+        $value = $new_value;
+        if ($break) {
+          break;
         }
       }
-      // No plugins or no value means do not set.
-      if ($plugins && !is_null($value)) {
+      else {
+        $plugin->reset();
+        try {
+          $value = $plugin->transform($value, $this, $row, $destination);
+        }
+        catch (MigrateSkipProcessException $e) {
+          $value = NULL;
+          break;
+        }
+        catch (MigrateException $e) {
+          // Prepend the process plugin id to the message.
+          $message = sprintf("%s: %s", $plugin->getPluginId(), $e->getMessage());
+          throw new MigrateException($message);
+        }
+        if ($plugin->isPipelineStopped()) {
+          break;
+        }
+        $multiple = $plugin->multiple();
+      }
+    }
+    // Ensure all values, including nulls, are migrated.
+    if ($plugins) {
+      if (isset($value)) {
         $row->setDestinationProperty($destination, $value);
       }
-      // Reset the value.
-      $value = NULL;
+      else {
+        $row->setEmptyDestinationProperty($destination);
+      }
     }
   }
 
@@ -409,7 +492,7 @@ class MigrateExecutable implements MigrateExecutableInterface {
    * {@inheritdoc}
    */
   public function saveMessage($message, $level = MigrationInterface::MESSAGE_ERROR) {
-    $this->migration->getIdMap()->saveMessage($this->sourceIdValues, $message, $level);
+    $this->getIdMap()->saveMessage($this->sourceIdValues, $message, $level);
   }
 
   /**
@@ -462,8 +545,8 @@ class MigrateExecutable implements MigrateExecutableInterface {
           'Memory usage is @usage (@pct% of limit @limit), reclaiming memory.',
           [
             '@pct' => round($pct_memory * 100),
-            '@usage' => $this->formatSize($usage),
-            '@limit' => $this->formatSize($this->memoryLimit),
+            '@usage' => ByteSizeMarkup::create($usage, NULL, $this->stringTranslation),
+            '@limit' => ByteSizeMarkup::create($this->memoryLimit, NULL, $this->stringTranslation),
           ]
         ),
         'warning'
@@ -478,8 +561,8 @@ class MigrateExecutable implements MigrateExecutableInterface {
             'Memory usage is now @usage (@pct% of limit @limit), not enough reclaimed, starting new batch',
             [
               '@pct' => round($pct_memory * 100),
-              '@usage' => $this->formatSize($usage),
-              '@limit' => $this->formatSize($this->memoryLimit),
+              '@usage' => ByteSizeMarkup::create($usage, NULL, $this->stringTranslation),
+              '@limit' => ByteSizeMarkup::create($this->memoryLimit, NULL, $this->stringTranslation),
             ]
           ),
           'warning'
@@ -492,8 +575,8 @@ class MigrateExecutable implements MigrateExecutableInterface {
             'Memory usage is now @usage (@pct% of limit @limit), reclaimed enough, continuing',
             [
               '@pct' => round($pct_memory * 100),
-              '@usage' => $this->formatSize($usage),
-              '@limit' => $this->formatSize($this->memoryLimit),
+              '@usage' => ByteSizeMarkup::create($usage, NULL, $this->stringTranslation),
+              '@limit' => ByteSizeMarkup::create($this->memoryLimit, NULL, $this->stringTranslation),
             ]
           ),
           'warning');
@@ -526,13 +609,10 @@ class MigrateExecutable implements MigrateExecutableInterface {
     // plenty of memory to continue.
     drupal_static_reset();
 
-    // Entity storage can blow up with caches so clear them out.
-    $manager = \Drupal::entityManager();
-    foreach ($manager->getDefinitions() as $id => $definition) {
-      $manager->getStorage($id)->resetCache();
-    }
+    // Entity storage can blow up with caches, so clear it out.
+    \Drupal::service('entity.memory_cache')->deleteAll();
 
-    // @TODO: explore resetting the container.
+    // @todo Explore resetting the container.
 
     // Run garbage collector to further reduce memory.
     gc_collect_cycles();
@@ -548,8 +628,15 @@ class MigrateExecutable implements MigrateExecutableInterface {
    *
    * @return string
    *   A translated string representation of the size.
+   *
+   * @deprecated in drupal:10.2.0 and is removed from drupal:11.0.0. Use
+   *   \Drupal\Core\StringTranslation\ByteSizeMarkup::create($size, $langcode)
+   *   instead.
+   *
+   * @see https://www.drupal.org/node/2999981
    */
   protected function formatSize($size) {
+    @trigger_error(__METHOD__ . '() is deprecated in drupal:10.2.0 and is removed from drupal:11.0.0. Use \Drupal\Core\StringTranslation\ByteSizeMarkup::create($size, $langcode) instead. See https://www.drupal.org/node/2999981', E_USER_DEPRECATED);
     return format_size($size);
   }
 
