@@ -2,8 +2,6 @@
 
 namespace Drupal\search;
 
-use Drupal\Core\Database\Query\Condition;
-use Drupal\Component\Utility\Unicode;
 use Drupal\Core\Database\Query\SelectExtender;
 use Drupal\Core\Database\Query\SelectInterface;
 
@@ -112,7 +110,7 @@ class SearchQuery extends SelectExtender {
    * This is always used for the second step in the query, but is not part of
    * the preparation step unless $this->simple is FALSE.
    *
-   * @var DatabaseCondition
+   * @var \Drupal\Core\Database\Query\ConditionInterface[]
    */
   protected $conditions;
 
@@ -179,6 +177,7 @@ class SearchQuery extends SelectExtender {
    *
    * @var int
    */
+  // phpcs:ignore Drupal.NamingConventions.ValidVariableName.LowerCamelName
   protected $relevance_count = 0;
 
   /**
@@ -206,7 +205,7 @@ class SearchQuery extends SelectExtender {
     $this->addTag('search_' . $type);
 
     // Initialize conditions and status.
-    $this->conditions = new Condition('AND');
+    $this->conditions = $this->connection->condition('AND');
     $this->status = 0;
 
     return $this;
@@ -234,6 +233,8 @@ class SearchQuery extends SelectExtender {
     // Classify tokens.
     $in_or = FALSE;
     $limit_combinations = \Drupal::config('search.settings')->get('and_or_limit');
+    /** @var \Drupal\search\SearchTextProcessorInterface $text_processor */
+    $text_processor = \Drupal::service('search.text_processor');
     // The first search expression does not count as AND.
     $and_count = -1;
     $or_count = 0;
@@ -247,7 +248,7 @@ class SearchQuery extends SelectExtender {
 
       // Strip off phrase quotes.
       $phrase = FALSE;
-      if ($match[2]{0} == '"') {
+      if ($match[2][0] == '"') {
         $match[2] = substr($match[2], 1, -1);
         $phrase = TRUE;
         $this->simple = FALSE;
@@ -256,7 +257,7 @@ class SearchQuery extends SelectExtender {
       // Simplify keyword according to indexing rules and external
       // preprocessors. Use same process as during search indexing, so it
       // will match search index.
-      $words = search_simplify($match[2]);
+      $words = $text_processor->analyze($match[2]);
       // Re-explode in case simplification added more words, except when
       // matching a phrase.
       $words = $phrase ? [$words] : preg_split('/ /', $words, -1, PREG_SPLIT_NO_EMPTY);
@@ -265,7 +266,7 @@ class SearchQuery extends SelectExtender {
         $this->keys['negative'] = array_merge($this->keys['negative'], $words);
       }
       // OR operator: instead of a single keyword, we store an array of all
-      // OR'd keywords.
+      // ORed keywords.
       elseif ($match[2] == 'OR' && count($this->keys['positive'])) {
         $last = array_pop($this->keys['positive']);
         // Starting a new OR?
@@ -307,21 +308,21 @@ class SearchQuery extends SelectExtender {
     foreach ($this->keys['positive'] as $key) {
       // Group of ORed terms.
       if (is_array($key) && count($key)) {
-        // If we had already found one OR, this is another one AND-ed with the
+        // If we had already found one OR, this is another one ANDed with the
         // first, meaning it is not a simple query.
         if ($has_or) {
           $this->simple = FALSE;
         }
         $has_or = TRUE;
         $has_new_scores = FALSE;
-        $queryor = new Condition('OR');
+        $query_or = $this->connection->condition('OR');
         foreach ($key as $or) {
-          list($num_new_scores) = $this->parseWord($or);
+          [$num_new_scores] = $this->parseWord($or);
           $has_new_scores |= $num_new_scores;
-          $queryor->condition('d.data', "% $or %", 'LIKE');
+          $query_or->condition('d.data', "% $or %", 'LIKE');
         }
-        if (count($queryor)) {
-          $this->conditions->condition($queryor);
+        if (count($query_or)) {
+          $this->conditions->condition($query_or);
           // A group of OR keywords only needs to match once.
           $this->matches += ($has_new_scores > 0);
         }
@@ -329,7 +330,7 @@ class SearchQuery extends SelectExtender {
       // Single ANDed term.
       else {
         $has_and = TRUE;
-        list($num_new_scores, $num_valid_words) = $this->parseWord($key);
+        [$num_new_scores, $num_valid_words] = $this->parseWord($key);
         $this->conditions->condition('d.data', "% $key %", 'LIKE');
         if (!$num_valid_words) {
           $this->simple = FALSE;
@@ -364,7 +365,7 @@ class SearchQuery extends SelectExtender {
     $split = explode(' ', $word);
     foreach ($split as $s) {
       $num = is_numeric($s);
-      if ($num || Unicode::strlen($s) >= \Drupal::config('search.settings')->get('index.minimum_word_size')) {
+      if ($num || mb_strlen($s) >= \Drupal::config('search.settings')->get('index.minimum_word_size')) {
         if (!isset($this->words[$s])) {
           $this->words[$s] = $s;
           $num_new_scores++;
@@ -402,14 +403,14 @@ class SearchQuery extends SelectExtender {
     }
 
     // Build the basic search query: match the entered keywords.
-    $or = new Condition('OR');
+    $or = $this->connection->condition('OR');
     foreach ($this->words as $word) {
       $or->condition('i.word', $word);
     }
     $this->condition($or);
 
     // Add keyword normalization information to the query.
-    $this->join('search_total', 't', 'i.word = t.word');
+    $this->join('search_total', 't', '[i].[word] = [t].[word]');
     $this
       ->condition('i.type', $this->type)
       ->groupBy('i.type')
@@ -429,7 +430,7 @@ class SearchQuery extends SelectExtender {
     // For complex search queries, add the LIKE conditions; if the query is
     // simple, we do not need them for normalization.
     if (!$this->simple) {
-      $normalize_query->join('search_dataset', 'd', 'i.sid = d.sid AND i.type = d.type AND i.langcode = d.langcode');
+      $normalize_query->join('search_dataset', 'd', '[i].[sid] = [d].[sid] AND [i].[type] = [d].[type] AND [i].[langcode] = [d].[langcode]');
       if (count($this->conditions)) {
         $normalize_query->condition($this->conditions);
       }
@@ -438,7 +439,7 @@ class SearchQuery extends SelectExtender {
     // Calculate normalization, which is the max of all the search scores for
     // positive keywords in the query. And note that the query could have other
     // fields added to it by the user of this extension.
-    $normalize_query->addExpression('SUM(i.score * t.count)', 'calculated_score');
+    $normalize_query->addExpression('SUM([i].[score] * [t].[count])', 'calculated_score');
     $result = $normalize_query
       ->range(0, 1)
       ->orderBy('calculated_score', 'DESC')
@@ -461,7 +462,7 @@ class SearchQuery extends SelectExtender {
   /**
    * {@inheritdoc}
    */
-  public function preExecute(SelectInterface $query = NULL) {
+  public function preExecute(?SelectInterface $query = NULL) {
     if (!$this->executedPrepare) {
       $this->prepareAndNormalize();
     }
@@ -521,7 +522,7 @@ class SearchQuery extends SelectExtender {
     // search expression. So, use string replacement to change this to a
     // calculated query expression, counting the number of occurrences so
     // in the execute() method we can add arguments.
-    while (($pos = strpos($score, 'i.relevance')) !== FALSE) {
+    while (str_contains($score, 'i.relevance')) {
       $pieces = explode('i.relevance', $score, 2);
       $score = implode('((ROUND(:normalization_' . $this->relevance_count . ', 4)) * i.score * t.count)', $pieces);
       $this->relevance_count++;
@@ -551,7 +552,7 @@ class SearchQuery extends SelectExtender {
     }
 
     // Add conditions to the query.
-    $this->join('search_dataset', 'd', 'i.sid = d.sid AND i.type = d.type AND i.langcode = d.langcode');
+    $this->join('search_dataset', 'd', '[i].[sid] = [d].[sid] AND [i].[type] = [d].[type] AND [i].[langcode] = [d].[langcode]');
     if (count($this->conditions)) {
       $this->condition($this->conditions);
     }
@@ -571,10 +572,9 @@ class SearchQuery extends SelectExtender {
       }
     }
 
-
     // Add arguments for the keyword relevance normalization number.
     $normalization = 1.0 / $this->normalize;
-    for ($i = 0; $i < $this->relevance_count; $i++ ) {
+    for ($i = 0; $i < $this->relevance_count; $i++) {
       $this->scoresArguments[':normalization_' . $i] = $normalization;
     }
 
@@ -610,7 +610,7 @@ class SearchQuery extends SelectExtender {
     $inner = clone $this->query;
 
     // Add conditions to query.
-    $inner->join('search_dataset', 'd', 'i.sid = d.sid AND i.type = d.type');
+    $inner->join('search_dataset', 'd', '[i].[sid] = [d].[sid] AND [i].[type] = [d].[type]');
     if (count($this->conditions)) {
       $inner->condition($this->conditions);
     }
@@ -623,7 +623,7 @@ class SearchQuery extends SelectExtender {
     $expressions = [];
 
     // Add sid as the only field and count them as a subquery.
-    $count = db_select($inner->fields('i', ['sid']), NULL, ['target' => 'replica']);
+    $count = $this->connection->select($inner->fields('i', ['sid']), NULL);
 
     // Add the COUNT() expression.
     $count->addExpression('COUNT(*)');
