@@ -1,9 +1,13 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\system\Kernel\Extension;
 
+use Drupal\Core\Entity\ContentEntityTypeInterface;
 use Drupal\Core\Extension\MissingDependencyException;
 use Drupal\Core\Extension\ModuleUninstallValidatorException;
+use Drupal\Core\Extension\ProfileExtensionList;
 use Drupal\entity_test\Entity\EntityTest;
 use Drupal\KernelTests\KernelTestBase;
 
@@ -17,14 +21,18 @@ class ModuleHandlerTest extends KernelTestBase {
   /**
    * {@inheritdoc}
    */
-  public static $modules = ['system'];
+  protected static $modules = ['system'];
 
   /**
    * The basic functionality of retrieving enabled modules.
    */
-  public function testModuleList() {
+  public function testModuleList(): void {
     $module_list = ['system'];
-
+    $database_module = \Drupal::database()->getProvider();
+    if ($database_module !== 'core') {
+      $module_list[] = $database_module;
+    }
+    sort($module_list);
     $this->assertModuleList($module_list, 'Initial');
 
     // Try to install a new module.
@@ -48,7 +56,7 @@ class ModuleHandlerTest extends KernelTestBase {
     ];
     $this->moduleHandler()->setModuleList($fixed_list);
     $new_module_list = array_combine(array_keys($fixed_list), array_keys($fixed_list));
-    $this->assertModuleList($new_module_list, t('When using a fixed list'));
+    $this->assertModuleList($new_module_list, 'When using a fixed list');
   }
 
   /**
@@ -56,12 +64,15 @@ class ModuleHandlerTest extends KernelTestBase {
    *
    * @param array $expected_values
    *   The expected values, sorted by weight and module name.
-   * @param $condition
+   * @param string $condition
+   *   The condition being tested, such as 'After adding a module'.
+   *
+   * @internal
    */
-  protected function assertModuleList(array $expected_values, $condition) {
+  protected function assertModuleList(array $expected_values, string $condition): void {
     $expected_values = array_values(array_unique($expected_values));
     $enabled_modules = array_keys($this->container->get('module_handler')->getModuleList());
-    $this->assertEqual($expected_values, $enabled_modules, format_string('@condition: extension handler returns correct results', ['@condition' => $condition]));
+    $this->assertEquals($expected_values, $enabled_modules, "$condition: extension handler returns correct results");
   }
 
   /**
@@ -76,134 +87,99 @@ class ModuleHandlerTest extends KernelTestBase {
    * @see module_test_system_info_alter()
    * @see https://www.drupal.org/files/issues/dep.gv__0.png
    */
-  public function testDependencyResolution() {
+  public function testDependencyResolution(): void {
     $this->enableModules(['module_test']);
     $this->assertTrue($this->moduleHandler()->moduleExists('module_test'), 'Test module is enabled.');
 
     // Ensure that modules are not enabled.
-    $this->assertFalse($this->moduleHandler()->moduleExists('color'), 'Color module is disabled.');
+    $this->assertFalse($this->moduleHandler()->moduleExists('dblog'), 'dblog module is disabled.');
     $this->assertFalse($this->moduleHandler()->moduleExists('config'), 'Config module is disabled.');
     $this->assertFalse($this->moduleHandler()->moduleExists('help'), 'Help module is disabled.');
 
     // Create a missing fake dependency.
-    // Color will depend on Config, which depends on a non-existing module Foo.
+    // dblog will depend on Config, which depends on a non-existing module Foo.
     // Nothing should be installed.
     \Drupal::state()->set('module_test.dependency', 'missing dependency');
-    \Drupal::service('extension.list.module')->reset();
 
     try {
-      $result = $this->moduleInstaller()->install(['color']);
-      $this->fail(t('ModuleInstaller::install() throws an exception if dependencies are missing.'));
+      $result = $this->moduleInstaller()->install(['dblog']);
+      $this->fail('ModuleInstaller::install() throws an exception if dependencies are missing.');
     }
     catch (MissingDependencyException $e) {
-      $this->pass(t('ModuleInstaller::install() throws an exception if dependencies are missing.'));
+      // Expected exception; just continue testing.
     }
 
-    $this->assertFalse($this->moduleHandler()->moduleExists('color'), 'ModuleInstaller::install() aborts if dependencies are missing.');
+    $this->assertFalse($this->moduleHandler()->moduleExists('dblog'), 'ModuleInstaller::install() aborts if dependencies are missing.');
 
     // Fix the missing dependency.
-    // Color module depends on Config. Config depends on Help module.
+    // dblog module depends on Config. Config depends on Help module.
     \Drupal::state()->set('module_test.dependency', 'dependency');
-    \Drupal::service('extension.list.module')->reset();
 
-    $result = $this->moduleInstaller()->install(['color']);
+    $result = $this->moduleInstaller()->install(['dblog']);
     $this->assertTrue($result, 'ModuleInstaller::install() returns the correct value.');
 
     // Verify that the fake dependency chain was installed.
-    $this->assertTrue($this->moduleHandler()->moduleExists('config') && $this->moduleHandler()->moduleExists('help'), 'Dependency chain was installed.');
+    $this->assertTrue($this->moduleHandler()->moduleExists('config'));
+    $this->assertTrue($this->moduleHandler()->moduleExists('help'));
 
     // Verify that the original module was installed.
-    $this->assertTrue($this->moduleHandler()->moduleExists('color'), 'Module installation with dependencies succeeded.');
+    $this->assertTrue($this->moduleHandler()->moduleExists('dblog'), 'Module installation with dependencies succeeded.');
 
     // Verify that the modules were enabled in the correct order.
-    $module_order = \Drupal::state()->get('module_test.install_order') ?: [];
-    $this->assertEqual($module_order, ['help', 'config', 'color']);
+    $module_order = \Drupal::state()->get('module_test.install_order', []);
+    $this->assertEquals(['help', 'config', 'dblog'], $module_order);
 
     // Uninstall all three modules explicitly, but in the incorrect order,
     // and make sure that ModuleInstaller::uninstall() uninstalled them in the
     // correct sequence.
-    $result = $this->moduleInstaller()->uninstall(['config', 'help', 'color']);
+    $result = $this->moduleInstaller()->uninstall(['config', 'help', 'dblog']);
     $this->assertTrue($result, 'ModuleInstaller::uninstall() returned TRUE.');
 
-    foreach (['color', 'config', 'help'] as $module) {
-      $this->assertEqual(drupal_get_installed_schema_version($module), SCHEMA_UNINSTALLED, "$module module was uninstalled.");
+    /** @var \Drupal\Core\Update\UpdateHookRegistry $update_registry */
+    $update_registry = \Drupal::service('update.update_hook_registry');
+    foreach (['dblog', 'config', 'help'] as $module) {
+      $this->assertEquals($update_registry::SCHEMA_UNINSTALLED, $update_registry->getInstalledVersion($module), "{$module} module was uninstalled.");
     }
-    $uninstalled_modules = \Drupal::state()->get('module_test.uninstall_order') ?: [];
-    $this->assertEqual($uninstalled_modules, ['color', 'config', 'help'], 'Modules were uninstalled in the correct order.');
+    $uninstalled_modules = \Drupal::state()->get('module_test.uninstall_order', []);
+    $this->assertEquals(['dblog', 'config', 'help'], $uninstalled_modules, 'Modules were uninstalled in the correct order.');
 
-    // Enable Color module again, which should enable both the Config module and
+    // Enable dblog module again, which should enable both the Config module and
     // Help module. But, this time do it with Config module declaring a
     // dependency on a specific version of Help module in its info file. Make
     // sure that Drupal\Core\Extension\ModuleInstaller::install() still works.
     \Drupal::state()->set('module_test.dependency', 'version dependency');
-    \Drupal::service('extension.list.module')->reset();
 
-    $result = $this->moduleInstaller()->install(['color']);
+    $result = $this->moduleInstaller()->install(['dblog']);
     $this->assertTrue($result, 'ModuleInstaller::install() returns the correct value.');
 
     // Verify that the fake dependency chain was installed.
-    $this->assertTrue($this->moduleHandler()->moduleExists('config') && $this->moduleHandler()->moduleExists('help'), 'Dependency chain was installed.');
+    $this->assertTrue($this->moduleHandler()->moduleExists('config'));
+    $this->assertTrue($this->moduleHandler()->moduleExists('help'));
 
     // Verify that the original module was installed.
-    $this->assertTrue($this->moduleHandler()->moduleExists('color'), 'Module installation with version dependencies succeeded.');
+    $this->assertTrue($this->moduleHandler()->moduleExists('dblog'), 'Module installation with version dependencies succeeded.');
 
     // Finally, verify that the modules were enabled in the correct order.
-    $enable_order = \Drupal::state()->get('module_test.install_order') ?: [];
-    $this->assertIdentical($enable_order, ['help', 'config', 'color']);
-  }
-
-  /**
-   * Tests uninstalling a module that is a "dependency" of a profile.
-   *
-   * Note this test does not trigger the deprecation error because of static
-   * caching in \Drupal\Core\Extension\InfoParser::parse().
-   *
-   * @group legacy
-   */
-  public function testUninstallProfileDependencyBC() {
-    $profile = 'testing_install_profile_dependencies_bc';
-    $dependency = 'dblog';
-    $this->setInstallProfile($profile);
-    // Prime the drupal_get_filename() static cache with the location of the
-    // testing profile as it is not the currently active profile and we don't
-    // yet have any cached way to retrieve its location.
-    // @todo Remove as part of https://www.drupal.org/node/2186491
-    drupal_get_filename('profile', $profile, 'core/profiles/' . $profile . '/' . $profile . '.info.yml');
-    $this->enableModules(['module_test', $profile]);
-
-    $data = \Drupal::service('extension.list.module')->reset()->getList();
-    $this->assertFalse(isset($data[$profile]->requires[$dependency]));
-    $this->assertContains($dependency, $data[$profile]->info['install']);
-
-    $this->moduleInstaller()->install([$dependency]);
-    $this->assertTrue($this->moduleHandler()->moduleExists($dependency));
-
-    // Uninstall the profile module "dependency".
-    $result = $this->moduleInstaller()->uninstall([$dependency]);
-    $this->assertTrue($result, 'ModuleInstaller::uninstall() returns TRUE.');
-    $this->assertFalse($this->moduleHandler()->moduleExists($dependency));
-    $this->assertEqual(drupal_get_installed_schema_version($dependency), SCHEMA_UNINSTALLED, "$dependency module was uninstalled.");
-
-    // Verify that the installation profile itself was not uninstalled.
-    $uninstalled_modules = \Drupal::state()->get('module_test.uninstall_order') ?: [];
-    $this->assertTrue(in_array($dependency, $uninstalled_modules), "$dependency module is in the list of uninstalled modules.");
-    $this->assertFalse(in_array($profile, $uninstalled_modules), 'The installation profile is not in the list of uninstalled modules.');
+    $enable_order = \Drupal::state()->get('module_test.install_order', []);
+    $this->assertSame(['help', 'config', 'dblog'], $enable_order);
   }
 
   /**
    * Tests uninstalling a module installed by a profile.
    */
-  public function testUninstallProfileDependency() {
+  public function testUninstallProfileDependency(): void {
     $profile = 'testing_install_profile_dependencies';
     $dependency = 'dblog';
     $non_dependency = 'ban';
     $this->setInstallProfile($profile);
-    // Prime the drupal_get_filename() static cache with the location of the
-    // testing_install_profile_dependencies profile as it is not the currently
-    // active profile and we don't yet have any cached way to retrieve its
-    // location.
+    // Prime the \Drupal\Core\Extension\ExtensionList::getPathname() static
+    // cache with the location of the testing_install_profile_dependencies
+    // profile as it is not the currently active profile and we don't yet have
+    // any cached way to retrieve its location.
     // @todo Remove as part of https://www.drupal.org/node/2186491
-    drupal_get_filename('profile', $profile, 'core/profiles/' . $profile . '/' . $profile . '.info.yml');
+    $profile_list = \Drupal::service('extension.list.profile');
+    assert($profile_list instanceof ProfileExtensionList);
+    $profile_list->setPathname($profile, 'core/profiles/' . $profile . '/' . $profile . '.info.yml');
     $this->enableModules(['module_test', $profile]);
 
     $data = \Drupal::service('extension.list.module')->reset()->getList();
@@ -214,35 +190,55 @@ class ModuleHandlerTest extends KernelTestBase {
     $this->assertTrue($this->moduleHandler()->moduleExists($dependency));
 
     // Uninstall the profile module that is not a dependent.
+    /** @var \Drupal\Core\Update\UpdateHookRegistry $update_registry */
+    $update_registry = \Drupal::service('update.update_hook_registry');
     $result = $this->moduleInstaller()->uninstall([$non_dependency]);
     $this->assertTrue($result, 'ModuleInstaller::uninstall() returns TRUE.');
     $this->assertFalse($this->moduleHandler()->moduleExists($non_dependency));
-    $this->assertEquals(drupal_get_installed_schema_version($non_dependency), SCHEMA_UNINSTALLED, "$non_dependency module was uninstalled.");
+    $this->assertEquals($update_registry::SCHEMA_UNINSTALLED, $update_registry->getInstalledVersion($non_dependency), "$non_dependency module was uninstalled.");
 
     // Verify that the installation profile itself was not uninstalled.
-    $uninstalled_modules = \Drupal::state()->get('module_test.uninstall_order') ?: [];
+    $uninstalled_modules = \Drupal::state()->get('module_test.uninstall_order', []);
     $this->assertContains($non_dependency, $uninstalled_modules, "$non_dependency module is in the list of uninstalled modules.");
     $this->assertNotContains($profile, $uninstalled_modules, 'The installation profile is not in the list of uninstalled modules.');
 
     // Try uninstalling the required module.
-    $this->setExpectedException(ModuleUninstallValidatorException::class, 'The following reasons prevent the modules from being uninstalled: The Testing install profile dependencies module is required');
-    $this->moduleInstaller()->uninstall([$dependency]);
+    try {
+      $this->moduleInstaller()->uninstall([$dependency]);
+      $this->fail('Expected ModuleUninstallValidatorException not thrown');
+    }
+    catch (ModuleUninstallValidatorException $e) {
+      $this->assertEquals("The following reasons prevent the modules from being uninstalled: The 'Testing install profile dependencies' install profile requires 'Database Logging'", $e->getMessage());
+    }
+
+    // Try uninstalling the install profile.
+    $this->assertSame('testing_install_profile_dependencies', $this->container->getParameter('install_profile'));
+    $result = $this->moduleInstaller()->uninstall([$profile]);
+    $this->assertTrue($result, 'ModuleInstaller::uninstall() returns TRUE.');
+    $this->assertFalse($this->moduleHandler()->moduleExists($profile));
+    $this->assertFalse($this->container->getParameter('install_profile'));
+
+    // Try uninstalling the required module again.
+    $result = $this->moduleInstaller()->uninstall([$dependency]);
+    $this->assertTrue($result, 'ModuleInstaller::uninstall() returns TRUE.');
+    $this->assertFalse($this->moduleHandler()->moduleExists($dependency));
   }
 
   /**
-   * Tests that a profile can supply only real dependencies
+   * Tests that a profile can supply only real dependencies.
    */
-  public function testProfileAllDependencies() {
+  public function testProfileAllDependencies(): void {
     $profile = 'testing_install_profile_all_dependencies';
     $dependencies = ['dblog', 'ban'];
-
     $this->setInstallProfile($profile);
-    // Prime the drupal_get_filename() static cache with the location of the
-    // testing_install_profile_dependencies profile as it is not the currently
-    // active profile and we don't yet have any cached way to retrieve its
-    // location.
+    // Prime the \Drupal\Core\Extension\ExtensionList::getPathname() static
+    // cache with the location of the testing_install_profile_dependencies
+    // profile as it is not the currently active profile and we don't yet have
+    // any cached way to retrieve its location.
     // @todo Remove as part of https://www.drupal.org/node/2186491
-    drupal_get_filename('profile', $profile, 'core/profiles/' . $profile . '/' . $profile . '.info.yml');
+    $profile_list = \Drupal::service('extension.list.profile');
+    assert($profile_list instanceof ProfileExtensionList);
+    $profile_list->setPathname($profile, 'core/profiles/' . $profile . '/' . $profile . '.info.yml');
     $this->enableModules(['module_test', $profile]);
 
     $data = \Drupal::service('extension.list.module')->reset()->getList();
@@ -256,22 +252,23 @@ class ModuleHandlerTest extends KernelTestBase {
     }
 
     // Try uninstalling the dependencies.
-    $this->setExpectedException(ModuleUninstallValidatorException::class, 'The following reasons prevent the modules from being uninstalled: The Testing install profile all dependencies module is required');
+    $this->expectException(ModuleUninstallValidatorException::class);
+    $this->expectExceptionMessage("The following reasons prevent the modules from being uninstalled: The 'Testing install profile all dependencies' install profile requires 'Database Logging'; The 'Testing install profile all dependencies' install profile requires 'Ban'");
     $this->moduleInstaller()->uninstall($dependencies);
   }
 
   /**
    * Tests uninstalling a module that has content.
    */
-  public function testUninstallContentDependency() {
+  public function testUninstallContentDependency(): void {
     $this->enableModules(['module_test', 'entity_test', 'text', 'user', 'help']);
     $this->assertTrue($this->moduleHandler()->moduleExists('entity_test'), 'Test module is enabled.');
     $this->assertTrue($this->moduleHandler()->moduleExists('module_test'), 'Test module is enabled.');
 
     $this->installSchema('user', 'users_data');
-    $entity_types = \Drupal::entityManager()->getDefinitions();
+    $entity_types = \Drupal::entityTypeManager()->getDefinitions();
     foreach ($entity_types as $entity_type) {
-      if ('entity_test' == $entity_type->getProvider()) {
+      if ($entity_type instanceof ContentEntityTypeInterface && 'entity_test' == $entity_type->getProvider()) {
         $this->installEntitySchema($entity_type->id());
       }
     }
@@ -280,7 +277,6 @@ class ModuleHandlerTest extends KernelTestBase {
     // entity_test will depend on help. This way help can not be uninstalled
     // when there is test content preventing entity_test from being uninstalled.
     \Drupal::state()->set('module_test.dependency', 'dependency');
-    \Drupal::service('extension.list.module')->reset();
 
     // Create an entity so that the modules can not be disabled.
     $entity = EntityTest::create(['name' => $this->randomString()]);
@@ -293,7 +289,7 @@ class ModuleHandlerTest extends KernelTestBase {
       $this->fail($message);
     }
     catch (ModuleUninstallValidatorException $e) {
-      $this->pass(get_class($e) . ': ' . $e->getMessage());
+      // Expected exception; just continue testing.
     }
 
     // Uninstalling help needs entity_test to be un-installable.
@@ -303,35 +299,38 @@ class ModuleHandlerTest extends KernelTestBase {
       $this->fail($message);
     }
     catch (ModuleUninstallValidatorException $e) {
-      $this->pass(get_class($e) . ': ' . $e->getMessage());
+      // Expected exception; just continue testing.
     }
 
     // Deleting the entity.
     $entity->delete();
 
+    /** @var \Drupal\Core\Update\UpdateHookRegistry $update_registry */
+    $update_registry = \Drupal::service('update.update_hook_registry');
     $result = $this->moduleInstaller()->uninstall(['help']);
     $this->assertTrue($result, 'ModuleInstaller::uninstall() returns TRUE.');
-    $this->assertEqual(drupal_get_installed_schema_version('entity_test'), SCHEMA_UNINSTALLED, "entity_test module was uninstalled.");
+    $this->assertEquals($update_registry::SCHEMA_UNINSTALLED, $update_registry->getInstalledVersion('entity_test'), "entity_test module was uninstalled.");
   }
 
   /**
    * Tests whether the correct module metadata is returned.
    */
-  public function testModuleMetaData() {
+  public function testModuleMetaData(): void {
     // Generate the list of available modules.
-    $modules = system_rebuild_module_data();
+    $modules = $this->container->get('extension.list.module')->getList();
     // Check that the mtime field exists for the system module.
-    $this->assertTrue(!empty($modules['system']->info['mtime']), 'The system.info.yml file modification time field is present.');
+    $this->assertNotEmpty($modules['system']->info['mtime'], 'The system.info.yml file modification time field is present.');
     // Use 0 if mtime isn't present, to avoid an array index notice.
     $test_mtime = !empty($modules['system']->info['mtime']) ? $modules['system']->info['mtime'] : 0;
     // Ensure the mtime field contains a number that is greater than zero.
-    $this->assertTrue(is_numeric($test_mtime) && ($test_mtime > 0), 'The system.info.yml file modification time field contains a timestamp.');
+    $this->assertIsNumeric($test_mtime);
+    $this->assertGreaterThan(0, $test_mtime);
   }
 
   /**
    * Tests whether module-provided stream wrappers are registered properly.
    */
-  public function testModuleStreamWrappers() {
+  public function testModuleStreamWrappers(): void {
     // file_test.module provides (among others) a 'dummy' stream wrapper.
     // Verify that it is not registered yet to prevent false positives.
     $stream_wrappers = \Drupal::service('stream_wrapper_manager')->getWrappers();
@@ -343,20 +342,23 @@ class ModuleHandlerTest extends KernelTestBase {
     file_exists('dummy://');
     $stream_wrappers = \Drupal::service('stream_wrapper_manager')->getWrappers();
     $this->assertTrue(isset($stream_wrappers['dummy']));
+    $this->assertTrue(isset($stream_wrappers['dummy1']));
+    $this->assertTrue(isset($stream_wrappers['dummy2']));
   }
 
   /**
    * Tests whether the correct theme metadata is returned.
    */
-  public function testThemeMetaData() {
+  public function testThemeMetaData(): void {
     // Generate the list of available themes.
-    $themes = \Drupal::service('theme_handler')->rebuildThemeData();
-    // Check that the mtime field exists for the bartik theme.
-    $this->assertTrue(!empty($themes['bartik']->info['mtime']), 'The bartik.info.yml file modification time field is present.');
+    $themes = \Drupal::service('extension.list.theme')->reset()->getList();
+    // Check that the mtime field exists for the olivero theme.
+    $this->assertNotEmpty($themes['olivero']->info['mtime'], 'The olivero.info.yml file modification time field is present.');
     // Use 0 if mtime isn't present, to avoid an array index notice.
-    $test_mtime = !empty($themes['bartik']->info['mtime']) ? $themes['bartik']->info['mtime'] : 0;
+    $test_mtime = !empty($themes['olivero']->info['mtime']) ? $themes['olivero']->info['mtime'] : 0;
     // Ensure the mtime field contains a number that is greater than zero.
-    $this->assertTrue(is_numeric($test_mtime) && ($test_mtime > 0), 'The bartik.info.yml file modification time field contains a timestamp.');
+    $this->assertIsNumeric($test_mtime);
+    $this->assertGreaterThan(0, $test_mtime);
   }
 
   /**
