@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\tracker\Functional;
 
 use Drupal\comment\CommentInterface;
@@ -17,6 +19,7 @@ use Drupal\Tests\system\Functional\Cache\AssertPageCacheContextsAndTagsTrait;
  * Create and delete nodes and check for their display in the tracker listings.
  *
  * @group tracker
+ * @group legacy
  */
 class TrackerTest extends BrowserTestBase {
 
@@ -55,6 +58,9 @@ class TrackerTest extends BrowserTestBase {
    */
   protected $otherUser;
 
+  /**
+   * {@inheritdoc}
+   */
   protected function setUp(): void {
     parent::setUp();
 
@@ -75,7 +81,7 @@ class TrackerTest extends BrowserTestBase {
   /**
    * Tests for the presence of nodes on the global tracker listing.
    */
-  public function testTrackerAll() {
+  public function testTrackerAll(): void {
     $this->drupalLogin($this->user);
 
     $unpublished = $this->drupalCreateNode([
@@ -88,8 +94,8 @@ class TrackerTest extends BrowserTestBase {
     ]);
 
     $this->drupalGet('activity');
-    $this->assertNoText($unpublished->label());
-    $this->assertText($published->label());
+    $this->assertSession()->pageTextNotContains($unpublished->label());
+    $this->assertSession()->pageTextContains($published->label());
     $this->assertSession()->linkExists('My recent content', 0, 'User tab shows up on the global tracker page.');
 
     // Assert cache contexts, specifically the pager and node access contexts.
@@ -97,12 +103,6 @@ class TrackerTest extends BrowserTestBase {
     // Assert cache tags for the action/tabs blocks, visible node, and node list
     // cache tag.
     $expected_tags = Cache::mergeTags($published->getCacheTags(), $published->getOwner()->getCacheTags());
-    // Because the 'user.permissions' cache context is being optimized away.
-    $role_tags = [];
-    foreach ($this->user->getRoles() as $rid) {
-      $role_tags[] = "config:user.role.$rid";
-    }
-    $expected_tags = Cache::mergeTags($expected_tags, $role_tags);
     $block_tags = [
       'block_view',
       'local_task',
@@ -121,7 +121,7 @@ class TrackerTest extends BrowserTestBase {
     // Delete a node and ensure it no longer appears on the tracker.
     $published->delete();
     $this->drupalGet('activity');
-    $this->assertNoText($published->label());
+    $this->assertSession()->pageTextNotContains($published->label());
 
     // Test proper display of time on activity page when comments are disabled.
     // Disable comments.
@@ -135,14 +135,14 @@ class TrackerTest extends BrowserTestBase {
     ]);
 
     $this->drupalGet('activity');
-    $this->assertText($node->label());
-    $this->assertText(\Drupal::service('date.formatter')->formatTimeDiffSince($node->getChangedTime()));
+    $this->assertSession()->pageTextContains($node->label());
+    $this->assertSession()->pageTextContains(\Drupal::service('date.formatter')->formatTimeDiffSince($node->getChangedTime()));
   }
 
   /**
    * Tests for the presence of nodes on a user's tracker listing.
    */
-  public function testTrackerUser() {
+  public function testTrackerUser(): void {
     $this->drupalLogin($this->user);
 
     $unpublished = $this->drupalCreateNode([
@@ -169,13 +169,14 @@ class TrackerTest extends BrowserTestBase {
       'subject[0][value]' => $this->randomMachineName(),
       'comment_body[0][value]' => $this->randomMachineName(20),
     ];
-    $this->drupalPostForm('comment/reply/node/' . $other_published_my_comment->id() . '/comment', $comment, 'Save');
+    $this->drupalGet('comment/reply/node/' . $other_published_my_comment->id() . '/comment');
+    $this->submitForm($comment, 'Save');
 
     $this->drupalGet('user/' . $this->user->id() . '/activity');
-    $this->assertNoText($unpublished->label());
-    $this->assertText($my_published->label());
-    $this->assertNoText($other_published_no_comment->label());
-    $this->assertText($other_published_my_comment->label());
+    $this->assertSession()->pageTextNotContains($unpublished->label());
+    $this->assertSession()->pageTextContains($my_published->label());
+    $this->assertSession()->pageTextNotContains($other_published_no_comment->label());
+    $this->assertSession()->pageTextContains($other_published_my_comment->label());
 
     // Assert cache contexts.
     $this->assertCacheContexts(['languages:language_interface', 'route', 'theme', 'url.query_args:' . MainContentViewSubscriber::WRAPPER_FORMAT, 'url.query_args.pagers:0', 'user', 'user.node_grants:view']);
@@ -184,12 +185,6 @@ class TrackerTest extends BrowserTestBase {
     $expected_tags = Cache::mergeTags($my_published->getCacheTags(), $my_published->getOwner()->getCacheTags());
     $expected_tags = Cache::mergeTags($expected_tags, $other_published_my_comment->getCacheTags());
     $expected_tags = Cache::mergeTags($expected_tags, $other_published_my_comment->getOwner()->getCacheTags());
-    // Because the 'user.permissions' cache context is being optimized away.
-    $role_tags = [];
-    foreach ($this->user->getRoles() as $rid) {
-      $role_tags[] = "config:user.role.$rid";
-    }
-    $expected_tags = Cache::mergeTags($expected_tags, $role_tags);
     $block_tags = [
       'block_view',
       'local_task',
@@ -210,7 +205,7 @@ class TrackerTest extends BrowserTestBase {
     $this->assertSession()->linkExists($my_published->label());
     $this->assertSession()->linkNotExists($unpublished->label());
     // Verify that title and tab title have been set correctly.
-    $this->assertText('Activity');
+    $this->assertSession()->pageTextContains('Activity');
     $this->assertSession()->titleEquals($this->user->getAccountName() . ' | Drupal');
 
     // Verify that unpublished comments are removed from the tracker.
@@ -220,9 +215,10 @@ class TrackerTest extends BrowserTestBase {
       'access user profiles',
     ]);
     $this->drupalLogin($admin_user);
-    $this->drupalPostForm('comment/1/edit', ['status' => CommentInterface::NOT_PUBLISHED], 'Save');
+    $this->drupalGet('comment/1/edit');
+    $this->submitForm(['status' => CommentInterface::NOT_PUBLISHED], 'Save');
     $this->drupalGet('user/' . $this->user->id() . '/activity');
-    $this->assertNoText($other_published_my_comment->label());
+    $this->assertSession()->pageTextNotContains($other_published_my_comment->label());
 
     // Test escaping of title on user's tracker tab.
     \Drupal::service('module_installer')->install(['user_hooks_test']);
@@ -235,13 +231,13 @@ class TrackerTest extends BrowserTestBase {
     Cache::invalidateTags(['rendered']);
     $this->drupalGet('user/' . $this->user->id() . '/activity');
     $this->assertSession()->assertNoEscaped('<em>' . $this->user->id() . '</em>');
-    $this->assertRaw('<em>' . $this->user->id() . '</em>');
+    $this->assertSession()->responseContains('<em>' . $this->user->id() . '</em>');
   }
 
   /**
    * Tests the metadata for the "new"/"updated" indicators.
    */
-  public function testTrackerHistoryMetadata() {
+  public function testTrackerHistoryMetadata(): void {
     $this->drupalLogin($this->user);
 
     // Create a page node.
@@ -266,7 +262,8 @@ class TrackerTest extends BrowserTestBase {
       'comment_body[0][value]' => $this->randomMachineName(20),
     ];
     sleep(1);
-    $this->drupalPostForm('comment/reply/node/' . $node->id() . '/comment', $comment, 'Save');
+    $this->drupalGet('comment/reply/node/' . $node->id() . '/comment');
+    $this->submitForm($comment, 'Save');
     // Reload the node so that comment.module's hook_node_load()
     // implementation can set $node->last_comment_timestamp for the freshly
     // posted comment.
@@ -274,25 +271,25 @@ class TrackerTest extends BrowserTestBase {
 
     // Verify that the history metadata is updated.
     $this->drupalGet('activity');
-    $this->assertHistoryMetadata($node->id(), $node->getChangedTime(), $node->get('comment')->last_comment_timestamp);
+    $this->assertHistoryMetadata($node->id(), $node->getChangedTime(), (int) $node->get('comment')->last_comment_timestamp);
     $this->drupalGet('activity/' . $this->user->id());
-    $this->assertHistoryMetadata($node->id(), $node->getChangedTime(), $node->get('comment')->last_comment_timestamp);
+    $this->assertHistoryMetadata($node->id(), $node->getChangedTime(), (int) $node->get('comment')->last_comment_timestamp);
     $this->drupalGet('user/' . $this->user->id() . '/activity');
-    $this->assertHistoryMetadata($node->id(), $node->getChangedTime(), $node->get('comment')->last_comment_timestamp);
+    $this->assertHistoryMetadata($node->id(), $node->getChangedTime(), (int) $node->get('comment')->last_comment_timestamp);
 
     // Log out, now verify that the metadata is still there, but the library is
     // not.
     $this->drupalLogout();
     $this->drupalGet('activity');
-    $this->assertHistoryMetadata($node->id(), $node->getChangedTime(), $node->get('comment')->last_comment_timestamp, FALSE);
+    $this->assertHistoryMetadata($node->id(), $node->getChangedTime(), (int) $node->get('comment')->last_comment_timestamp, FALSE);
     $this->drupalGet('user/' . $this->user->id() . '/activity');
-    $this->assertHistoryMetadata($node->id(), $node->getChangedTime(), $node->get('comment')->last_comment_timestamp, FALSE);
+    $this->assertHistoryMetadata($node->id(), $node->getChangedTime(), (int) $node->get('comment')->last_comment_timestamp, FALSE);
   }
 
   /**
    * Tests for ordering on a users tracker listing when comments are posted.
    */
-  public function testTrackerOrderingNewComments() {
+  public function testTrackerOrderingNewComments(): void {
     $this->drupalLogin($this->user);
 
     $node_one = $this->drupalCreateNode([
@@ -311,7 +308,8 @@ class TrackerTest extends BrowserTestBase {
       'subject[0][value]' => $this->randomMachineName(),
       'comment_body[0][value]' => $this->randomMachineName(20),
     ];
-    $this->drupalPostForm('comment/reply/node/' . $node_one->id() . '/comment', $comment, 'Save');
+    $this->drupalGet('comment/reply/node/' . $node_one->id() . '/comment');
+    $this->submitForm($comment, 'Save');
 
     // If the comment is posted in the same second as the last one then Drupal
     // can't tell the difference, so we wait one second here.
@@ -322,7 +320,8 @@ class TrackerTest extends BrowserTestBase {
       'subject[0][value]' => $this->randomMachineName(),
       'comment_body[0][value]' => $this->randomMachineName(20),
     ];
-    $this->drupalPostForm('comment/reply/node/' . $node_two->id() . '/comment', $comment, 'Save');
+    $this->drupalGet('comment/reply/node/' . $node_two->id() . '/comment');
+    $this->submitForm($comment, 'Save');
 
     // We should at this point have in our tracker for otherUser:
     // 1. node_two
@@ -342,7 +341,8 @@ class TrackerTest extends BrowserTestBase {
       'subject[0][value]' => $this->randomMachineName(),
       'comment_body[0][value]' => $this->randomMachineName(20),
     ];
-    $this->drupalPostForm('comment/reply/node/' . $node_one->id() . '/comment', $comment, 'Save');
+    $this->drupalGet('comment/reply/node/' . $node_one->id() . '/comment');
+    $this->submitForm($comment, 'Save');
 
     // Switch back to the otherUser and assert that the order has swapped.
     $this->drupalLogin($this->otherUser);
@@ -351,7 +351,6 @@ class TrackerTest extends BrowserTestBase {
     // on the tracker page.
     // It's almost certainly too brittle.
     $pattern = '/' . preg_quote($node_one->getTitle()) . '.+' . preg_quote($node_two->getTitle()) . '/s';
-    $this->verbose($pattern);
     // Verify that the most recent comment on node appears at the top of
     // tracker.
     $this->assertSession()->responseMatches($pattern);
@@ -360,7 +359,7 @@ class TrackerTest extends BrowserTestBase {
   /**
    * Tests that existing nodes are indexed by cron.
    */
-  public function testTrackerCronIndexing() {
+  public function testTrackerCronIndexing(): void {
     $this->drupalLogin($this->user);
 
     // Create 3 nodes.
@@ -379,7 +378,8 @@ class TrackerTest extends BrowserTestBase {
       'subject[0][value]' => $this->randomMachineName(),
       'comment_body[0][value]' => $this->randomMachineName(20),
     ];
-    $this->drupalPostForm('comment/reply/node/' . $nodes[3]->id() . '/comment', $comment, 'Save');
+    $this->drupalGet('comment/reply/node/' . $nodes[3]->id() . '/comment');
+    $this->submitForm($comment, 'Save');
 
     // Create an unpublished node.
     $unpublished = $this->drupalCreateNode([
@@ -387,6 +387,9 @@ class TrackerTest extends BrowserTestBase {
       'status' => 0,
     ]);
 
+    $this->drupalGet('activity');
+    $this->assertSession()->responseNotContains($unpublished->label());
+
     // Start indexing backwards from node 4.
     \Drupal::state()->set('tracker.index_nid', 4);
 
@@ -405,7 +408,7 @@ class TrackerTest extends BrowserTestBase {
 
     // Assert that all node titles are displayed.
     foreach ($nodes as $i => $node) {
-      $this->assertText($node->label());
+      $this->assertSession()->pageTextContains($node->label());
     }
 
     // Fetch the site-wide tracker.
@@ -413,14 +416,14 @@ class TrackerTest extends BrowserTestBase {
 
     // Assert that all node titles are displayed.
     foreach ($nodes as $i => $node) {
-      $this->assertText($node->label());
+      $this->assertSession()->pageTextContains($node->label());
     }
   }
 
   /**
    * Tests that publish/unpublish works at admin/content/node.
    */
-  public function testTrackerAdminUnpublish() {
+  public function testTrackerAdminUnpublish(): void {
     \Drupal::service('module_installer')->install(['views']);
     $admin_user = $this->drupalCreateUser([
       'access content overview',
@@ -435,17 +438,18 @@ class TrackerTest extends BrowserTestBase {
 
     // Assert that the node is displayed.
     $this->drupalGet('activity');
-    $this->assertText($node->label());
+    $this->assertSession()->pageTextContains($node->label());
 
     // Unpublish the node and ensure that it's no longer displayed.
     $edit = [
       'action' => 'node_unpublish_action',
       'node_bulk_form[0]' => $node->id(),
     ];
-    $this->drupalPostForm('admin/content', $edit, 'Apply to selected items');
+    $this->drupalGet('admin/content');
+    $this->submitForm($edit, 'Apply to selected items');
 
     $this->drupalGet('activity');
-    $this->assertText('No content available.');
+    $this->assertSession()->pageTextContains('No content available.');
   }
 
   /**
@@ -457,7 +461,7 @@ class TrackerTest extends BrowserTestBase {
    * indicators, as well as the "x new" replies link to the tracker.
    * We do this in JavaScript to prevent breaking the render cache.
    *
-   * @param int $node_id
+   * @param string|int $node_id
    *   A node ID, that must exist as a data-history-node-id attribute
    * @param int $node_timestamp
    *   A node timestamp, that must exist as a data-history-node-timestamp
@@ -467,12 +471,14 @@ class TrackerTest extends BrowserTestBase {
    *   data-history-node-last-comment-timestamp attribute.
    * @param bool $library_is_present
    *   Whether the drupal.tracker-history library should be present or not.
+   *
+   * @internal
    */
-  public function assertHistoryMetadata($node_id, $node_timestamp, $node_last_comment_timestamp, $library_is_present = TRUE) {
+  public function assertHistoryMetadata(string|int $node_id, int $node_timestamp, int $node_last_comment_timestamp, bool $library_is_present = TRUE): void {
     $settings = $this->getDrupalSettings();
     $this->assertSame($library_is_present, isset($settings['ajaxPageState']) && in_array('tracker/history', explode(',', $settings['ajaxPageState']['libraries'])), 'drupal.tracker-history library is present.');
-    $this->assertCount(1, $this->xpath('//table/tbody/tr/td[@data-history-node-id="' . $node_id . '" and @data-history-node-timestamp="' . $node_timestamp . '"]'), 'Tracker table cell contains the data-history-node-id and data-history-node-timestamp attributes for the node.');
-    $this->assertCount(1, $this->xpath('//table/tbody/tr/td[@data-history-node-last-comment-timestamp="' . $node_last_comment_timestamp . '"]'), 'Tracker table cell contains the data-history-node-last-comment-timestamp attribute for the node.');
+    $this->assertSession()->elementsCount('xpath', '//table/tbody/tr/td[@data-history-node-id="' . $node_id . '" and @data-history-node-timestamp="' . $node_timestamp . '"]', 1);
+    $this->assertSession()->elementsCount('xpath', '//table/tbody/tr/td[@data-history-node-last-comment-timestamp="' . $node_last_comment_timestamp . '"]', 1);
   }
 
 }
