@@ -1,8 +1,9 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\user\Functional;
 
-use Drupal\Component\Render\FormattableMarkup;
 use Drupal\Core\Entity\Entity\EntityFormDisplay;
 use Drupal\Core\Field\FieldStorageDefinitionInterface;
 use Drupal\field\Entity\FieldConfig;
@@ -14,33 +15,39 @@ use Drupal\user\UserInterface;
  * Tests registration of user under different configurations.
  *
  * @group user
+ * @group #slow
  */
 class UserRegistrationTest extends BrowserTestBase {
 
   /**
-   * Modules to enable.
-   *
-   * @var array
+   * {@inheritdoc}
    */
-  public static $modules = ['field_test'];
+  protected static $modules = ['field_test'];
 
-  public function testRegistrationWithEmailVerification() {
+  /**
+   * {@inheritdoc}
+   */
+  protected $defaultTheme = 'stark';
+
+  public function testRegistrationWithEmailVerification(): void {
     $config = $this->config('user.settings');
     // Require email verification.
     $config->set('verify_mail', TRUE)->save();
 
-    // Set registration to administrator only.
+    // Set registration to administrator only and ensure the user registration
+    // page is inaccessible.
     $config->set('register', UserInterface::REGISTER_ADMINISTRATORS_ONLY)->save();
     $this->drupalGet('user/register');
-    $this->assertResponse(403, 'Registration page is inaccessible when only administrators can create accounts.');
+    $this->assertSession()->statusCodeEquals(403);
 
     // Allow registration by site visitors without administrator approval.
     $config->set('register', UserInterface::REGISTER_VISITORS)->save();
     $edit = [];
     $edit['name'] = $name = $this->randomMachineName();
     $edit['mail'] = $mail = $edit['name'] . '@example.com';
-    $this->drupalPostForm('user/register', $edit, t('Create new account'));
-    $this->assertText(t('A welcome message with further instructions has been sent to your email address.'), 'User registered successfully.');
+    $this->drupalGet('user/register');
+    $this->submitForm($edit, 'Create new account');
+    $this->assertSession()->pageTextContains('A welcome message with further instructions has been sent to your email address.');
 
     /** @var EntityStorageInterface $storage */
     $storage = $this->container->get('entity_type.manager')->getStorage('user');
@@ -49,21 +56,22 @@ class UserRegistrationTest extends BrowserTestBase {
     $this->assertTrue($new_user->isActive(), 'New account is active after registration.');
     $resetURL = user_pass_reset_url($new_user);
     $this->drupalGet($resetURL);
-    $this->assertTitle(t('Set password | Drupal'), 'Page title is "Set password".');
+    $this->assertSession()->titleEquals('Set password | Drupal');
 
     // Allow registration by site visitors, but require administrator approval.
     $config->set('register', UserInterface::REGISTER_VISITORS_ADMINISTRATIVE_APPROVAL)->save();
     $edit = [];
     $edit['name'] = $name = $this->randomMachineName();
     $edit['mail'] = $mail = $edit['name'] . '@example.com';
-    $this->drupalPostForm('user/register', $edit, t('Create new account'));
-    $this->container->get('entity.manager')->getStorage('user')->resetCache();
+    $this->drupalGet('user/register');
+    $this->submitForm($edit, 'Create new account');
+    $this->container->get('entity_type.manager')->getStorage('user')->resetCache();
     $accounts = $storage->loadByProperties(['name' => $name, 'mail' => $mail]);
     $new_user = reset($accounts);
     $this->assertFalse($new_user->isActive(), 'New account is blocked until approved by an administrator.');
   }
 
-  public function testRegistrationWithoutEmailVerification() {
+  public function testRegistrationWithoutEmailVerification(): void {
     $config = $this->config('user.settings');
     // Don't require email verification and allow registration by site visitors
     // without administrator approval.
@@ -79,19 +87,21 @@ class UserRegistrationTest extends BrowserTestBase {
     // Try entering a mismatching password.
     $edit['pass[pass1]'] = '99999.0';
     $edit['pass[pass2]'] = '99999';
-    $this->drupalPostForm('user/register', $edit, t('Create new account'));
-    $this->assertText(t('The specified passwords do not match.'), 'Typing mismatched passwords displays an error message.');
+    $this->drupalGet('user/register');
+    $this->submitForm($edit, 'Create new account');
+    $this->assertSession()->pageTextContains('The specified passwords do not match.');
 
     // Enter a correct password.
     $edit['pass[pass1]'] = $new_pass = $this->randomMachineName();
     $edit['pass[pass2]'] = $new_pass;
-    $this->drupalPostForm('user/register', $edit, t('Create new account'));
-    $this->container->get('entity.manager')->getStorage('user')->resetCache();
+    $this->drupalGet('user/register');
+    $this->submitForm($edit, 'Create new account');
+    $this->container->get('entity_type.manager')->getStorage('user')->resetCache();
     $accounts = $this->container->get('entity_type.manager')->getStorage('user')
       ->loadByProperties(['name' => $name, 'mail' => $mail]);
     $new_user = reset($accounts);
     $this->assertNotNull($new_user, 'New account successfully created with matching passwords.');
-    $this->assertText(t('Registration successful. You are now logged in.'), 'Users are logged in after registering.');
+    $this->assertSession()->pageTextContains('Registration successful. You are now logged in.');
     $this->drupalLogout();
 
     // Allow registration by site visitors, but require administrator approval.
@@ -101,16 +111,18 @@ class UserRegistrationTest extends BrowserTestBase {
     $edit['mail'] = $mail = $edit['name'] . '@example.com';
     $edit['pass[pass1]'] = $pass = $this->randomMachineName();
     $edit['pass[pass2]'] = $pass;
-    $this->drupalPostForm('user/register', $edit, t('Create new account'));
-    $this->assertText(t('Thank you for applying for an account. Your account is currently pending approval by the site administrator.'), 'Users are notified of pending approval');
+    $this->drupalGet('user/register');
+    $this->submitForm($edit, 'Create new account');
+    $this->assertSession()->pageTextContains('Thank you for applying for an account. Your account is currently pending approval by the site administrator.');
 
     // Try to log in before administrator approval.
     $auth = [
       'name' => $name,
       'pass' => $pass,
     ];
-    $this->drupalPostForm('user/login', $auth, t('Log in'));
-    $this->assertText(t('The username @name has not been activated or is blocked.', ['@name' => $name]), 'User cannot log in yet.');
+    $this->drupalGet('user/login');
+    $this->submitForm($auth, 'Log in');
+    $this->assertSession()->pageTextContains('The username ' . $name . ' has not been activated or is blocked.');
 
     // Activate the new account.
     $accounts = $this->container->get('entity_type.manager')->getStorage('user')
@@ -121,15 +133,17 @@ class UserRegistrationTest extends BrowserTestBase {
     $edit = [
       'status' => 1,
     ];
-    $this->drupalPostForm('user/' . $new_user->id() . '/edit', $edit, t('Save'));
+    $this->drupalGet('user/' . $new_user->id() . '/edit');
+    $this->submitForm($edit, 'Save');
     $this->drupalLogout();
 
     // Log in after administrator approval.
-    $this->drupalPostForm('user/login', $auth, t('Log in'));
-    $this->assertText(t('Member for'), 'User can log in after administrator approval.');
+    $this->drupalGet('user/login');
+    $this->submitForm($auth, 'Log in');
+    $this->assertSession()->pageTextContains('Member for');
   }
 
-  public function testRegistrationEmailDuplicates() {
+  public function testRegistrationEmailDuplicates(): void {
     // Don't require email verification and allow registration by site visitors
     // without administrator approval.
     $this->config('user.settings')
@@ -145,14 +159,16 @@ class UserRegistrationTest extends BrowserTestBase {
     $edit['mail'] = $duplicate_user->getEmail();
 
     // Attempt to create a new account using an existing email address.
-    $this->drupalPostForm('user/register', $edit, t('Create new account'));
-    $this->assertText(t('The email address @email is already taken.', ['@email' => $duplicate_user->getEmail()]), 'Supplying an exact duplicate email address displays an error message');
+    $this->drupalGet('user/register');
+    $this->submitForm($edit, 'Create new account');
+    $this->assertSession()->pageTextContains('The email address ' . $duplicate_user->getEmail() . ' is already taken.');
 
     // Attempt to bypass duplicate email registration validation by adding spaces.
     $edit['mail'] = '   ' . $duplicate_user->getEmail() . '   ';
 
-    $this->drupalPostForm('user/register', $edit, t('Create new account'));
-    $this->assertText(t('The email address @email is already taken.', ['@email' => $duplicate_user->getEmail()]), 'Supplying a duplicate email address with added whitespace displays an error message');
+    $this->drupalGet('user/register');
+    $this->submitForm($edit, 'Create new account');
+    $this->assertSession()->pageTextContains('The email address ' . $duplicate_user->getEmail() . ' is already taken.');
   }
 
   /**
@@ -161,9 +177,8 @@ class UserRegistrationTest extends BrowserTestBase {
    * This is a regression test for https://www.drupal.org/node/2500527 to ensure
    * that the form is not cached on GET requests.
    */
-  public function testUuidFormState() {
+  public function testUuidFormState(): void {
     \Drupal::service('module_installer')->install(['image']);
-    \Drupal::service('router.builder')->rebuild();
 
     // Add a picture field in order to ensure that no form cache is written,
     // which breaks registration of more than 1 user every 6 hours.
@@ -205,12 +220,13 @@ class UserRegistrationTest extends BrowserTestBase {
     $edit['pass[pass2]'] = $edit['pass[pass1]'] = $this->randomMachineName();
 
     // Create one account.
-    $this->drupalPostForm('user/register', $edit, t('Create new account'));
-    $this->assertResponse(200);
+    $this->drupalGet('user/register');
+    $this->submitForm($edit, 'Create new account');
+    $this->assertSession()->statusCodeEquals(200);
 
-    $user_storage = \Drupal::entityManager()->getStorage('user');
+    $user_storage = \Drupal::entityTypeManager()->getStorage('user');
 
-    $this->assertTrue($user_storage->loadByProperties(['name' => $edit['name']]));
+    $this->assertNotEmpty($user_storage->loadByProperties(['name' => $edit['name']]));
     $this->drupalLogout();
 
     // Create a second account.
@@ -218,13 +234,14 @@ class UserRegistrationTest extends BrowserTestBase {
     $edit['mail'] = $edit['name'] . '@example.com';
     $edit['pass[pass2]'] = $edit['pass[pass1]'] = $this->randomMachineName();
 
-    $this->drupalPostForm('user/register', $edit, t('Create new account'));
-    $this->assertResponse(200);
+    $this->drupalGet('user/register');
+    $this->submitForm($edit, 'Create new account');
+    $this->assertSession()->statusCodeEquals(200);
 
-    $this->assertTrue($user_storage->loadByProperties(['name' => $edit['name']]));
+    $this->assertNotEmpty($user_storage->loadByProperties(['name' => $edit['name']]));
   }
 
-  public function testRegistrationDefaultValues() {
+  public function testRegistrationDefaultValues(): void {
     // Don't require email verification and allow registration by site visitors
     // without administrator approval.
     $config_user_settings = $this->config('user.settings')
@@ -240,27 +257,28 @@ class UserRegistrationTest extends BrowserTestBase {
 
     // Check the presence of expected cache tags.
     $this->drupalGet('user/register');
-    $this->assertCacheTag('config:user.settings');
+    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', 'config:user.settings');
 
     $edit = [];
     $edit['name'] = $name = $this->randomMachineName();
     $edit['mail'] = $mail = $edit['name'] . '@example.com';
     $edit['pass[pass1]'] = $new_pass = $this->randomMachineName();
     $edit['pass[pass2]'] = $new_pass;
-    $this->drupalPostForm(NULL, $edit, t('Create new account'));
+    $this->submitForm($edit, 'Create new account');
 
     // Check user fields.
     $accounts = $this->container->get('entity_type.manager')->getStorage('user')
       ->loadByProperties(['name' => $name, 'mail' => $mail]);
     $new_user = reset($accounts);
-    $this->assertEqual($new_user->getAccountName(), $name, 'Username matches.');
-    $this->assertEqual($new_user->getEmail(), $mail, 'Email address matches.');
-    $this->assertTrue(($new_user->getCreatedTime() > REQUEST_TIME - 20), 'Correct creation time.');
-    $this->assertEqual($new_user->isActive(), $config_user_settings->get('register') == UserInterface::REGISTER_VISITORS ? 1 : 0, 'Correct status field.');
-    $this->assertEqual($new_user->getTimezone(), $config_system_date->get('timezone.default'), 'Correct time zone field.');
-    $this->assertEqual($new_user->langcode->value, \Drupal::languageManager()->getDefaultLanguage()->getId(), 'Correct language field.');
-    $this->assertEqual($new_user->preferred_langcode->value, \Drupal::languageManager()->getDefaultLanguage()->getId(), 'Correct preferred language field.');
-    $this->assertEqual($new_user->init->value, $mail, 'Correct init field.');
+    $this->assertEquals($name, $new_user->getAccountName(), 'Username matches.');
+    $this->assertEquals($mail, $new_user->getEmail(), 'Email address matches.');
+    // Verify that the creation time is correct.
+    $this->assertGreaterThan(\Drupal::time()->getRequestTime() - 20, $new_user->getCreatedTime());
+    $this->assertEquals($config_user_settings->get('register') == UserInterface::REGISTER_VISITORS ? 1 : 0, $new_user->isActive(), 'Correct status field.');
+    $this->assertEquals($config_system_date->get('timezone.default'), $new_user->getTimezone(), 'Correct time zone field.');
+    $this->assertEquals(\Drupal::languageManager()->getDefaultLanguage()->getId(), $new_user->langcode->value, 'Correct language field.');
+    $this->assertEquals(\Drupal::languageManager()->getDefaultLanguage()->getId(), $new_user->preferred_langcode->value, 'Correct preferred language field.');
+    $this->assertEquals($mail, $new_user->init->value, 'Correct init field.');
   }
 
   /**
@@ -269,22 +287,24 @@ class UserRegistrationTest extends BrowserTestBase {
    * @see \Drupal\user\Plugin\Validation\Constraint\UserNameUnique
    * @see \Drupal\user\Plugin\Validation\Constraint\UserMailUnique
    */
-  public function testUniqueFields() {
+  public function testUniqueFields(): void {
     $account = $this->drupalCreateUser();
 
     $edit = ['mail' => 'test@example.com', 'name' => $account->getAccountName()];
-    $this->drupalPostForm('user/register', $edit, t('Create new account'));
-    $this->assertRaw(new FormattableMarkup('The username %value is already taken.', ['%value' => $account->getAccountName()]));
+    $this->drupalGet('user/register');
+    $this->submitForm($edit, 'Create new account');
+    $this->assertSession()->pageTextContains("The username {$account->getAccountName()} is already taken.");
 
     $edit = ['mail' => $account->getEmail(), 'name' => $this->randomString()];
-    $this->drupalPostForm('user/register', $edit, t('Create new account'));
-    $this->assertRaw(new FormattableMarkup('The email address %value is already taken.', ['%value' => $account->getEmail()]));
+    $this->drupalGet('user/register');
+    $this->submitForm($edit, 'Create new account');
+    $this->assertSession()->pageTextContains("The email address {$account->getEmail()} is already taken.");
   }
 
   /**
    * Tests Field API fields on user registration forms.
    */
-  public function testRegistrationWithUserFields() {
+  public function testRegistrationWithUserFields(): void {
     // Create a field on 'user' entity type.
     $field_storage = FieldStorageConfig::create([
       'field_name' => 'test_user_field',
@@ -300,25 +320,29 @@ class UserRegistrationTest extends BrowserTestBase {
       'required' => TRUE,
     ]);
     $field->save();
-    entity_get_form_display('user', 'user', 'default')
+
+    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $display_repository */
+    $display_repository = \Drupal::service('entity_display.repository');
+
+    $display_repository->getFormDisplay('user', 'user')
       ->setComponent('test_user_field', ['type' => 'test_field_widget'])
       ->save();
-    entity_get_form_display('user', 'user', 'register')
+    $display_repository->getFormDisplay('user', 'user', 'register')
       ->save();
 
     // Check that the field does not appear on the registration form.
     $this->drupalGet('user/register');
-    $this->assertNoText($field->label(), 'The field does not appear on user registration form');
-    $this->assertCacheTag('config:core.entity_form_display.user.user.register');
-    $this->assertCacheTag('config:user.settings');
+    $this->assertSession()->pageTextNotContains($field->label());
+    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', 'config:core.entity_form_display.user.user.register');
+    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', 'config:user.settings');
 
     // Have the field appear on the registration form.
-    entity_get_form_display('user', 'user', 'register')
+    $display_repository->getFormDisplay('user', 'user', 'register')
       ->setComponent('test_user_field', ['type' => 'test_field_widget'])
       ->save();
 
     $this->drupalGet('user/register');
-    $this->assertText($field->label(), 'The field appears on user registration form');
+    $this->assertSession()->pageTextContains($field->label());
     $this->assertRegistrationFormCacheTagsWithUserFields();
 
     // Check that validation errors are correctly reported.
@@ -327,24 +351,24 @@ class UserRegistrationTest extends BrowserTestBase {
     $edit['mail'] = $mail = $edit['name'] . '@example.com';
     // Missing input in required field.
     $edit['test_user_field[0][value]'] = '';
-    $this->drupalPostForm(NULL, $edit, t('Create new account'));
+    $this->submitForm($edit, 'Create new account');
     $this->assertRegistrationFormCacheTagsWithUserFields();
-    $this->assertRaw(t('@name field is required.', ['@name' => $field->label()]), 'Field validation error was correctly reported.');
+    $this->assertSession()->pageTextContains("{$field->label()} field is required.");
     // Invalid input.
     $edit['test_user_field[0][value]'] = '-1';
-    $this->drupalPostForm(NULL, $edit, t('Create new account'));
+    $this->submitForm($edit, 'Create new account');
     $this->assertRegistrationFormCacheTagsWithUserFields();
-    $this->assertRaw(t('%name does not accept the value -1.', ['%name' => $field->label()]), 'Field validation error was correctly reported.');
+    $this->assertSession()->pageTextContains("{$field->label()} does not accept the value -1.");
 
     // Submit with valid data.
     $value = rand(1, 255);
     $edit['test_user_field[0][value]'] = $value;
-    $this->drupalPostForm(NULL, $edit, t('Create new account'));
+    $this->submitForm($edit, 'Create new account');
     // Check user fields.
     $accounts = $this->container->get('entity_type.manager')->getStorage('user')
       ->loadByProperties(['name' => $name, 'mail' => $mail]);
     $new_user = reset($accounts);
-    $this->assertEqual($new_user->test_user_field->value, $value, 'The field value was correctly saved.');
+    $this->assertEquals($value, $new_user->test_user_field->value, 'The field value was correctly saved.');
 
     // Check that the 'add more' button works.
     $field_storage->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
@@ -355,31 +379,33 @@ class UserRegistrationTest extends BrowserTestBase {
     $value = rand(1, 255);
     $edit = [];
     $edit['test_user_field[0][value]'] = $value;
-    $this->drupalPostForm(NULL, $edit, t('Add another item'));
-    $this->drupalPostForm(NULL, $edit, t('Add another item'));
+    $this->submitForm($edit, 'Add another item');
+    $this->submitForm($edit, 'Add another item');
     // Submit with three values.
     $edit['test_user_field[1][value]'] = $value + 1;
     $edit['test_user_field[2][value]'] = $value + 2;
     $edit['name'] = $name = $this->randomMachineName();
     $edit['mail'] = $mail = $edit['name'] . '@example.com';
-    $this->drupalPostForm(NULL, $edit, t('Create new account'));
+    $this->submitForm($edit, 'Create new account');
     // Check user fields.
     $accounts = $this->container->get('entity_type.manager')->getStorage('user')
       ->loadByProperties(['name' => $name, 'mail' => $mail]);
     $new_user = reset($accounts);
-    $this->assertEqual($new_user->test_user_field[0]->value, $value, 'The field value was correctly saved.');
-    $this->assertEqual($new_user->test_user_field[1]->value, $value + 1, 'The field value was correctly saved.');
-    $this->assertEqual($new_user->test_user_field[2]->value, $value + 2, 'The field value was correctly saved.');
+    $this->assertEquals($value, $new_user->test_user_field[0]->value, 'The field value was correctly saved.');
+    $this->assertEquals($value + 1, $new_user->test_user_field[1]->value, 'The field value was correctly saved.');
+    $this->assertEquals($value + 2, $new_user->test_user_field[2]->value, 'The field value was correctly saved.');
   }
 
   /**
    * Asserts the presence of cache tags on registration form with user fields.
+   *
+   * @internal
    */
-  protected function assertRegistrationFormCacheTagsWithUserFields() {
-    $this->assertCacheTag('config:core.entity_form_display.user.user.register');
-    $this->assertCacheTag('config:field.field.user.user.test_user_field');
-    $this->assertCacheTag('config:field.storage.user.test_user_field');
-    $this->assertCacheTag('config:user.settings');
+  protected function assertRegistrationFormCacheTagsWithUserFields(): void {
+    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', 'config:core.entity_form_display.user.user.register');
+    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', 'config:field.field.user.user.test_user_field');
+    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', 'config:field.storage.user.test_user_field');
+    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', 'config:user.settings');
   }
 
 }
