@@ -1,16 +1,20 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\views\Kernel\EventSubscriber;
 
+use Drupal\Core\Entity\ContentEntityTypeInterface;
 use Drupal\Core\Entity\EntityTypeEvent;
 use Drupal\Core\Entity\EntityTypeEvents;
 use Drupal\Tests\system\Functional\Entity\Traits\EntityDefinitionTestTrait;
 use Drupal\Tests\views\Kernel\ViewsKernelTestBase;
 
 /**
- * Tests \Drupal\views\EventSubscriber\ViewsEntitySchemaSubscriber
+ * Tests \Drupal\views\EventSubscriber\ViewsEntitySchemaSubscriber.
  *
  * @group Views
+ * @group #slow
  */
 class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
 
@@ -43,7 +47,7 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * The event dispatcher.
    *
-   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
+   * @var \Symfony\Contracts\EventDispatcher\EventDispatcherInterface
    */
   protected $eventDispatcher;
 
@@ -80,19 +84,19 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
     $this->entityTypeManager = $this->container->get('entity_type.manager');
     $this->state = $this->container->get('state');
 
-    $this->database = $this->container->get('database');
-
     // Install every entity type's schema that wasn't installed in the parent
     // method.
     foreach (array_diff_key($this->entityTypeManager->getDefinitions(), array_flip(['user', 'entity_test'])) as $entity_type_id => $entity_type) {
-      $this->installEntitySchema($entity_type_id);
+      if ($entity_type instanceof ContentEntityTypeInterface) {
+        $this->installEntitySchema($entity_type_id);
+      }
     }
   }
 
   /**
    * Tests that views are disabled when an entity type is deleted.
    */
-  public function testDeleteEntityType() {
+  public function testDeleteEntityType(): void {
     $entity_storage = $this->entityTypeManager->getStorage('view');
 
     // Make the test entity type revisionable.
@@ -107,7 +111,7 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
     $this->assertTrue(isset($views['test_view_entity_test_additional_base_field']));
 
     $event = new EntityTypeEvent($this->entityTypeManager->getDefinition('entity_test_update'));
-    $this->eventDispatcher->dispatch(EntityTypeEvents::DELETE, $event);
+    $this->eventDispatcher->dispatch($event, EntityTypeEvents::DELETE);
 
     // We expect that views which use 'entity_test_update' as base tables are
     // disabled.
@@ -130,7 +134,7 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * Tests that renaming base tables adapts the views.
    */
-  public function testBaseTableRename() {
+  public function testBaseTableRename(): void {
     $this->renameBaseTable();
     $this->applyEntityUpdates('entity_test_update');
 
@@ -139,10 +143,10 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
     $view = $entity_storage->load('test_view_entity_test');
 
     // Ensure the base table got renamed, so also the views fields.
-    $this->assertEqual('entity_test_update_new', $view->get('base_table'));
+    $this->assertEquals('entity_test_update_new', $view->get('base_table'));
     $display = $view->getDisplay('default');
-    $this->assertEqual('entity_test_update_new', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_new', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update_new', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_new', $display['display_options']['fields']['name']['table']);
 
     // Check that only the impacted views have been updated.
     $this->assertUpdatedViews([
@@ -155,16 +159,16 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * Tests that renaming data tables adapts the views.
    */
-  public function testDataTableRename() {
+  public function testDataTableRename(): void {
     $this->updateEntityTypeToTranslatable(TRUE);
 
     $entity_storage = $this->entityTypeManager->getStorage('view');
     $view = $entity_storage->load('test_view_entity_test_data');
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
     $display = $view->getDisplay('default');
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
     // Ensure that the data table is used.
-    $this->assertEqual('entity_test_update_data', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update_data', $display['display_options']['fields']['name']['table']);
 
     $this->renameDataTable();
     $this->applyEntityUpdates('entity_test_update');
@@ -174,10 +178,10 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
     $view = $entity_storage->load('test_view_entity_test_data');
 
     // Ensure the data table got renamed, so also the views fields.
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
     $display = $view->getDisplay('default');
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_data_new', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_data_new', $display['display_options']['fields']['name']['table']);
 
     // Check that only the impacted views have been updated.
     $this->assertUpdatedViews([
@@ -189,16 +193,16 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * Tests that renaming revision tables adapts the views.
    */
-  public function testRevisionBaseTableRename() {
+  public function testRevisionBaseTableRename(): void {
     $this->updateEntityTypeToRevisionable(TRUE);
 
     /** @var \Drupal\views\Entity\View $view */
     $entity_storage = $this->entityTypeManager->getStorage('view');
     $view = $entity_storage->load('test_view_entity_test_revision');
-    $this->assertEqual('entity_test_update_revision', $view->get('base_table'));
+    $this->assertEquals('entity_test_update_revision', $view->get('base_table'));
     $display = $view->getDisplay('default');
-    $this->assertEqual('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_revision', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_revision', $display['display_options']['fields']['name']['table']);
 
     $this->renameRevisionBaseTable();
     $this->applyEntityUpdates('entity_test_update');
@@ -208,10 +212,10 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
     $view = $entity_storage->load('test_view_entity_test_revision');
 
     // Ensure the base table got renamed, so also the views fields.
-    $this->assertEqual('entity_test_update_revision_new', $view->get('base_table'));
+    $this->assertEquals('entity_test_update_revision_new', $view->get('base_table'));
     $display = $view->getDisplay('default');
-    $this->assertEqual('entity_test_update_revision_new', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_revision_new', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update_revision_new', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_revision_new', $display['display_options']['fields']['name']['table']);
 
     // Check that only the impacted views have been updated.
     $this->assertUpdatedViews([
@@ -222,16 +226,16 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * Tests that renaming revision tables adapts the views.
    */
-  public function testRevisionDataTableRename() {
+  public function testRevisionDataTableRename(): void {
     $this->updateEntityTypeToRevisionableAndTranslatable(TRUE);
 
     /** @var \Drupal\views\Entity\View $view */
     $entity_storage = $this->entityTypeManager->getStorage('view');
     $view = $entity_storage->load('test_view_entity_test_revision');
-    $this->assertEqual('entity_test_update_revision', $view->get('base_table'));
+    $this->assertEquals('entity_test_update_revision', $view->get('base_table'));
     $display = $view->getDisplay('default');
-    $this->assertEqual('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_revision_data', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_revision_data', $display['display_options']['fields']['name']['table']);
 
     $this->renameRevisionDataTable();
     $this->applyEntityUpdates('entity_test_update');
@@ -241,10 +245,10 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
     $view = $entity_storage->load('test_view_entity_test_revision');
 
     // Ensure the base table got renamed, so also the views fields.
-    $this->assertEqual('entity_test_update_revision', $view->get('base_table'));
+    $this->assertEquals('entity_test_update_revision', $view->get('base_table'));
     $display = $view->getDisplay('default');
-    $this->assertEqual('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_revision_data_new', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_revision_data_new', $display['display_options']['fields']['name']['table']);
 
     // Check that only the impacted views have been updated.
     $this->assertUpdatedViews([
@@ -256,7 +260,7 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * Tests that adding data tables adapts the views.
    */
-  public function testDataTableAddition() {
+  public function testDataTableAddition(): void {
     $this->updateEntityTypeToTranslatable(TRUE);
 
     /** @var \Drupal\views\Entity\View $view */
@@ -264,10 +268,10 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
     $view = $entity_storage->load('test_view_entity_test');
 
     // Ensure the data table got renamed, so also the views fields.
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
     $display = $view->getDisplay('default');
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_data', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_data', $display['display_options']['fields']['name']['table']);
 
     // Check that only the impacted views have been updated.
     $this->assertUpdatedViews([
@@ -278,7 +282,7 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * Tests that enabling revisions doesn't do anything.
    */
-  public function testRevisionEnabling() {
+  public function testRevisionEnabling(): void {
     $this->updateEntityTypeToRevisionable(TRUE);
 
     /** @var \Drupal\views\Entity\View $view */
@@ -286,10 +290,10 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
     $view = $entity_storage->load('test_view_entity_test');
 
     // Ensure that nothing happens.
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
     $display = $view->getDisplay('default');
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['name']['table']);
 
     // Check that only the impacted views have been updated.
     $this->assertUpdatedViews([]);
@@ -298,7 +302,7 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * Tests that removing revision support disables the view.
    */
-  public function testRevisionDisabling() {
+  public function testRevisionDisabling(): void {
     $this->updateEntityTypeToRevisionable(TRUE);
     $this->updateEntityTypeToNotRevisionable(TRUE);
 
@@ -317,7 +321,7 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * Tests a bunch possible entity definition table updates.
    */
-  public function testVariousTableUpdates() {
+  public function testVariousTableUpdates(): void {
     // We want to test the following permutations of entity definition updates:
     // base <-> base + translation
     // base + translation <-> base + translation + revision
@@ -325,104 +329,104 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
     // base <-> base + revision
     // base <-> base + translation + revision
 
-    // base <-> base + translation
+    // Base <-> base + translation
     $this->updateEntityTypeToTranslatable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_data', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_data', $display['display_options']['fields']['name']['table']);
 
     $this->updateEntityTypeToNotTranslatable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['name']['table']);
 
     $this->resetEntityType();
 
-    // base + translation <-> base + translation + revision
+    // Base + translation <-> base + translation + revision
     $this->updateEntityTypeToTranslatable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_data', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_data', $display['display_options']['fields']['name']['table']);
 
     $this->updateEntityTypeToRevisionable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_data', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_data', $display['display_options']['fields']['name']['table']);
 
     $this->updateEntityTypeToNotRevisionable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_data', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_data', $display['display_options']['fields']['name']['table']);
 
     $this->resetEntityType();
 
-    // base + revision <-> base + translation + revision
+    // Base + revision <-> base + translation + revision
     $this->updateEntityTypeToRevisionable();
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['name']['table']);
 
     $this->updateEntityTypeToTranslatable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_data', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_data', $display['display_options']['fields']['name']['table']);
 
     $this->updateEntityTypeToNotTranslatable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['name']['table']);
 
     $this->resetEntityType();
 
-    // base <-> base + revision
+    // Base <-> base + revision
     $this->updateEntityTypeToRevisionable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['name']['table']);
 
     $this->updateEntityTypeToNotRevisionable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['name']['table']);
 
     $this->resetEntityType();
 
-    // base <-> base + translation + revision
+    // Base <-> base + translation + revision
     $this->updateEntityTypeToRevisionable(TRUE);
     $this->updateEntityTypeToTranslatable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_data', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_data', $display['display_options']['fields']['name']['table']);
 
     $this->updateEntityTypeToNotRevisionable(TRUE);
     $this->updateEntityTypeToNotTranslatable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay();
+    [$view, $display] = $this->getUpdatedViewAndDisplay();
 
-    $this->assertEqual('entity_test_update', $view->get('base_table'));
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update', $view->get('base_table'));
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update', $display['display_options']['fields']['name']['table']);
 
     // Check that only the impacted views have been updated.
     $this->assertUpdatedViews([
@@ -435,29 +439,29 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * Tests some possible entity table updates for a revision view.
    */
-  public function testVariousTableUpdatesForRevisionView() {
-    // base + revision <-> base + translation + revision
+  public function testVariousTableUpdatesForRevisionView(): void {
+    // Base + revision <-> base + translation + revision
     $this->updateEntityTypeToRevisionable(TRUE);
 
-    list($view, $display) = $this->getUpdatedViewAndDisplay(TRUE);
+    [$view, $display] = $this->getUpdatedViewAndDisplay(TRUE);
 
-    $this->assertEqual('entity_test_update_revision', $view->get('base_table'));
-    $this->assertEqual('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_revision', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update_revision', $view->get('base_table'));
+    $this->assertEquals('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_revision', $display['display_options']['fields']['name']['table']);
 
     $this->updateEntityTypeToTranslatable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay(TRUE);
+    [$view, $display] = $this->getUpdatedViewAndDisplay(TRUE);
 
-    $this->assertEqual('entity_test_update_revision', $view->get('base_table'));
-    $this->assertEqual('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_revision_data', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update_revision', $view->get('base_table'));
+    $this->assertEquals('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_revision_data', $display['display_options']['fields']['name']['table']);
 
     $this->updateEntityTypeToNotTranslatable(TRUE);
-    list($view, $display) = $this->getUpdatedViewAndDisplay(TRUE);
+    [$view, $display] = $this->getUpdatedViewAndDisplay(TRUE);
 
-    $this->assertEqual('entity_test_update_revision', $view->get('base_table'));
-    $this->assertEqual('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
-    $this->assertEqual('entity_test_update_revision', $display['display_options']['fields']['name']['table']);
+    $this->assertEquals('entity_test_update_revision', $view->get('base_table'));
+    $this->assertEquals('entity_test_update_revision', $display['display_options']['fields']['id']['table']);
+    $this->assertEquals('entity_test_update_revision', $display['display_options']['fields']['name']['table']);
 
     // Check that only the impacted views have been updated.
     $this->assertUpdatedViews([
@@ -470,7 +474,7 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * Tests the case when a view could not be updated automatically.
    */
-  public function testViewSaveException() {
+  public function testViewSaveException(): void {
     $this->renameBaseTable();
     \Drupal::state()->set('entity_test_update.throw_view_exception', 'test_view_entity_test');
     $this->applyEntityUpdates('entity_test_update');
@@ -509,7 +513,7 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
   /**
    * Tests that broken views are handled gracefully.
    */
-  public function testBrokenView() {
+  public function testBrokenView(): void {
     $view_id = 'test_view_entity_test';
     $this->state->set('views_test_config.broken_view', $view_id);
     $this->updateEntityTypeToTranslatable(TRUE);
@@ -548,8 +552,10 @@ class ViewsEntitySchemaSubscriberIntegrationTest extends ViewsKernelTestBase {
    *
    * @param string[] $updated_view_ids
    *   An array of view IDs.
+   *
+   * @internal
    */
-  protected function assertUpdatedViews($updated_view_ids) {
+  protected function assertUpdatedViews(array $updated_view_ids): void {
     $all_view_ids = array_keys($this->entityTypeManager->getStorage('view')->loadMultiple());
 
     $view_save_count = \Drupal::state()->get('views_test_data.view_save_count', []);
