@@ -69,14 +69,6 @@ use Drupal\views\ViewExecutable;
  * @}
  */
 
-/**
- * @defgroup views_hooks Views hooks
- * @{
- * Hooks that allow other modules to implement the Views API.
- * @ingroup views_overview
- * @}
- */
-
 /**
  * @addtogroup hooks
  * @{
@@ -92,7 +84,7 @@ use Drupal\views\ViewExecutable;
  *   Array of warning messages built by Analyzer::formatMessage to be displayed
  *   to the user following analysis of the view.
  */
-function hook_views_analyze(Drupal\views\ViewExecutable $view) {
+function hook_views_analyze(\Drupal\views\ViewExecutable $view) {
   $messages = [];
 
   if ($view->display_handler->options['pager']['type'] == 'none') {
@@ -110,10 +102,10 @@ function hook_views_analyze(Drupal\views\ViewExecutable $view) {
  *
  * To provide views data for an entity, instead of implementing this hook,
  * create a class implementing \Drupal\views\EntityViewsDataInterface and
- * reference this in the "views" annotation in the entity class. The return
- * value of the getViewsData() method on the interface is the same as this hook,
- * and base class in \Drupal\views\EntityViewsData will take care of adding the
- * basic Views tables and fields for your entity. See the
+ * reference this in the "handlers.views_data" annotation in the entity class.
+ * The return value of the getViewsData() method on the interface is the same as
+ * this hook, and base class in \Drupal\views\EntityViewsData will take care of
+ * adding the basic Views tables and fields for your entity. See the
  * @link entity_api Entity API topic @endlink for more information about
  * entities.
  *
@@ -133,6 +125,7 @@ function hook_views_analyze(Drupal\views\ViewExecutable $view) {
 function hook_views_data() {
   // This example describes how to write hook_views_data() for a table defined
   // like this:
+  // @code
   // CREATE TABLE example_table (
   //   nid INT(11) NOT NULL         COMMENT 'Primary key: {node}.nid.',
   //   plain_text_field VARCHAR(32) COMMENT 'Just a plain text field.',
@@ -142,6 +135,7 @@ function hook_views_data() {
   //   langcode VARCHAR(12)         COMMENT 'Language code field.',
   //   PRIMARY KEY(nid)
   // );
+  // @endcode
 
   // Define the return array.
   $data = [];
@@ -193,9 +187,11 @@ function hook_views_data() {
   //
   // If you've decided an automatic join is a good idea, here's how to do it;
   // the resulting SQL query will look something like this:
+  // @code
   //   ... FROM example_table et ... JOIN node_field_data nfd
   //   ON et.nid = nfd.nid AND ('extra' clauses will be here) ...
-  // although the table aliases will be different.
+  // @endcode
+  // (The table aliases will be different.)
   $data['example_table']['table']['join'] = [
     // Within the 'join' section, list one or more tables to automatically
     // join to. In this example, every time 'node_field_data' is available in
@@ -243,11 +239,13 @@ function hook_views_data() {
   // shown above), you could join to 'node_field_table' via the 'foo' table.
   // Here's how to do this, and the resulting SQL query would look something
   // like this:
+  // @code
   //   ... FROM example_table et ... JOIN foo foo
   //   ON et.nid = foo.nid AND ('extra' clauses will be here) ...
   //   JOIN node_field_data nfd ON (definition of the join from the foo
   //   module goes here) ...
-  // although the table aliases will be different.
+  // @endcode
+  // Although the table aliases will be different.
   $data['example_table']['table']['join']['node_field_data'] = [
     // 'node_field_data' above is the base we're joining to in Views.
     // 'left_table' is the table we're actually joining to, in order to get to
@@ -422,6 +420,25 @@ function hook_views_data() {
     ],
   ];
 
+  // Computed field example. Computed fields are not associated with actual data
+  // tables and fields, and therefore have no schema. Instead, they are computed
+  // when the value is read from the entity. Here's the definition of a computed
+  // field that exists for a particular entity type. The value of the field will
+  // be calculated by a defined class. If the defined class for the computed
+  // fields differs between multiple bundles of the same entity type, then each
+  // of those fields should be added separately.
+  // @see \Drupal\Core\TypedData\DataDefinitionInterface::setComputed().
+  // @see \Drupal\Core\TypedData\DataDefinitionInterface::setClass().
+  $data['example_table']['computed_bundle_field'] = [
+    'title' => t('Computed Bundle Field'),
+    'help' => t('The computed bundle field'),
+    'field' => [
+      'id' => 'field',
+      'default_formatter' => 'string',
+      'field_name' => 'computed_bundle_field',
+    ],
+  ];
+
   // Area example. Areas are not generally associated with actual data
   // tables and fields. This example is from views_views_data(), which defines
   // the "Global" table (not really a table, but a group of Fields, Filters,
@@ -503,9 +520,11 @@ function hook_views_data_alter(array &$data) {
 /**
  * Override the default Views data for a Field API field.
  *
- * The field module's implementation of hook_views_data() invokes this for each
- * field storage, in the module that defines the field type. It is not invoked
- * in other modules.
+ * When collecting the views data, views_views_data() invokes this hook for each
+ * field storage definition, on the module that provides the field storage
+ * definition. If the return value is empty, the result of
+ * views_field_default_views_data() is used instead. Then the result is altered
+ * by invoking hook_field_views_data_alter() on all modules.
  *
  * If no hook implementation exists, hook_views_data() falls back to
  * views_field_default_views_data().
@@ -557,11 +576,11 @@ function hook_field_views_data(\Drupal\field\FieldStorageConfigInterface $field_
 function hook_field_views_data_alter(array &$data, \Drupal\field\FieldStorageConfigInterface $field_storage) {
   $entity_type_id = $field_storage->getTargetEntityTypeId();
   $field_name = $field_storage->getName();
-  $entity_type = \Drupal::entityManager()->getDefinition($entity_type_id);
+  $entity_type = \Drupal::entityTypeManager()->getDefinition($entity_type_id);
   $pseudo_field_name = 'reverse_' . $field_name . '_' . $entity_type_id;
-  $table_mapping = \Drupal::entityManager()->getStorage($entity_type_id)->getTableMapping();
+  $table_mapping = \Drupal::entityTypeManager()->getStorage($entity_type_id)->getTableMapping();
 
-  list($label) = views_entity_field_label($entity_type_id, $field_name);
+  [$label] = views_entity_field_label($entity_type_id, $field_name);
 
   $data['file_managed'][$pseudo_field_name]['relationship'] = [
     'title' => t('@entity using @field', ['@entity' => $entity_type->getLabel(), '@field' => $label]),
@@ -587,7 +606,7 @@ function hook_field_views_data_alter(array &$data, \Drupal\field\FieldStorageCon
 /**
  * Alter the Views data on a per field basis.
  *
- * The field module's implementation of hook_views_data_alter() invokes this for
+ * The Views module's implementation of hook_views_data_alter() invokes this for
  * each field storage, in the module that defines the field type. It is not
  * invoked in other modules.
  *
@@ -595,8 +614,8 @@ function hook_field_views_data_alter(array &$data, \Drupal\field\FieldStorageCon
  * data. This allows a field type to add data that concerns its fields in
  * other tables, which would not yet be defined at the point when
  * hook_field_views_data() and hook_field_views_data_alter() are invoked. For
- * example, entityreference adds reverse relationships on the tables for the
- * entities which are referenced by entityreference fields.
+ * example, entity_reference adds reverse relationships on the tables for the
+ * entities which are referenced by entity_reference fields.
  *
  * (Note: this is weirdly named so as not to conflict with
  * hook_field_views_data_alter().)
@@ -613,11 +632,11 @@ function hook_field_views_data_alter(array &$data, \Drupal\field\FieldStorageCon
 function hook_field_views_data_views_data_alter(array &$data, \Drupal\field\FieldStorageConfigInterface $field) {
   $field_name = $field->getName();
   $data_key = 'field_data_' . $field_name;
-  $entity_type_id = $field->entity_type;
-  $entity_type = \Drupal::entityManager()->getDefinition($entity_type_id);
+  $entity_type_id = $field->getTargetEntityTypeId();
+  $entity_type = \Drupal::entityTypeManager()->getDefinition($entity_type_id);
   $pseudo_field_name = 'reverse_' . $field_name . '_' . $entity_type_id;
-  list($label) = views_entity_field_label($entity_type_id, $field_name);
-  $table_mapping = \Drupal::entityManager()->getStorage($entity_type_id)->getTableMapping();
+  [$label] = views_entity_field_label($entity_type_id, $field_name);
+  $table_mapping = \Drupal::entityTypeManager()->getStorage($entity_type_id)->getTableMapping();
 
   // Views data for this field is in $data[$data_key].
   $data[$data_key][$pseudo_field_name]['relationship'] = [
@@ -661,7 +680,7 @@ function hook_views_query_substitutions(ViewExecutable $view) {
   // Example from views_views_query_substitutions().
   return [
     '***CURRENT_VERSION***' => \Drupal::VERSION,
-    '***CURRENT_TIME***' => REQUEST_TIME,
+    '***CURRENT_TIME***' => \Drupal::time()->getRequestTime(),
     '***LANGUAGE_language_content***' => \Drupal::languageManager()->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId(),
     PluginBase::VIEWS_QUERY_LANGUAGE_SITE_DEFAULT => \Drupal::languageManager()->getDefaultLanguage()->getId(),
   ];
@@ -822,36 +841,37 @@ function hook_views_pre_render(ViewExecutable $view) {
 }
 
 /**
- * Post-process any rendered data.
+ * Post-process any render data.
  *
- * This can be valuable to be able to cache a view and still have some level of
- * dynamic output. In an ideal world, the actual output will include HTML
- * comment-based tokens, and then the post process can replace those tokens.
- * This hook can be used by themes.
+ * The module or theme may add, modify or remove elements in $output after
+ * rendering.
  *
- * Example usage. If it is known that the view is a node view and that the
- * primary field will be a nid, you can do something like this:
+ * If a module wishes to act on the rendered HTML of the view rather than the
+ * structured content array, it may use this hook to add a #post_render
+ * callback:
  * @code
- *   <!--post-FIELD-NID-->
+ * // The object must implement \Drupal\Core\Security\TrustedCallbackInterface.
+ * $output['#post_render'][] = '\Drupal\my_module\View::postRender';
  * @endcode
- * And then in the post-render, create an array with the text that should
- * go there:
- * @code
- *   strtr($output, array('<!--post-FIELD-1-->' => 'output for FIELD of nid 1');
- * @endcode
- * All of the cached result data will be available in $view->result, as well,
- * so all ids used in the query should be discoverable.
+ *
+ * See \Drupal\Core\Render\RendererInterface::render() for #post_render
+ * documentation.
+ *
+ * Alternatively, it could also implement hook_preprocess_HOOK() for
+ * the particular view template, if there is one.
  *
  * @param \Drupal\views\ViewExecutable $view
- *   The view object about to be processed.
- * @param string $output
- *   A flat string with the rendered output of the view.
+ *   The view object being processed.
+ * @param array $output
+ *   A structured content array representing the view output. The given array
+ *   depends on the style plugin and can be either a render array or an array of
+ *   render arrays.
  * @param \Drupal\views\Plugin\views\cache\CachePluginBase $cache
  *   The cache settings.
  *
  * @see \Drupal\views\ViewExecutable
  */
-function hook_views_post_render(ViewExecutable $view, &$output, CachePluginBase $cache) {
+function hook_views_post_render(ViewExecutable $view, array &$output, CachePluginBase $cache) {
   // When using full pager, disable any time-based caching if there are fewer
   // than 10 results.
   if ($view->pager instanceof Drupal\views\Plugin\views\pager\Full && $cache instanceof Drupal\views\Plugin\views\cache\Time && count($view->result) < 10) {
@@ -865,7 +885,7 @@ function hook_views_post_render(ViewExecutable $view, &$output, CachePluginBase
  *
  * @param \Drupal\views\ViewExecutable $view
  *   The view object about to be processed.
- * @param QueryPluginBase $query
+ * @param \Drupal\views\Plugin\views\query\QueryPluginBase $query
  *   The query plugin object for the query.
  *
  * @see hook_views_query_substitutions()
@@ -879,7 +899,7 @@ function hook_views_query_alter(ViewExecutable $view, QueryPluginBase $query) {
     // Traverse through the 'where' part of the query.
     foreach ($query->where as &$condition_group) {
       foreach ($condition_group['conditions'] as &$condition) {
-        // If this is the part of the query filtering on title, chang the
+        // If this is the part of the query filtering on title, change the
         // condition to filter on node ID.
         if ($condition['field'] == 'node.title') {
           $condition = [
@@ -921,27 +941,6 @@ function hook_views_preview_info_alter(array &$rows, ViewExecutable $view) {
   ];
 }
 
-/**
- * Alter the links displayed at the top of the view edit form.
- *
- * @param array $links
- *   A renderable array of links which will be displayed at the top of the
- *   view edit form. Each entry will be in a form suitable for
- *   '#theme' => 'links'.
- * @param \Drupal\views\ViewExecutable $view
- *   The view object being edited.
- * @param string $display_id
- *   The ID of the display being edited, e.g. 'default' or 'page_1'.
- *
- * @see \Drupal\views_ui\ViewUI::renderDisplayTop()
- */
-function hook_views_ui_display_top_links_alter(array &$links, ViewExecutable $view, $display_id) {
-  // Put the export link first in the list.
-  if (isset($links['export'])) {
-    $links = ['export' => $links['export']] + $links;
-  }
-}
-
 // @todo Describe how to alter a view ajax response with event listeners.
 
 /**
@@ -1081,7 +1080,7 @@ function hook_views_plugins_exposed_form_alter(array &$plugins) {
  */
 function hook_views_plugins_join_alter(array &$plugins) {
   // Print out all join plugin names for debugging purposes.
-  debug($plugins);
+  dump($plugins);
 }
 
 /**
@@ -1113,7 +1112,7 @@ function hook_views_plugins_pager_alter(array &$plugins) {
  */
 function hook_views_plugins_query_alter(array &$plugins) {
   // Print out all query plugin names for debugging purposes.
-  debug($plugins);
+  dump($plugins);
 }
 
 /**
@@ -1264,7 +1263,3 @@ function hook_views_plugins_sort_alter(array &$plugins) {
 /**
  * @} End of "addtogroup hooks".
  */
-
-/**
- * @}
- */
