@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\views_ui\Functional;
 
 use Drupal\views\Entity\View;
@@ -8,6 +10,7 @@ use Drupal\views\Entity\View;
  * Tests exposed forms UI functionality.
  *
  * @group views_ui
+ * @group #slow
  */
 class ExposedFormUITest extends UITestBase {
 
@@ -44,8 +47,11 @@ class ExposedFormUITest extends UITestBase {
    */
   protected $groupFormUiErrors = [];
 
-  protected function setUp($import_test_views = TRUE): void {
-    parent::setUp($import_test_views);
+  /**
+   * {@inheritdoc}
+   */
+  protected function setUp($import_test_views = TRUE, $modules = ['views_test_config']): void {
+    parent::setUp($import_test_views, $modules);
 
     $this->drupalCreateContentType(['type' => 'article']);
     $this->drupalCreateContentType(['type' => 'page']);
@@ -64,7 +70,7 @@ class ExposedFormUITest extends UITestBase {
   /**
    * Tests the admin interface of exposed filter and sort items.
    */
-  public function testExposedAdminUi() {
+  public function testExposedAdminUi(): void {
     $edit = [];
 
     $this->drupalGet('admin/structure/views/nojs/handler/test_exposed_admin_ui/default/filter/type');
@@ -147,10 +153,13 @@ class ExposedFormUITest extends UITestBase {
     $this->submitForm($edit, 'Apply');
     $this->assertSession()->pageTextContains('Sort field identifier field is required.');
 
-    // Try with an invalid identifier.
+    // Try with an invalid identifiers.
     $edit['options[expose][field_identifier]'] = 'abc&! ###08.';
     $this->submitForm($edit, 'Apply');
     $this->assertSession()->pageTextContains('This identifier has illegal characters.');
+    $edit['options[expose][field_identifier]'] = '^abcde';
+    $this->submitForm($edit, 'Apply');
+    $this->assertSession()->pageTextContains('This identifier has illegal characters.');
 
     // Use a valid identifier.
     $edit['options[expose][field_identifier]'] = $this->randomMachineName() . '_-~.';
@@ -178,7 +187,7 @@ class ExposedFormUITest extends UITestBase {
   /**
    * Tests the admin interface of exposed grouped filters.
    */
-  public function testGroupedFilterAdminUi() {
+  public function testGroupedFilterAdminUi(): void {
     $edit = [];
 
     $this->drupalGet('admin/structure/views/nojs/handler/test_exposed_admin_ui/default/filter/type');
@@ -260,7 +269,7 @@ class ExposedFormUITest extends UITestBase {
     $this->assertNoGroupedFilterErrors();
   }
 
-  public function testGroupedFilterAdminUiErrors() {
+  public function testGroupedFilterAdminUiErrors(): void {
     // Select the empty operator without a title specified.
     $this->drupalGet('admin/structure/views/nojs/handler/test_exposed_admin_ui/default/filter/body_value');
     $edit = [];
@@ -308,7 +317,7 @@ class ExposedFormUITest extends UITestBase {
   /**
    * Tests the configuration of grouped exposed filters.
    */
-  public function testExposedGroupedFilter() {
+  public function testExposedGroupedFilter(): void {
     // Click the Expose filter button.
     $this->drupalGet('admin/structure/views/nojs/handler/test_exposed_admin_ui/default/filter/type');
     $this->submitForm([], 'Expose filter');
@@ -324,12 +333,27 @@ class ExposedFormUITest extends UITestBase {
       'options[group_info][group_items][2][value][article]' => 'article',
       'options[group_info][group_items][3][title]' => '3rd',
       'options[group_info][group_items][3][value][page]' => 'page',
+      'options[group_info][default_group]' => '3',
     ];
     // Apply the filter settings.
     $this->submitForm($edit, 'Apply');
     // Check that the view is saved without errors.
     $this->submitForm([], 'Save');
     $this->assertSession()->statusCodeEquals(200);
+    // Check the default filter value.
+    $this->drupalGet('test_exposed_admin_ui');
+    $this->assertSession()->fieldValueEquals('type', '3');
+    // Enable "Allow multiple selections" option and set a default group.
+    $this->drupalGet('admin/structure/views/nojs/handler/test_exposed_admin_ui/default/filter/type');
+    $edit['options[group_info][multiple]'] = 1;
+    $edit['options[group_info][default_group_multiple][1]'] = 1;
+    $this->submitForm($edit, 'Apply');
+    $this->submitForm([], 'Save');
+    // Check the default filter values again.
+    $this->drupalGet('test_exposed_admin_ui');
+    $this->assertSession()->checkboxChecked('type[1]');
+    $this->assertSession()->checkboxNotChecked('type[2]');
+    $this->assertSession()->checkboxNotChecked('type[3]');
 
     // Click the Expose filter button.
     $this->drupalGet('admin/structure/views/nojs/add-handler/test_exposed_admin_ui/default/filter');
@@ -348,18 +372,33 @@ class ExposedFormUITest extends UITestBase {
       'options[group_info][group_items][2][value]' => 1,
       'options[group_info][group_items][3][title]' => 'Unpublished',
       'options[group_info][group_items][3][value]' => 0,
+      'options[group_info][default_group]' => 2,
     ];
     // Apply the filter settings.
     $this->submitForm($edit, 'Apply');
     // Check that the view is saved without errors.
     $this->submitForm([], 'Save');
     $this->assertSession()->statusCodeEquals(200);
-
     $this->drupalGet('admin/structure/views/nojs/handler/test_exposed_admin_ui/default/filter/status');
     // Assert the same settings defined before still are there.
     $this->assertSession()->checkboxChecked('edit-options-group-info-group-items-1-value-all');
     $this->assertSession()->checkboxChecked('edit-options-group-info-group-items-2-value-1');
     $this->assertSession()->checkboxChecked('edit-options-group-info-group-items-3-value-0');
+
+    // Check the default filter value.
+    $this->drupalGet('test_exposed_admin_ui');
+    $this->assertSession()->fieldValueEquals('status', '2');
+    // Enable "Allow multiple selections" option and set a default group.
+    $this->drupalGet('admin/structure/views/nojs/handler/test_exposed_admin_ui/default/filter/status');
+    $edit['options[group_info][multiple]'] = 1;
+    $edit['options[group_info][default_group_multiple][3]'] = 1;
+    $this->submitForm($edit, 'Apply');
+    $this->submitForm([], 'Save');
+    // Check the default filter value again.
+    $this->drupalGet('test_exposed_admin_ui');
+    $this->assertSession()->checkboxNotChecked('status[1]');
+    $this->assertSession()->checkboxNotChecked('status[2]');
+    $this->assertSession()->checkboxChecked('status[3]');
   }
 
 }
