@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\FunctionalTests\Installer;
 
 /**
@@ -9,6 +11,11 @@ namespace Drupal\FunctionalTests\Installer;
  */
 class InstallerExistingConfigNoConfigTest extends InstallerExistingConfigTestBase {
 
+  /**
+   * {@inheritdoc}
+   */
+  protected $defaultTheme = 'stark';
+
   protected $profile = 'no_config_profile';
 
   /**
@@ -16,7 +23,6 @@ class InstallerExistingConfigNoConfigTest extends InstallerExistingConfigTestBas
    */
   protected function setUpSite() {
     // There are errors therefore there is nothing to do here.
-    return;
   }
 
   /**
@@ -29,14 +35,14 @@ class InstallerExistingConfigNoConfigTest extends InstallerExistingConfigTestBas
   /**
    * Tests that profiles with an empty config/sync directory do not work.
    */
-  public function testConfigSync() {
-    $this->assertTitle('Configuration validation | Drupal');
-    $this->assertText('The configuration synchronization failed validation.');
-    $this->assertText('This import is empty and if applied would delete all of your configuration, so has been rejected.');
+  public function testConfigSync(): void {
+    $this->assertSession()->titleEquals('Configuration validation | Drupal');
+    $this->assertSession()->pageTextContains('The configuration synchronization failed validation.');
+    $this->assertSession()->pageTextContains('This import is empty and if applied would delete all of your configuration, so has been rejected.');
 
     // Ensure there is no continuation button.
-    $this->assertNoText('Save and continue');
-    $this->assertNoFieldById('edit-submit');
+    $this->assertSession()->pageTextNotContains('Save and continue');
+    $this->assertSession()->buttonNotExists('edit-submit');
   }
 
 }
