@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\KernelTests\Core\Entity;
 
 use Drupal\entity_test\Entity\EntityTestMulChanged;
@@ -14,11 +16,17 @@ use Drupal\language\Entity\ConfigurableLanguage;
 class ContentEntityChangedTest extends EntityKernelTestBase {
 
   /**
-   * Modules to enable.
-   *
-   * @var array
+   * {@inheritdoc}
    */
-  public static $modules = ['language', 'user', 'system', 'field', 'text', 'filter', 'entity_test'];
+  protected static $modules = [
+    'language',
+    'user',
+    'system',
+    'field',
+    'text',
+    'filter',
+    'entity_test',
+  ];
 
   /**
    * The EntityTestMulChanged entity type storage.
@@ -37,7 +45,7 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
   /**
    * {@inheritdoc}
    */
-  protected function setUp() {
+  protected function setUp(): void {
     parent::setUp();
 
     // Enable an additional language.
@@ -47,14 +55,14 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
     $this->installEntitySchema('entity_test_mul_changed');
     $this->installEntitySchema('entity_test_mulrev_changed');
 
-    $this->mulChangedStorage = $this->entityManager->getStorage('entity_test_mul_changed');
-    $this->mulRevChangedStorage = $this->entityManager->getStorage('entity_test_mulrev_changed');
+    $this->mulChangedStorage = $this->entityTypeManager->getStorage('entity_test_mul_changed');
+    $this->mulRevChangedStorage = $this->entityTypeManager->getStorage('entity_test_mulrev_changed');
   }
 
   /**
    * Tests basic EntityChangedInterface functionality.
    */
-  public function testChanged() {
+  public function testChanged(): void {
     $user1 = $this->createUser();
     $user2 = $this->createUser();
 
@@ -67,10 +75,8 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
     ]);
     $entity->save();
 
-    $this->assertTrue(
-      $entity->getChangedTime() >= REQUEST_TIME,
-      'Changed time of original language is valid.'
-    );
+    $requestTime = \Drupal::time()->getRequestTime();
+    $this->assertGreaterThanOrEqual($requestTime, $entity->getChangedTime(), 'Changed time of original language is valid.');
 
     // We can't assert equality here because the created time is set to the
     // request time, while instances of ChangedTestItem use the current
@@ -78,14 +84,11 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
     // between the created time and now.
     $this->assertTrue(
       ($entity->getChangedTime() >= $entity->get('created')->value) &&
-      (($entity->getChangedTime() - $entity->get('created')->value) <= time() - REQUEST_TIME),
+      (($entity->getChangedTime() - $entity->get('created')->value) <= time() - $requestTime),
       'Changed and created time of original language can be assumed to be identical.'
     );
 
-    $this->assertEqual(
-      $entity->getChangedTime(), $entity->getChangedTimeAcrossTranslations(),
-      'Changed time of original language is the same as changed time across all translations.'
-    );
+    $this->assertEquals($entity->getChangedTimeAcrossTranslations(), $entity->getChangedTime(), 'Changed time of original language is the same as changed time across all translations.');
 
     $changed_en = $entity->getChangedTime();
 
@@ -94,34 +97,22 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
 
     $entity->save();
 
-    $this->assertEqual(
-      $entity->getChangedTime(), $changed_en,
-      'Changed time of original language did not change.'
-    );
+    $this->assertEquals($changed_en, $entity->getChangedTime(), 'Changed time of original language did not change.');
 
     $this->assertTrue(
       $german->getChangedTime() > $entity->getChangedTime(),
       'Changed time of the German translation is newer then the original language.'
     );
 
-    $this->assertEqual(
-      $german->getChangedTime(), $entity->getChangedTimeAcrossTranslations(),
-      'Changed time of the German translation is the newest time across all translations.'
-    );
+    $this->assertEquals($entity->getChangedTimeAcrossTranslations(), $german->getChangedTime(), 'Changed time of the German translation is the newest time across all translations.');
 
     $changed_de = $german->getChangedTime();
 
     $entity->save();
 
-    $this->assertEqual(
-      $entity->getChangedTime(), $changed_en,
-      'Changed time of original language did not change.'
-    );
+    $this->assertEquals($changed_en, $entity->getChangedTime(), 'Changed time of original language did not change.');
 
-    $this->assertEqual(
-      $german->getChangedTime(), $changed_de,
-      'Changed time of the German translation did not change.'
-    );
+    $this->assertEquals($changed_de, $german->getChangedTime(), 'Changed time of the German translation did not change.');
 
     // Update a non-translatable field to make sure that the changed timestamp
     // is updated for all translations.
@@ -151,131 +142,95 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
       'Changed time of original language did change.'
     );
 
-    $this->assertEqual(
-      $german->getChangedTime(), $changed_de,
-      'Changed time of the German translation did not change.'
-    );
+    $this->assertEquals($changed_de, $german->getChangedTime(), 'Changed time of the German translation did not change.');
 
     $this->assertTrue(
       $entity->getChangedTime() > $german->getChangedTime(),
       'Changed time of original language is newer then the German translation.'
     );
 
-    $this->assertEqual(
-      $entity->getChangedTime(), $entity->getChangedTimeAcrossTranslations(),
-      'Changed time of the original language is the newest time across all translations.'
-    );
+    $this->assertEquals($entity->getChangedTimeAcrossTranslations(), $entity->getChangedTime(), 'Changed time of the original language is the newest time across all translations.');
 
     $changed_en = $entity->getChangedTime();
 
     // Save entity without any changes.
     $entity->save();
 
-    $this->assertEqual(
-      $entity->getChangedTime(), $changed_en,
-      'Changed time of original language did not change.'
-    );
+    $this->assertEquals($changed_en, $entity->getChangedTime(), 'Changed time of original language did not change.');
 
-    $this->assertEqual(
-      $german->getChangedTime(), $changed_de,
-      'Changed time of the German translation did not change.'
-    );
+    $this->assertEquals($changed_de, $german->getChangedTime(), 'Changed time of the German translation did not change.');
 
     // At this point the changed time of the original language (en) is newer
     // than the changed time of the German translation. Now test that entity
     // queries work as expected.
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', $changed_en)->execute();
 
-    $this->assertEqual(
-      reset($ids), $entity->id(),
-      'Entity query can access changed time of original language.'
-    );
+    $this->assertEquals($entity->id(), reset($ids), 'Entity query can access changed time of original language.');
 
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', $changed_en, '=', 'en')->execute();
 
-    $this->assertEqual(
-      reset($ids), $entity->id(),
-      'Entity query can access changed time of original language by setting the original language as condition.'
-    );
+    $this->assertEquals($entity->id(), reset($ids), 'Entity query can access changed time of original language by setting the original language as condition.');
 
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', $changed_de, '=', 'en')->execute();
 
-    $this->assertFalse(
+    $this->assertEmpty(
       $ids,
       'There\'s no original entity stored having the changed time of the German translation.'
     );
 
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', $changed_en)->condition('default_langcode', '1')->execute();
 
-    $this->assertEqual(
-      reset($ids), $entity->id(),
-      'Entity query can access changed time of default language.'
-    );
+    $this->assertEquals($entity->id(), reset($ids), 'Entity query can access changed time of default language.');
 
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', $changed_de)->condition('default_langcode', '1')->execute();
 
-    $this->assertFalse(
+    $this->assertEmpty(
       $ids,
       'There\'s no entity stored using the default language having the changed time of the German translation.'
     );
 
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', $changed_de)->execute();
 
-    $this->assertEqual(
-      reset($ids), $entity->id(),
-      'Entity query can access changed time of the German translation.'
-    );
+    $this->assertEquals($entity->id(), reset($ids), 'Entity query can access changed time of the German translation.');
 
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', $changed_de, '=', 'de')->execute();
 
-    $this->assertEqual(
-      reset($ids), $entity->id(),
-      'Entity query can access changed time of the German translation.'
-    );
+    $this->assertEquals($entity->id(), reset($ids), 'Entity query can access changed time of the German translation.');
 
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', $changed_en, '=', 'de')->execute();
 
-    $this->assertFalse(
+    $this->assertEmpty(
       $ids,
       'There\'s no German translation stored having the changed time of the original language.'
     );
 
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', $changed_de, '>')->execute();
 
-    $this->assertEqual(
-      reset($ids), $entity->id(),
-      'Entity query can access changed time regardless of translation.'
-    );
+    $this->assertEquals($entity->id(), reset($ids), 'Entity query can access changed time regardless of translation.');
 
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', $changed_en, '<')->execute();
 
-    $this->assertEqual(
-      reset($ids), $entity->id(),
-      'Entity query can access changed time regardless of translation.'
-    );
+    $this->assertEquals($entity->id(), reset($ids), 'Entity query can access changed time regardless of translation.');
 
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', 0, '>')->execute();
 
-    $this->assertEqual(
-      reset($ids), $entity->id(),
-      'Entity query can access changed time regardless of translation.'
-    );
+    $this->assertEquals($entity->id(), reset($ids), 'Entity query can access changed time regardless of translation.');
 
-    $query = $this->mulChangedStorage->getQuery();
+    $query = $this->mulChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('changed', $changed_en, '>')->execute();
 
-    $this->assertFalse(
+    $this->assertEmpty(
       $ids,
       'Entity query can access changed time regardless of translation.'
     );
@@ -284,7 +239,7 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
   /**
    * Tests revisionable EntityChangedInterface functionality.
    */
-  public function testRevisionChanged() {
+  public function testRevisionChanged(): void {
     $user1 = $this->createUser();
     $user2 = $this->createUser();
 
@@ -297,7 +252,7 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
     $entity->save();
 
     $this->assertTrue(
-      $entity->getChangedTime() >= REQUEST_TIME,
+      $entity->getChangedTime() >= \Drupal::time()->getRequestTime(),
       'Changed time of original language is valid.'
     );
 
@@ -306,14 +261,11 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
     // timestamp every time.
     $this->assertTrue(
       ($entity->getChangedTime() >= $entity->get('created')->value) &&
-      (($entity->getChangedTime() - $entity->get('created')->value) <= time() - REQUEST_TIME),
+      (($entity->getChangedTime() - $entity->get('created')->value) <= time() - \Drupal::time()->getRequestTime()),
       'Changed and created time of original language can be assumed to be identical.'
     );
 
-    $this->assertEqual(
-      $entity->getChangedTime(), $entity->getChangedTimeAcrossTranslations(),
-      'Changed time of original language is the same as changed time across all translations.'
-    );
+    $this->assertEquals($entity->getChangedTimeAcrossTranslations(), $entity->getChangedTime(), 'Changed time of original language is the same as changed time across all translations.');
 
     $this->assertTrue(
       $this->getRevisionTranslationAffectedFlag($entity),
@@ -326,10 +278,7 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
     // Save entity without any changes but create new revision.
     $entity->save();
     // A new revision without any changes should not set a new changed time.
-    $this->assertEqual(
-      $entity->getChangedTime(), $changed_en,
-      'Changed time of original language did not change.'
-    );
+    $this->assertEquals($changed_en, $entity->getChangedTime(), 'Changed time of original language did not change.');
 
     $this->assertFalse(
       $this->getRevisionTranslationAffectedFlag($entity),
@@ -357,20 +306,14 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
 
     $entity->save();
 
-    $this->assertEqual(
-      $entity->getChangedTime(), $changed_en,
-      'Changed time of original language did not change.'
-    );
+    $this->assertEquals($changed_en, $entity->getChangedTime(), 'Changed time of original language did not change.');
 
     $this->assertTrue(
       $german->getChangedTime() > $entity->getChangedTime(),
       'Changed time of the German translation is newer then the original language.'
     );
 
-    $this->assertEqual(
-      $german->getChangedTime(), $entity->getChangedTimeAcrossTranslations(),
-      'Changed time of the German translation is the newest time across all translations.'
-    );
+    $this->assertEquals($entity->getChangedTimeAcrossTranslations(), $german->getChangedTime(), 'Changed time of the German translation is the newest time across all translations.');
 
     $this->assertTrue(
       $this->getRevisionTranslationAffectedFlag($entity),
@@ -388,15 +331,9 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
     // Save entity without any changes but create new revision.
     $entity->save();
 
-    $this->assertEqual(
-      $entity->getChangedTime(), $changed_en,
-      'Changed time of original language did not change.'
-    );
+    $this->assertEquals($changed_en, $entity->getChangedTime(), 'Changed time of original language did not change.');
 
-    $this->assertEqual(
-      $german->getChangedTime(), $changed_de,
-      'Changed time of the German translation did not change.'
-    );
+    $this->assertEquals($changed_de, $german->getChangedTime(), 'Changed time of the German translation did not change.');
 
     $this->assertFalse(
       $this->getRevisionTranslationAffectedFlag($entity),
@@ -412,20 +349,14 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
     $german->setOwner($user2);
     $entity->save();
 
-    $this->assertEqual(
-      $entity->getChangedTime(), $changed_en,
-      'Changed time of original language did not change.'
-    );
+    $this->assertEquals($changed_en, $entity->getChangedTime(), 'Changed time of original language did not change.');
 
     $this->assertTrue(
       $german->getChangedTime() > $changed_de,
       'Changed time of the German translation did change.'
     );
 
-    $this->assertEqual(
-      $german->getChangedTime(), $entity->getChangedTimeAcrossTranslations(),
-      'Changed time of the German translation is the newest time across all translations.'
-    );
+    $this->assertEquals($entity->getChangedTimeAcrossTranslations(), $german->getChangedTime(), 'Changed time of the German translation is the newest time across all translations.');
 
     $this->assertFalse(
       $this->getRevisionTranslationAffectedFlag($entity),
@@ -442,10 +373,7 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
     $entity->setNewRevision();
     $entity->save();
 
-    $this->assertEqual(
-      $entity->getChangedTime(), $changed_en,
-      'Changed time of original language did not change.'
-    );
+    $this->assertEquals($changed_en, $entity->getChangedTime(), 'Changed time of original language did not change.');
 
     $this->assertTrue(
       $french->getChangedTime() > $entity->getChangedTime(),
@@ -457,10 +385,7 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
       'Changed time of the French translation is newer then the German translation.'
     );
 
-    $this->assertEqual(
-      $french->getChangedTime(), $entity->getChangedTimeAcrossTranslations(),
-      'Changed time of the French translation is the newest time across all translations.'
-    );
+    $this->assertEquals($entity->getChangedTimeAcrossTranslations(), $french->getChangedTime(), 'Changed time of the French translation is the newest time across all translations.');
 
     $this->assertFalse(
       $this->getRevisionTranslationAffectedFlag($entity),
@@ -555,7 +480,7 @@ class ContentEntityChangedTest extends EntityKernelTestBase {
    *   The flag value.
    */
   protected function getRevisionTranslationAffectedFlag(EntityTestMulRevChanged $entity) {
-    $query = $this->mulRevChangedStorage->getQuery();
+    $query = $this->mulRevChangedStorage->getQuery()->accessCheck(FALSE);
     $ids = $query->condition('revision_translation_affected', 1, '=', $entity->language()->getId())->execute();
     $id = reset($ids);
     return (bool) ($id == $entity->id());
