@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\KernelTests\Core\Entity;
 
 use Drupal\Core\Entity\EntityStorageException;
@@ -14,6 +16,7 @@ use Drupal\Tests\system\Functional\Entity\Traits\EntityDefinitionTestTrait;
  * @coversDefaultClass \Drupal\Core\Entity\EntityDefinitionUpdateManager
  *
  * @group Entity
+ * @group #slow
  */
 class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
 
@@ -76,16 +79,18 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
   protected $testEntities = [];
 
   /**
-   * Modules to enable.
-   *
-   * @var array
+   * {@inheritdoc}
    */
-  public static $modules = ['content_translation', 'entity_test_update', 'language'];
+  protected static $modules = [
+    'content_translation',
+    'entity_test_update',
+    'language',
+  ];
 
   /**
    * {@inheritdoc}
    */
-  protected function setUp() {
+  protected function setUp(): void {
     parent::setUp();
     $this->entityDefinitionUpdateManager = $this->container->get('entity.definition_update_manager');
     $this->lastInstalledSchemaRepository = $this->container->get('entity.last_installed_schema.repository');
@@ -104,13 +109,12 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
     // provide test coverage for this special case.
     $fields['changed'] = BaseFieldDefinition::create('changed')
       ->setLabel(t('Changed'))
-      ->setDescription(t('The time that the custom block was last edited.'))
+      ->setDescription(t('The time that the content block was last edited.'))
       ->setTranslatable(TRUE)
       ->setRevisionable(TRUE);
     $this->state->set('entity_test_update.additional_base_field_definitions', $fields);
 
     $this->installEntitySchema($this->entityTypeId);
-    $this->installEntitySchema('configurable_language');
 
     // Enable an additional language.
     ConfigurableLanguage::createFromLangcode('ro')->save();
@@ -121,162 +125,6 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
     new Settings($settings);
   }
 
-  /**
-   * @covers ::updateFieldableEntityType
-   * @dataProvider providerTestFieldableEntityTypeUpdates
-   */
-  public function testFieldableEntityTypeUpdates($initial_rev, $initial_mul, $new_rev, $new_mul, $data_migration_supported) {
-    // The 'entity_test_update' entity type is neither revisionable nor
-    // translatable by default, so we need to get it into the initial testing
-    // state. This also covers the "no existing data" scenario for fieldable
-    // entity type updates.
-    if ($initial_rev || $initial_mul) {
-      $entity_type = $this->getUpdatedEntityTypeDefinition($initial_rev, $initial_mul);
-      $field_storage_definitions = $this->getUpdatedFieldStorageDefinitions($initial_rev, $initial_mul);
-
-      $this->entityDefinitionUpdateManager->updateFieldableEntityType($entity_type, $field_storage_definitions);
-      $this->assertEntityTypeSchema($initial_rev, $initial_mul);
-    }
-
-    // Add a few entities so we can test the data copying step.
-    $this->insertData($initial_rev, $initial_mul);
-
-    $updated_entity_type = $this->getUpdatedEntityTypeDefinition($new_rev, $new_mul);
-    $updated_field_storage_definitions = $this->getUpdatedFieldStorageDefinitions($new_rev, $new_mul);
-
-    if (!$data_migration_supported) {
-      $this->setExpectedException(EntityStorageException::class, 'Converting an entity type from revisionable to non-revisionable or from translatable to non-translatable is not supported.');
-    }
-
-    // Check that existing data can be retrieved from the storage before the
-    // entity schema is updated.
-    if ($data_migration_supported) {
-      $this->assertEntityData($initial_rev, $initial_mul);
-    }
-
-    // Enable the creation of a new base field during a fieldable entity type
-    // update.
-    $this->state->set('entity_test_update.install_new_base_field_during_update', TRUE);
-
-    // Simulate a batch run since we are converting the entities one by one.
-    $sandbox = [];
-    do {
-      $this->entityDefinitionUpdateManager->updateFieldableEntityType($updated_entity_type, $updated_field_storage_definitions, $sandbox);
-    } while ($sandbox['#finished'] != 1);
-
-    $this->assertEntityTypeSchema($new_rev, $new_mul, TRUE);
-    $this->assertEntityData($initial_rev, $initial_mul);
-
-    $change_list = $this->entityDefinitionUpdateManager->getChangeList();
-    $this->assertArrayNotHasKey('entity_test_update', $change_list, "There are no remaining updates for the 'entity_test_update' entity type.");
-
-    // Check that we can still save new entities after the schema has been
-    // updated.
-    $this->insertData($new_rev, $new_mul);
-
-    // Check that the backup tables have been kept in place.
-    $this->assertBackupTables();
-  }
-
-  /**
-   * Data provider for testFieldableEntityTypeUpdates().
-   */
-  public function providerTestFieldableEntityTypeUpdates() {
-    return [
-      'no change' => [
-        'initial_rev' => FALSE,
-        'initial_mul' => FALSE,
-        'new_rev' => FALSE,
-        'new_mul' => FALSE,
-        'data_migration_supported' => TRUE,
-      ],
-      'non_rev non_mul to rev non_mul' => [
-        'initial_rev' => FALSE,
-        'initial_mul' => FALSE,
-        'new_rev' => TRUE,
-        'new_mul' => FALSE,
-        'data_migration_supported' => TRUE,
-      ],
-      'non_rev non_mul to rev mul' => [
-        'initial_rev' => FALSE,
-        'initial_mul' => FALSE,
-        'new_rev' => TRUE,
-        'new_mul' => TRUE,
-        'data_migration_supported' => TRUE,
-      ],
-      'non_rev non_mul to non_rev mul' => [
-        'initial_rev' => FALSE,
-        'initial_mul' => FALSE,
-        'new_rev' => FALSE,
-        'new_mul' => TRUE,
-        'data_migration_supported' => TRUE,
-      ],
-      'rev non_mul to non_rev non_mul' => [
-        'initial_rev' => TRUE,
-        'initial_mul' => FALSE,
-        'new_rev' => FALSE,
-        'new_mul' => FALSE,
-        'data_migration_supported' => FALSE,
-      ],
-      'rev non_mul to non_rev mul' => [
-        'initial_rev' => TRUE,
-        'initial_mul' => FALSE,
-        'new_rev' => FALSE,
-        'new_mul' => TRUE,
-        'data_migration_supported' => FALSE,
-      ],
-      'rev non_mul to rev mul' => [
-        'initial_rev' => TRUE,
-        'initial_mul' => FALSE,
-        'new_rev' => TRUE,
-        'new_mul' => TRUE,
-        'data_migration_supported' => TRUE,
-      ],
-      'non_rev mul to non_rev non_mul' => [
-        'initial_rev' => FALSE,
-        'initial_mul' => TRUE,
-        'new_rev' => FALSE,
-        'new_mul' => FALSE,
-        'data_migration_supported' => FALSE,
-      ],
-      'non_rev mul to rev non_mul' => [
-        'initial_rev' => FALSE,
-        'initial_mul' => TRUE,
-        'new_rev' => TRUE,
-        'new_mul' => FALSE,
-        'data_migration_supported' => FALSE,
-      ],
-      'non_rev mul to rev mul' => [
-        'initial_rev' => FALSE,
-        'initial_mul' => TRUE,
-        'new_rev' => TRUE,
-        'new_mul' => TRUE,
-        'data_migration_supported' => TRUE,
-      ],
-      'rev mul to non_rev non_mul' => [
-        'initial_rev' => TRUE,
-        'initial_mul' => TRUE,
-        'new_rev' => FALSE,
-        'new_mul' => FALSE,
-        'data_migration_supported' => FALSE,
-      ],
-      'rev mul to rev non_mul' => [
-        'initial_rev' => TRUE,
-        'initial_mul' => TRUE,
-        'new_rev' => TRUE,
-        'new_mul' => FALSE,
-        'data_migration_supported' => FALSE,
-      ],
-      'rev mul to non_rev mul' => [
-        'initial_rev' => TRUE,
-        'initial_mul' => TRUE,
-        'new_rev' => FALSE,
-        'new_mul' => TRUE,
-        'data_migration_supported' => FALSE,
-      ],
-    ];
-  }
-
   /**
    * Generates test entities for the 'entity_test_update' entity type.
    *
@@ -290,7 +138,7 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
     // least three times.
     /** @var \Drupal\Core\Entity\TranslatableRevisionableStorageInterface|\Drupal\Core\Entity\EntityStorageInterface $storage */
     $storage = $this->entityTypeManager->getStorage($this->entityTypeId);
-    $next_id = $storage->getQuery()->count()->execute() + 1;
+    $next_id = $storage->getQuery()->accessCheck(FALSE)->count()->execute() + 1;
 
     // Create test entities with two translations and two revisions.
     /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
@@ -376,8 +224,10 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
    *   Whether the entity type was revisionable prior to the update.
    * @param bool $translatable
    *   Whether the entity type was translatable prior to the update.
+   *
+   * @internal
    */
-  protected function assertEntityData($revisionable, $translatable) {
+  protected function assertEntityData(bool $revisionable, bool $translatable): void {
     $entities = $this->entityTypeManager->getStorage($this->entityTypeId)->loadMultiple();
     $this->assertCount(3, $entities);
     foreach ($entities as $entity_id => $entity) {
@@ -405,7 +255,12 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
     }
 
     if ($revisionable) {
-      $revisions_result = $this->entityTypeManager->getStorage($this->entityTypeId)->getQuery()->allRevisions()->execute();
+      $revisions_result = $this->entityTypeManager
+        ->getStorage($this->entityTypeId)
+        ->getQuery()
+        ->accessCheck(FALSE)
+        ->allRevisions()
+        ->execute();
       $revisions = $this->entityTypeManager->getStorage($this->entityTypeId)->loadMultipleRevisions(array_keys($revisions_result));
       $this->assertCount(6, $revisions);
 
@@ -446,8 +301,10 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
    * @param bool $new_base_field
    *   (optional) Whether a new base field was added as part of the update.
    *   Defaults to FALSE.
+   *
+   * @internal
    */
-  protected function assertEntityTypeSchema($revisionable, $translatable, $new_base_field = FALSE) {
+  protected function assertEntityTypeSchema(bool $revisionable, bool $translatable, bool $new_base_field = FALSE): void {
     // Check whether the 'new_base_field' field has been installed correctly.
     $field_storage_definition = $this->entityDefinitionUpdateManager->getFieldStorageDefinition('new_base_field', $this->entityTypeId);
     if ($new_base_field) {
@@ -475,8 +332,10 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
 
   /**
    * Asserts the revisionable characteristics of an entity type.
+   *
+   * @internal
    */
-  protected function assertRevisionable() {
+  protected function assertRevisionable(): void {
     /** @var \Drupal\Core\Entity\ContentEntityTypeInterface $entity_type */
     $entity_type = $this->entityDefinitionUpdateManager->getEntityType($this->entityTypeId);
     $this->assertTrue($entity_type->isRevisionable());
@@ -514,8 +373,10 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
 
   /**
    * Asserts the translatable characteristics of an entity type.
+   *
+   * @internal
    */
-  protected function assertTranslatable() {
+  protected function assertTranslatable(): void {
     /** @var \Drupal\Core\Entity\ContentEntityTypeInterface $entity_type */
     $entity_type = $this->entityDefinitionUpdateManager->getEntityType($this->entityTypeId);
     $this->assertTrue($entity_type->isTranslatable());
@@ -543,8 +404,10 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
 
   /**
    * Asserts the revisionable / translatable characteristics of an entity type.
+   *
+   * @internal
    */
-  protected function assertRevisionableAndTranslatable() {
+  protected function assertRevisionableAndTranslatable(): void {
     $this->assertRevisionable();
     $this->assertTranslatable();
 
@@ -589,8 +452,10 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
 
   /**
    * Asserts that an entity type is neither revisionable nor translatable.
+   *
+   * @internal
    */
-  protected function assertNonRevisionableAndNonTranslatable() {
+  protected function assertNonRevisionableAndNonTranslatable(): void {
     /** @var \Drupal\Core\Entity\ContentEntityTypeInterface $entity_type */
     $entity_type = $this->entityDefinitionUpdateManager->getEntityType($this->entityTypeId);
     $this->assertFalse($entity_type->isRevisionable());
@@ -608,8 +473,10 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
    *
    * @param bool $revisionable
    *   Whether the entity type is revisionable or not.
+   *
+   * @internal
    */
-  protected function assertBundleFieldSchema($revisionable) {
+  protected function assertBundleFieldSchema(bool $revisionable): void {
     $entity_type_id = 'entity_test_update';
     $field_storage_definition = $this->entityFieldManager->getFieldStorageDefinitions($entity_type_id)['new_bundle_field'];
     $database_schema = $this->database->schema();
@@ -625,8 +492,10 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
 
   /**
    * Asserts that the backup tables have been kept after a successful update.
+   *
+   * @internal
    */
-  protected function assertBackupTables() {
+  protected function assertBackupTables(): void {
     $backups = \Drupal::keyValue('entity.update_backup')->getAll();
     $backup = reset($backups);
 
@@ -639,7 +508,7 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
   /**
    * Tests that a failed entity schema update preserves the existing data.
    */
-  public function testFieldableEntityTypeUpdatesErrorHandling() {
+  public function testFieldableEntityTypeUpdatesErrorHandling(): void {
     $schema = $this->database->schema();
 
     // First, convert the entity type to be translatable for better coverage and
@@ -671,7 +540,8 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
 
     // Make the update throw an exception during the entity save process.
     \Drupal::state()->set('entity_test_update.throw_exception', TRUE);
-    $this->setExpectedException(EntityStorageException::class, 'The entity update process failed while processing the entity type entity_test_update, ID: 1.');
+    $this->expectException(EntityStorageException::class);
+    $this->expectExceptionMessage('The entity update process failed while processing the entity type entity_test_update, ID: 1.');
 
     try {
       $updated_entity_type = $this->getUpdatedEntityTypeDefinition(TRUE, TRUE);
@@ -806,7 +676,7 @@ class FieldableEntityDefinitionUpdateTest extends EntityKernelTestBase {
   /**
    * Tests the removal of the backup tables after a successful update.
    */
-  public function testFieldableEntityTypeUpdatesRemoveBackupTables() {
+  public function testFieldableEntityTypeUpdatesRemoveBackupTables(): void {
     $schema = $this->database->schema();
 
     // Convert the entity type to be revisionable.
