@@ -1,8 +1,9 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\KernelTests\Core\Extension;
 
-use Drupal\Core\Site\Settings;
 use Drupal\KernelTests\KernelTestBase;
 
 /**
@@ -14,11 +15,7 @@ class ThemeEngineExtensionListTest extends KernelTestBase {
   /**
    * @covers ::getList
    */
-  public function testGetlist() {
-    $settings = Settings::getAll();
-    $settings['install_profile'] = 'testing';
-    new Settings($settings);
-
+  public function testGetList(): void {
     // Confirm that all theme engines are available.
     $theme_engines = \Drupal::service('extension.list.theme_engine')->getList();
     $this->assertArrayHasKey('twig', $theme_engines);
